<?php

/**
 * webEdition CMS
 *
 * $Rev: f6962871c2fa $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-27 15:40 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*
  data of variations have the following format in document

  we_doc->elements[WE_VARIANTS_ELEMENT_NAME] =
  [0] =>
  'VARIATIONNAME1' =>
  'fieldName1' =>
  'type' = 'txt',
  'dat' = 'Text'
  ),
  'fieldName2' =>
  'type' = 'img',
  'dat' = 152
  )
  ),
  [1] =>
  'VARIATIONNAME2' =>
  'fieldName1' =>
  'type' = 'txt',
  'dat' = 'CU'
  ),
  'fieldName2' =>
  'type' = 'img',
  'dat' = 155
  )
  )
  )
  )
  =====>>

  in editmode available in document
  we_doc->elements[WE_VARIANTS_PREFIX . '0'] ='type' = 'txt', 'dat' = 'VARIATIONNAME1');
  we_doc->elements[WE_VARIANTS_PREFIX . '0' . '_' . fieldName1] ='type' = 'txt', 'dat' = 'Text');
  we_doc->elements[WE_VARIANTS_PREFIX . '0' . '_' . fieldName2] ='type' = 'img', 'dat' = 152);

  we_doc->elements[WE_VARIANTS_PREFIX . '1'] ='type' = 'txt', 'dat' = 'VARIATIONNAME2');
  we_doc->elements[WE_VARIANTS_PREFIX . '1' . '_' . fieldName1] ='type' = 'txt', 'dat' = 'CU');
  ...
 */

abstract class we_base_variants{
	const PREFIX = 'we__intern_variant___';
	const ELEMENT_NAME = 'weInternVariantElement';
	const REQUEST = 'we_variant';
	const VARIANT_TAGS = ['input', 'link', 'textarea', 'img', 'select'];
	const ALLVARIANT_FIELD = 'allVariants';

	/**
	 * Searchs all elements of document/object
	 * fetches all variation-data in one single field
	 * and deletes all other fields
	 * when not save, the field is resettet for the editor
	 *
	 * @param object $model
	 * @param boolean $save
	 */
	public static function correctModelFields(we_contents_document $model, $save = true){

		$elements = $model->elements;

		// all variant fields must be stored in one single field of the content table
		// store variationfields in one array
		$variationElements = [];

		foreach($elements as $element => $elemArr){
			if(strpos($element, self::PREFIX) !== false){
				//since delete might have deleted this instance, check if this id is still set
				list($pos) = explode('_', substr($element, strlen(self::PREFIX), 2));
				if(isset($model->elements[self::ELEMENT_NAME]['dat'][$pos])){
					//only add elements which are not deleted
					$variationElements[$element] = $elemArr;
				}
				if($save){
					$model->elements[$element] = null;
					unset($model->elements[$element]);
				}
			}
		}
		// :ATTENTION: if nr of variants is > 10 a ksort of the elements is not
		// enough to build blocks of data of a single variant.
		ksort($variationElements);

		$variationElement = [];
		$nameOfPosition = [];

		// :ATTENTION: if nr of variants is > 10 a ksort of the elements is not
		// enough to build blocks of data of a single variant.
		foreach($variationElements as $element => $data){

			$elemNr = self::getNrFromElemName($element);

			if(!isset($nameOfPosition["nameof_$elemNr"])){
				$nameOfPosition["nameof_$elemNr"] = $data['dat'];
				$variationElement[$elemNr][$nameOfPosition["nameof_$elemNr"]] = [];
			} else {
				$fieldName = self::getFieldNameFromElemName($element);
				$variationElement[$elemNr][$nameOfPosition["nameof_$elemNr"]][$fieldName] = $data;
			}
		}

		// now create element for the model
		// just overwrite new values ...
		$model->elements[self::ELEMENT_NAME]['type'] = 'variant';
		$model->elements[self::ELEMENT_NAME]['dat'] = ($save ? we_serialize($variationElement, SERIALIZE_JSON) : $variationElement);
	}

	/**
	 * this function is reverse function to correctModelFields
	 * initialises variant data in the model and stores them in special fields
	 * @param object $model
	 * @param boolean $unserialize
	 */
	public static function setVariantDataForModel(we_contents_document $model, $unserialize = false){

		// set variation data from array and
		if(!$model->issetElement(self::ELEMENT_NAME)){
			return;
		}

		if($unserialize){
			$model->elements[self::ELEMENT_NAME]['dat'] = is_array($model->elements[self::ELEMENT_NAME]['dat']) ?
				$model->elements[self::ELEMENT_NAME]['dat'] :
				we_unserialize($model->elements[self::ELEMENT_NAME]['dat']);
		}

		$variations = $model->getElement(self::ELEMENT_NAME, we_contents_base::ETYPE_TEXT, '', true); //Fix #9349
		if(!$variations || !is_array($variations)){
			return;
		}
		foreach($variations as $i => $variation){
			if(!is_array($variation)){
				continue;
			}

			foreach($variation as $name => $varArr){
				$model->elements[self::PREFIX . $i] = ['type' => 'txt',
					'dat' => $name
				];

				foreach($varArr as $n => $datArr){
					$model->elements[self::PREFIX . $i . '_' . $n] = $datArr;
				}
			}
		}
	}

	private static function getNrFromElemName($elemName){
		return preg_replace('/_(.*)/', '', substr($elemName, strlen(self::PREFIX)));
	}

	private static function getFieldNameFromElemName($elemName){
		$fieldNameTmp = substr($elemName, strlen(self::PREFIX));
		$fieldName = preg_replace('/(\d+_*)/', '', $fieldNameTmp, 1);

		return ($fieldNameTmp == $fieldName ? '' : $fieldName);
	}

	public static function getNumberOfVariants(we_contents_document $model){
		return ($model->issetElement(self::ELEMENT_NAME) && is_array(($el = $model->getElement(self::ELEMENT_NAME, we_contents_base::ETYPE_TEXT))) ?
			count($el) :
			0);
	}

	private static function insertVariant(we_contents_document $model, $position){
		$amount = we_base_variants::getNumberOfVariants($model);

		// init model->elements if neccessary

		$el = $model->getElement(self::ELEMENT_NAME, we_contents_base::ETYPE_TEXT, []) ?: [];

		// add new element at end of array, move it when neccesary
		$el[] = self::createNewVariantElement($model);
		$model->setElement(self::ELEMENT_NAME, $el, 'variant');

		// now move element, it is actually at last position
		if($amount > $position){ // move all elements
			$newElemPos = $amount;
			while($position < $newElemPos){
				self::changeVariantPosition($newElemPos, --$newElemPos, $model);
			}
		}
	}

	private static function createNewVariantElement(we_contents_document $model){

		// :TODO: improve me
		return [];
	}

	public static function getAllVariationFields(we_contents_document $model, $pos = false){
		$elements = $model->elements;
		$variationElements = [];

		foreach($elements as $element => $elemArr){
			if(strpos($element, self::PREFIX) !== false){
				$variationElements[$element] = $elemArr;
			}
		}
		ksort($variationElements);

		if($pos === false){
			return $variationElements;
		}
		foreach(array_keys($variationElements) as $name){
			if(self::getNrFromElemName($name) != $pos){
				unset($variationElements[$name]);
			}
		}
		return $variationElements;
	}

	private static function moveVariant(we_contents_document $model, $pos, $direction){
		// check if a move is possible
		self::changeVariantPosition($pos, ($pos + ($direction === 'up' ? -1 : 1)), $model);
	}

	/**
	 * @param integer $pos1
	 * @param integer $pos2
	 * @param array $model
	 */
	private static function changeVariantPosition($pos1, $pos2, we_contents_document $model){
		// first move all fields in the $modell
		$elements = $model->getElement(self::ELEMENT_NAME, 'dat', []);

		$tmp = $elements[$pos1];
		$elements[$pos1] = $elements[$pos2];
		$elements[$pos2] = $tmp;
		$model->setElement(self::ELEMENT_NAME, $elements, 'variant');
		// move elements for editmode
		$variationElements_1 = we_base_variants::getAllVariationFields($model, $pos1);
		$variationElements_2 = we_base_variants::getAllVariationFields($model, $pos2);

		// backup pos 1
		$tmp = [];
		foreach($variationElements_1 as $name => $arr){
			$tmp[$name] = $arr;
			$model->delElement($name);
		}

		// overwrite pos 1 with pos 2
		foreach($variationElements_2 as $name => $arr){
			$model->elements[self::getNameForPosition($name, $pos1)] = $model->elements[$name];
			$model->delElement($name);
		}

		// restore pos 1 to pos2
		foreach($tmp as $name => $arr){
			$model->elements[self::getNameForPosition($name, $pos2)] = $tmp[$name];
		}
		// delete backup
		unset($tmp);
	}

	private static function getNameForPosition($name, $pos){
		return self::PREFIX . $pos .
			(($fieldName = self::getFieldNameFromElemName($name)) ? '_' . self::getFieldNameFromElemName($name) : '');
	}

	private static function removeVariant(we_contents_document $model, $delPos){
		$total = we_base_variants::getNumberOfVariants($model);

		$lastPos = $total - 1;
		// move at last position, then remove it
		while($delPos < $lastPos){
			self::moveVariant($model, $delPos++, 'down');
		}
		// first remove all fields from doc
		$variationFields = we_base_variants::getAllVariationFields($model, $delPos);
		foreach(array_keys($variationFields) as $name){
			$model->delElement($name);
		}
		if(is_array($model->elements[self::ELEMENT_NAME]['dat'][$delPos])){
			unset($model->elements[self::ELEMENT_NAME]['dat'][$delPos]);
		}
	}

	public static function getVariantsEditorMultiBoxArrayObjectFile(we_base_jsCmd $jsCmd, we_objectFile $model){
		$variantFields = $model->getVariantFields();

		$count = we_base_variants::getNumberOfVariants($model);

		$parts = $regs = [];

		if($count){
			for($i = 0; $i < $count; $i++){
				$plusBut = we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('insert_variant','" . $i . "');");
				$upbut = ($i == 0 ? we_html_button::create_button(we_html_button::DIRUP, '', '', '', true) : we_html_button::create_button(we_html_button::DIRUP, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('move_variant_up','" . $i . "');"));
				$downbut = ($i == ($count - 1) ? we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) : we_html_button::create_button(we_html_button::DIRDOWN, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('move_variant_down','" . $i . "');"));
				$trashbut = we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('remove_variant','" . $i . "');");

				$content = '<table class="lowContrast" style="width:700px;float:left;">
<tr><td>' . we_html_element::b('Name') . '</td></tr>
<tr><td>' . $model->getFieldHTML($jsCmd, self::PREFIX . $i, 'input', [], true, true) . '</td></tr>';

				foreach($variantFields as $realName => $attributes){

					$fieldInfo = explode('_', $realName); // Verursacht Bug #4682
					$type = $fieldInfo[0];
					$realname = $fieldInfo[1];
					if(preg_match('/(.+?)_(.*)/', $realName, $regs)){//und hier der fix #4682
						$type = $regs[1];
						$realname = $regs[2];
					}
					$name = self::PREFIX . $i . '_' . $realname;
					$content .= '<tr><td>' . we_html_element::b($realname) . we_html_element::div(['class' => 'weObjectDescription'], (!empty($model->DefArray[$type . '_' . $realname]['editdescription']) ? str_replace("\n", we_html_element::br(), $model->DefArray[$type . '_' . $realname]['editdescription']) : '')) . '</td></tr>
<tr><td style="padding-bottom:8px;">' . $model->getFieldHTML($jsCmd, $name, $type, $attributes, true, true) . '</td></tr>';
				}
				$content .= '</table><span class="clearfix">' . $plusBut . $upbut . $downbut . $trashbut . '</span>';
				$parts[] = ['headline' => g_l('modules_shop', '[variant]') . ' ' . ($i + 1),
					'html' => $content,
				];
			}
		} else {
			$i = 0;
		}

		$parts[] = ['html' => we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('insert_variant','" . $i . "');"),
		];
		return $parts;
	}

	public static function getVariantsEditorMultiBoxArray(we_webEditionDocument $model){
		$variationFields = $model->getVariantFields();

		$count = we_base_variants::getNumberOfVariants($model);

		$parts = [];

		if($count){
			for($i = 0; $i < $count; $i++){
				$content = '<table class="lowContrast" style="width:100%;">
<tr><td class="bold">Name</td></tr>
<tr><td>' . $model->formTextInput('input', self::PREFIX . $i, '') . '</td></tr>';

				foreach($variationFields as $name => $fieldInformation){
					$fieldInformation['attributes']['name'] = self::PREFIX . $i . '_' . $fieldInformation['attributes']['name'];
					$content .= '
	<tr><td><b>' . $name . '</b></td></tr>
	<tr><td>' . we_tag($fieldInformation['type'], $fieldInformation['attributes'], (isset($fieldInformation['content']) ? $fieldInformation['content'] : '')) . '</td></tr>
	<tr>';
				}
				$content .= '</table>';

				$parts[] = [
					'headline' => ($model->getElement(self::PREFIX . $i) ?: g_l('modules_shop', '[variant]') . ' ' . ($i + 1)) .
					we_html_element::div(['class' => 'addTrashUpDown'], we_html_element::div([], we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('insert_variant','" . $i . "');") .
							we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('remove_variant','" . $i . "');")) .
						we_html_element::div(['class' => 'UpDown'], ($i == 0 ? we_html_button::create_button(we_html_button::DIRUP, '', '', '', true) : we_html_button::create_button(we_html_button::DIRUP, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('move_variant_up','" . $i . "');")) .
							($i == ($count - 1) ? we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) : we_html_button::create_button(we_html_button::DIRDOWN, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('move_variant_down','" . $i . "');"))) .
						we_html_element::div(['class' => 'UpDown'], we_html_button::create_button(we_html_button::VIEW, "javascript:we_cmd('preview_variant','" . $GLOBALS['we_transaction'] . "','" . ($model->getElement(self::PREFIX . $i)) . "');")
						)
					),
					'html' => $content,
					'space' => we_html_multiIconBox::SPACE_MED,
				];
			}
		} else {
			$i = 0;
		}

		$parts[] = [
			'headline' => '',
			'html' => we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('insert_variant','" . $i . "');"),
		];
		return $parts;
	}

	public static function useVariant(we_contents_document $model, $name){
		$variantDatArray = $model->getElement(self::ELEMENT_NAME);

		$model->Variant = $name;
		if(!is_array($variantDatArray)){
			return;
		}

		foreach($variantDatArray as $variant){
			if(!is_array($variant)){
				continue;
			}
			if(isset($variant[$name])){//can we break if one is found??
				$variantData = $variant[$name];
				foreach($variantData as $elementName => $elementData){
					$model->elements[$elementName] = $elementData;
				}
			}
		}
	}

	/**
	 * This function sets variant data for serialised document in the shopping basket
	 * different function, due to performance reasons and the shop itself
	 *
	 * @param array $record
	 * @param string $name
	 */
	public static function useVariantForShop(array &$record, $name){
		if(!isset($record[self::ELEMENT_NAME])){
			return;
		}
		$variantDatArray = we_unserialize($record[self::ELEMENT_NAME]);

		foreach($variantDatArray as $variant){
			foreach($variant as $variantName => $variantData){
				if($variantName == $name){
					foreach($variantData as $elementName => $elementData){
						$record[$elementName] = ($elementData['type'] === 'img' ? $elementData[we_contents_base::ETYPE_NUMBER] : $elementData[we_contents_base::ETYPE_TEXT]);
					}
				}
			}
		}
	}

	/**
	 * This function sets variant data for serialised object in the shopping basket
	 * different function, due to performance reasons and the shop itself
	 *
	 * @param array $record
	 * @param string $name
	 * @param we_objectFile $model
	 */
	public static function useVariantForShopObject(array $record, $name, we_objectFile $model){
		if(!$model->issetElement(self::ELEMENT_NAME)){
			return $record;
		}
		$variantDatArray = $model->getElement(self::ELEMENT_NAME, we_contents_base::ETYPE_TEXT);

		if(!is_array($variantDatArray)){
			return $record;
		}

		foreach($variantDatArray as $variant){
			foreach($variant as $variantName => $variantData){
				if($variantName == $name){
					foreach($variantData as $elementName => $elementData){
						// fields have the prefix we_
						$record[$elementName] = ($elementData['type'] === 'img' ? (isset($elementData[we_contents_base::ETYPE_NUMBER]) ? $elementData[we_contents_base::ETYPE_NUMBER] : '') : (isset($elementData[we_contents_base::ETYPE_TEXT]) ? $elementData[we_contents_base::ETYPE_TEXT] : ''));
					}
				}
			}
		}
		return $record;
	}

	public static function getVariantData(we_contents_document $model, $defaultname){
		if(!$model->issetElement(self::ELEMENT_NAME)){
			return [];
		}

		// add default data to listview
		$elements = $model->getElement(self::ELEMENT_NAME, we_contents_base::ETYPE_TEXT);
		//this elemets contains only the variant fields, not the non-variant fields of the object
		if(!is_array($elements) && ($elements[0] === 'a' || $elements[0] === '{')){
			$elements = we_unserialize($elements);
		}

		$newPos = is_array($elements) ? count($elements) : 0;

		if($newPos > 0){

			$elemdata = $elements[0];
			if(is_array($elemdata) && $defaultname != ''){
				$noFirst = (strpos($defaultname, 'FIRST') === false);
				foreach($elemdata as $name => $varArr){
					foreach($varArr as $key => $fieldArr){
						if(isset($model->elements[$key])){
							if($noFirst){
								$elements[$newPos][$defaultname][$key] = $model->elements[$key];
							} else {
								$elementF[$defaultname][$key] = $model->elements[$key];
							}
						}
					}
				}
				if(!$noFirst){
					array_unshift($elements, $elementF);
				}
			}
		}
		// attemot to add the other fields
		$modelelemets = $model->elements; //get a copy of the non variant fields
		unset($modelelemets[self::ELEMENT_NAME]); // get rid of some keys
		foreach(array_keys($modelelemets) as $key){
			if(strpos($key, self::PREFIX) !== false && strpos($key, self::PREFIX) == 0){
				unset($modelelemets[$key]);
			}
		}
		if($newPos > 0 && $elements){ //Fix #6883 - not sure if this has an impact
			foreach($elements as $name => &$varArr){//now add the elements
				foreach($varArr as $key => &$fieldArr){
					$fieldArr = array_merge($modelelemets, $fieldArr);
				}
			}
			unset($varArr, $fieldArr);
		}
		//
		return $elements;
	}

	public static function edit(we_base_jsCmd $jsCmd, $isObject, $command, we_contents_document $we_doc){
		switch($command){
			case 'insert_variant':
				self::insertVariant($we_doc, we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1));
				break;
			case 'move_variant_up':
				self::moveVariant($we_doc, we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1), 'up');
				break;
			case 'move_variant_down':
				self::moveVariant($we_doc, we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1), 'down');
				break;
			case 'remove_variant':
				self::removeVariant($we_doc, we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1));
				break;
			case 'preview_variant':
				self::correctModelFields($we_doc, false);
				self::useVariant($we_doc, we_base_request::_(we_base_request::STRING, 'we_cmd', 0, 2));

				echo $we_doc->getDocument();
				exit;
		}
		$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = true;

		return we_html_multiIconBox::getHTML('', ($isObject ? self::getVariantsEditorMultiBoxArrayObjectFile($jsCmd, $we_doc) : self::getVariantsEditorMultiBoxArray($we_doc)), true);
	}

	public static function getTemplateCode(we_template $we_doc){
		$fields = $we_doc->getVariantFieldNames();

		$content = [];
		foreach($fields as $field){
			$element = $we_doc->getElement('variant_' . $field, we_contents_base::ETYPE_TEXT);
			$content[] = [['dat' => we_html_forms::checkboxWithHidden($element ? true : false, 'we_' . $we_doc->Name . '_variant[variant_' . $field . ']', $field, false, 'middlefont', '_EditorFrame.setEditorIsHot(true);')
			]];
		}

		$parts = [['headline' => '',
			'html' => we_html_tools::alertAttentionBox(g_l('weClass', '[variant_info]'), we_html_tools::TYPE_INFO, 620, false),
			'noline' => true
			],
			['headline' => '',
				'html' => we_html_tools::dialogBorder(600, $content, [g_l('weClass', '[variant_fields]')]),
				'noline' => true
			]
		];

		return we_contents_base::hiddenTrans() .
			we_html_multiIconBox::getHTML('template_variant', $parts, true);
	}

}
