<?php

/**
 * webEdition CMS
 *
 * $Rev: 46d5cda804ed $
 * $Author: Lukas Imhof $
 * $Date: 2019-08-31 17:02 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_audio extends we_contents_binaryDocument{

	public function __construct(){
		parent::__construct();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_PREVIEW;
		}
		$this->ContentType = we_base_ContentTypes::AUDIO;
	}

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_document_preview($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	public function getHtml($dyn = false){
		$data = $this->getElement('data', we_contents_base::ETYPE_TEXT);
		if($this->ID || ($data && !is_dir($data) && is_readable($data))){

			return '<audio controls preload="none" style="margin-left:2em;">
							<source src="' . ( $dyn ?
				WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=show_binaryDoc&we_cmd[1]=' . $this->ContentType . '&we_cmd[2]=' . $GLOBALS['we_transaction'] . '&rand=' . we_base_file::getUniqueId() :
				$this->Path) . '" type="audio/' . str_replace('.', '', $this->Extension) . '">
						</audio>';
		}
		return '';
	}

	static function checkAndPrepare($formname, $key = 'we_document'){
		if(!(isset($_FILES['we_ui_' . $formname]) && is_array($_FILES['we_ui_' . $formname]) && isset($_FILES['we_ui_' . $formname]['name']) && is_array($_FILES['we_ui_' . $formname]['name']) )){
			return;
		}

		foreach($_FILES['we_ui_' . $formname]['name'] as $audioName => $filename){
			$audioDataId = we_base_request::_(we_base_request::STRING, we_fileupload::USERINPUT_TYPE_AUDIO . $audioName);
			if(($audioDataId = we_base_request::_(we_base_request::STRING, we_fileupload::USERINPUT_TYPE_AUDIO . $audioName)) === false || empty($_SESSION[$audioDataId])){
				continue;
			}

			$_SESSION[$audioDataId]['doDelete'] = false;
			if(we_base_request::_(we_base_request::BOOL, we_fileupload::USERINPUT_TYPE_DEL . $audioName)){
				$_SESSION[$audioDataId]['doDelete'] = true;
			} elseif($filename){
				if(($ct = getContentTypeFromFile($filename)) === we_base_ContentTypes::AUDIO){
					$audioid = intval($GLOBALS[$key][$formname]->getElement($audioName));
					$_SESSION[$audioDataId]['serverPath'] = TEMP_PATH . we_base_file::getUniqueId();
					we_base_file::insertIntoCleanUp($_SESSION[$audioDataId]['serverPath']);
					move_uploaded_file($_FILES['we_ui_' . $formname]['tmp_name'][$audioName], $_SESSION[$audioDataId]['serverPath']);
					$unique = we_base_file::getUniqueId();
					$tmp_Filename = $audioName . '_' . $unique . '_' . preg_replace('[^A-Za-z0-9._-]', '', $_FILES['we_ui_' . $formname]['name'][$audioName]);

					if($audioid){
						$_SESSION[$audioDataId]['id'] = $audioid;
					}
					$_SESSION[$audioDataId]['fileName'] = preg_replace('#^(.+)\..+$#', '${1}', $tmp_Filename);
					$_SESSION[$audioDataId]['extension'] = (strpos($tmp_Filename, ".") > 0) ? preg_replace('#^.+(\..+)$#', '${1}', $tmp_Filename) : '';
					$_SESSION[$audioDataId]['text'] = $_SESSION[$audioDataId]['fileName'] . $_SESSION[$audioDataId]['extension'];
					$_SESSION[$audioDataId]['unique'] = $unique;

					$_SESSION[$audioDataId]['type'] = $_FILES['we_ui_' . $formname]['type'][$audioName];
					$_SESSION[$audioDataId]['size'] = $_FILES['we_ui_' . $formname]['size'][$audioName];
				}
			}
		}
	}

}
