<?php

/**
 * webEdition CMS
 *
 * $Rev: 0194cde98bae $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-12-19 02:49 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview>
 *
 */
class we_listview_document extends we_listview_base{

	var $docType = ''; /* doctype string */
	var $casesensitive = false; /* set to true when a search should be case sensitive */
	var $contentTypes = '';
	var $subfolders = true; // regard subfolders
	var $languages = ''; //string of Languages, separated by ,
	var $numorder = false; // #3846
	public $triggerID = 0;
	protected $joins = [];
	protected $table = FILE_TABLE;
	protected $group = '';

	/**
	 *
	 * constructor of class
	 *
	 * @param   name          string  - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   offset        integer - start offset of first page
	 * @param   order         string  - field name(s) to order by
	 * @param   desc          boolean - set to true, if order should be descendend
	 * @param   docType       string  - doctype
	 * @param   cats          string  - comma separated categories
	 * @param   catOr         boolean - set to true if it should be an "OR condition"
	 * @param   casesensitive boolean - set to true when a search should be case sensitive
	 * @param   workspaceID   array - commaseperated list of id's of workspace
	 * @param   contentTypes  string  - contenttypes of documents (image,text ...)
	 * @param   cols   		  integer - to display a table this is the number of cols
	 * @param   searchable 	  boolean - if false then show also documents which are not marked as searchable
	 * @param string $condition
	 * @param unknown_type $calendar
	 * @param unknown_type $datefield
	 * @param unknown_type $date
	 * @param unknown_type $weekstart
	 * @param string $categoryids
	 */
	function __construct($name, $rows, $offset, $order, $desc, $docType, $cats, $catOr, $casesensitive, array $workspaceID, $contentTypes, $cols, $searchable, $condition, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $subfolders, $customers, $id, $languages, $numorder, $hidedirindex, $triggerID){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id);
		$this->docType = trim($docType);
		$this->casesensitive = $casesensitive;
		$this->contentTypes = $contentTypes;
		$this->searchable = $searchable;
		$this->subfolders = $subfolders;
		$this->customers = $customers;
		$this->languages = $languages;
		$this->condition = $condition;
		$this->numorder = $numorder;
		$this->hidedirindex = $hidedirindex;
		$this->triggerID = $triggerID;

		$this->customerArray = [];
		if($this->table === VFILE_TABLE){
			$id = $this->id = 0;
		}

		$this->group .= ($this->group ? ',' : '') . 'f.ID';

		$calendar_select = $calendar_where = '';

		if($calendar != ''){
			$this->fetchCalendar($condition, $calendar_select, $calendar_where);
		}

		$cond_where = // #3763
				($this->condition != '' && ($condition_sql = $this->makeConditionSql($this->condition)) ? '(' . $condition_sql . ')' : '');

		$langArray = $this->languages ? array_filter(array_map('trim', explode(',', $this->languages))) : '';

		$where_lang = ($langArray ? 'f.Language IN("","' . implode('","', array_map('escape_sql_query', $langArray)) . '") ' : '');

		if(stripos($this->order, ' desc') !== false){//was #3849
			$this->order = str_ireplace(' desc', '', $this->order);
			$this->desc = true;
		}

		$this->order = trim($this->order);
		$random = false;

		$order = [];
		$tmpOrder = explode(',', $this->order);
		foreach($tmpOrder as $ord){
			switch(strtolower(trim($ord))){
				case self::ID_FIELD:
					$order[] = 'f.ID' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATECREATION_FIELD:
					$order[] = 'f.CreationDate' . ($this->desc ? ' DESC' : '');
					break;
				case self::PATH_FIELD:
					$order[] = 'f.Path' . ($this->desc ? ' DESC' : '');
					break;
				case self::FILENAME_FIELD:
					$order[] = 'f.Text' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATEMOD_FIELD:
					$order[] = 'f.ModDate' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATEPUBLISHED_FIELD:
					$order[] = 'f.Published' . ($this->desc ? ' DESC' : '');
					break;
				case self::RANDOM_FIELD:
					$random = true;
					$order[] = 'RANDOM';
				case '':
					break;
				case 'vfile':
					$order[] = 'fl.position';
					break;
				default:
					$cnt = count($this->joins);
					$this->joins[] = ' LEFT JOIN ' . CONTENT_TABLE . ' cc' . $cnt . ' ON (cc' . $cnt . '.DID=f.ID AND cc' . $cnt . '.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND cc' . $cnt . '.nHash=x\'' . md5($ord) . '\')';
					if($this->search){
						$order[] = 'ranking';
					}
					//FIXME how to know if we sort by dat or bdid?! otherwise numeric sort is not correct unless numorder is set
					$order[] = ($this->numorder ? 'CAST(' : '') . 'COALESCE(cc' . $cnt . '.Dat,cc' . $cnt . '.BDID)' . ($this->numorder ? ' AS UNSIGNED) ' : '') . ($this->desc ? ' DESC' : '');
					break;
			}
		}
		$orderstring = $order ? ' ORDER BY ' . implode(',', $order) : '';
		$joinstring = implode('', $this->joins);

		$cat_tail = ($this->cats || $this->categoryids ? we_category_category::getCatSQLTail($this->cats, 'f', $this->catOr, $this->DB_WE, 'Category', $this->categoryids) : '');

		$dt = ($this->docType ? f('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE IsFolder=0 AND DocType LIKE "' . $this->DB_WE->escape($this->docType) . '"', '', $this->DB_WE) : -1);

		$ws_where = '';

		if($this->contentTypes){
			$this->contentTypes = str_replace(['img', 'wepage', 'binary'], [we_base_ContentTypes::IMAGE, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::APPLICATION,
				we_base_ContentTypes::PDF], $this->contentTypes);
			$CtArr = explode(',', $this->contentTypes);
			$ctTail = ($CtArr ? 'f.ContentType IN ("' . implode('","', array_map('escape_sql_query', $CtArr)) . '")' : '');
		} else {
			$ctTail = '';
		}
		$sql_tail = (we_modules_info::isActive(we_modules_info::CUSTOMER) ? we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this, 0, $id) : '');

		$cust = '';
		if($this->customers && $this->customers !== '*'){
			$this->customerArray = $this->DB_WE->getAllFirstq('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . $this->customers . ')', true, MYSQLI_ASSOC);

			$cust = '(f.WebUserID IN(' . $this->customers . '))';
		}

		if($this->search){
			if($this->workspaceID){
				$cond = ['f.ParentID IN(' . implode(',', $this->workspaceID) . ')'];
				$workspaces = array_map('escape_sql_query', id_to_path($this->workspaceID, FILE_TABLE, $this->DB_WE, true));

				foreach($workspaces as $workspace){
					$cond[] = 'f.Path LIKE "' . $workspace . '/%"';
				}
				$ws_where = '(' . implode(' OR ', $cond) . ')';
			}
			$bedingungen = preg_split('/ +/', $this->search);

			$ranking = '0';
			$spalten = [($this->casesensitive ? 'BINARY ' : '') . ($this->searchable ? 'i.Text' : 'ci.Dat')];

			foreach($bedingungen as $v1){
				if(preg_match('|^[-\+]|', $v1)){
					$not = (preg_match('/^-/', $v1));
					$bed = preg_replace('/^[-\+]/', '', $v1);
					$klammer = [];
					reset($spalten);
					foreach($spalten as $v){
						$klammer[] = sprintf('%s LIKE "%%%s%%"', $v, addslashes($bed));
					}
					if($not){
						$bedingungen_sql[] = ' NOT (' . implode(' OR ', $klammer) . ')';
					} else {
						$bedingungen_sql[] = '(' . implode(' OR ', $klammer) . ')';
					}
				} else {
					$klammer = [];
					foreach($spalten as $v){
						$klammer[] = sprintf('%s LIKE "%%%s%%"', $v, addslashes($v1));
					}
					$bed2 = '(' . implode(' OR ', $klammer) . ')';
					$ranking .= '-' . $bed2;
					$bedingungen2_sql[] = $bed2;
				}
			}

			if(isset($bedingungen2_sql) && $bedingungen2_sql){
				$bedingung_sql = '(' .
						implode(' OR ', $bedingungen2_sql) . ')' .
						(empty($bedingungen_sql) ? '' : ' AND ' . implode(' AND ', $bedingungen_sql));
			} else if(isset($bedingungen_sql) && $bedingungen_sql){
				$bedingung_sql = implode(' AND ', $bedingungen_sql);
			} else {
				$bedingung_sql = '';
			}


			$extraSelect = ',' . ($random ? ' RAND() AS RANDOM ' : $ranking . ' AS ranking ') . $calendar_select;
			$limit = (($this->maxItemsPerPage) ? (' LIMIT ' . abs($this->start) . ',' . abs($this->maxItemsPerPage)) : '');
		} else {
			$bedingung_sql = '';
			if($this->workspaceID){
				$workspaces = $this->workspaceID;
				if($this->subfolders){ // all entries with given parentIds
					$cond = [];
					$workspacePaths = id_to_path($workspaces, FILE_TABLE, $this->DB_WE, true);
					foreach($workspacePaths as $workspace){
						$cond[] = 'Path LIKE "' . $this->DB_WE->escape($workspace) . '/%"';
					}
					if($cond){
						$workspaces = array_unique(array_merge($workspaces, $this->DB_WE->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE IsFolder=1 AND (' . implode(' OR ', $cond) . ')', true)));
					} else {
						$cond = [-1];
					}
				}
				$ws_where = '(f.ParentID IN (' . implode(', ', $workspaces) . '))';
			}
			$extraSelect = ($random ? ', RAND() AS RANDOM' : '');
			$limit = (($rows > 0) ? (' LIMIT ' . abs($this->start) . ',' . abs($this->maxItemsPerPage)) : "");
		}
		$where = implode(' AND ', array_filter([
					'default' => 'f.IsFolder=0 AND f.Published>0',
					'search' => ($this->searchable ? ' f.IsSearchable=1' : 1),
					'lang' => $where_lang,
					'cond' => $cond_where,
					'beding' => $bedingung_sql,
					'docT' => ($this->docType ? ($dt ? 'f.DocType=' . intval($dt) : 'FALSE'/* invalid DT => no results */) : ''),
					'cal' => $calendar_where,
					'ws' => $ws_where,
					'cat' => $cat_tail,
					'contentType' => $ctTail,
					'cust' => $cust,
					'id' => $this->getIdQuery('f.ID'),
				])) . ' ' . $sql_tail .
				' GROUP BY ' . $this->group;

		$this->DB_WE->query(
				'SELECT f.ID,f.WebUserID' . $extraSelect .
				' FROM ' . FILE_TABLE . ' f JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '") ' . $joinstring .
				($this->search ? ' JOIN ' .
						($this->searchable ?
								INDEX_TABLE . ' i ON (i.ID=f.ID AND i.ClassID=0) ' :
								CONTENT_TABLE . ' ci ON (f.ID=ci.DID AND ci.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND ci.Type IN ("input","txt")) ') : '') .
				' WHERE ' . $where . ' ' . $orderstring . $limit
		);

		$this->anz = $this->DB_WE->num_rows();

		$idListArray = [];

		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$this->IDs[] = $this->DB_WE->f('ID');
			if($calendar != ''){
				$this->calendar_struct['storage'][$this->DB_WE->f('ID')] = intval($this->DB_WE->f('Calendar'));
			}
			if($this->customers === '*' && intval($this->DB_WE->f('WebUserID')) > 0){
				$idListArray[] = $this->DB_WE->f('WebUserID');
			}
		}
		if($this->customers === '*' && $idListArray){
			$this->customerArray = $this->DB_WE->getAllFirstq('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . $this->customers . ')', true, MYSQLI_ASSOC);
			unset($idListArray);
		}

		$this->DB_WE->query(
				'SELECT f.ID,f.WebUserID' .
				($random ? ',RAND() AS RANDOM' : ($this->search ? ',' . $ranking . ' AS ranking' : '')) .
				' FROM ' . FILE_TABLE . ' f JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '") ' .
				($this->search ? ' JOIN ' .
						($this->searchable ?
								INDEX_TABLE . ' i ON (i.ID=f.ID AND i.ClassID=0) ' :
								CONTENT_TABLE . ' ci ON (f.ID=ci.DID AND ci.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND ci.Type IN ("input","txt")) ') : '') .
				$joinstring .
				' WHERE ' . $where . ' ' . $orderstring);

		$this->anz_all = $this->DB_WE->num_rows();
		if($calendar != ''){
			$this->postFetchCalendar();
		}
	}

	public function next_record(){
		if($this->count < $this->anz){
			$count = $this->count;
			$fetch = false;
			if($this->calendar_struct){
				parent::next_record();
				$count = $this->calendar_struct['count'];
				$fetch = $this->calendar_struct['forceFetch'];
			}

			if(!$this->calendar_struct || $fetch){
				$id = $this->IDs[$count];
				$data = getHash('SELECT ID,ParentID,Text,IsFolder,ContentType,CreationDate,ModDate,Path,TemplateID,Filename,Extension,IsDynamic,IsSearchable,DocType,ClassName,Category,Published,CreatorID,ModifierID,RestrictOwners,Owners,OwnersReadOnly,Language,WebUserID,InGlossar
FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id), $this->DB_WE, MYSQLI_ASSOC);

				$this->Record = array_change_key_case(array_merge($this->DB_WE->getAllFirstq('SELECT c.Name,IF(c.BDID!=0,c.BDID,c.Dat) AS data FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . intval($id) . ' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"', false), self::preFixArrayKeys($data)));

				if(!empty($this->Record[we_base_variants::ELEMENT_NAME])){
					$variants = is_string($this->Record[we_base_variants::ELEMENT_NAME]) ? we_unserialize($this->Record[we_base_variants::ELEMENT_NAME]) : [];
					$this->Record[self::SHOPVARIANTS_FIELD] = (is_array($variants) && $variants ? count($variants) : 0);
				} else {
					$this->Record[self::SHOPVARIANTS_FIELD] = 0; //check this for global variants
				}

				if($this->customers && $this->Record[self::CUSTOMER_FIELD] && isset($this->customerArray[$this->Record[self::CUSTOMER_FIELD]])){
					$this->Record = array_merge($this->Record, self::preFixArrayKeys($this->customerArray[$this->Record[self::CUSTOMER_FIELD]]));
				}

				$this->count++;
			}

			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}
		return false;
	}

	public function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

	public function makeConditionSql($cond){
		//FIXME: a>5 AND b>5 will not work, we have a flat join, so both conditions on different link-values cannot match
		$cond = strtr($cond, ['&gt;' => '>', '&lt;' => '<']);
		$func = function($value){
			return trim($value, " \t\n\r\0\x0B()");
		};
		$arr = array_map($func, preg_split('/( AND | OR | && | \|\| | FIND_IN_SET\()/i', $cond, -1, PREG_SPLIT_NO_EMPTY));
		$opPatterns = ['<>', '!=', '<=', '>=', '=', '<', '>', 'NOT LIKE', 'LIKE', 'NOT IN', '[^_]IN'];
		$funcPatterns = ['FIND_IN_SET'];
		foreach($arr as $exp){
			foreach($opPatterns as $pattern){
				$match = preg_split('/' . $pattern . '/', $exp, -1, PREG_SPLIT_NO_EMPTY);
				if(count($match) > 1){
					$sqlarr = $this->makeFieldOpCondition(trim($match[0]), $pattern, trim($match[1]));
					$cond = str_replace($match[0] . $pattern . $match[1], $sqlarr, $cond);
					break;
				}
			}
			foreach($funcPatterns as $pattern){
				$match = preg_split('/' . $pattern . '/', $exp, -1);
				if(!empty($match[1])){
					list($value, $name) = explode(',', trim($match[1], '( '), 2);
					$sqlarr = $this->makeFieldFuncCondition($pattern, trim($value), trim($name));
					$cond = str_replace($pattern . $match[1], $sqlarr, $cond);
				}
			}
		}
		return $cond;
	}

	private function makeFieldFuncCondition($function, $value, $name){
		return (stripos($name, self::PROPPREFIX) === 0) ? //Fix: #9389
				'(' . $function . '(' . $value . ',f.' . substr($name, 3) . '))' :
				'(c.nHash=x\'' . md5($name) . '\' AND ' . $function . '(' . $value . ',COALESCE(c.Dat,c.BDID))';
	}

	private function makeFieldOpCondition($name, $operation, $value){
		return (stripos($name, self::PROPPREFIX) === 0) ? //Fix: #9389
				'(f.' . substr($name, 3) . ' ' . $operation . ' ' . $value . ')' :
				'(c.nHash=x\'' . md5($name) . '\' AND COALESCE(c.Dat,c.BDID) ' . $operation . ' ' . $value . ')';
	}

	public function getCustomerRestrictionQuery($specificCustomersQuery, $classID, $mfilter, $listQuery){
		return 'FROM ' . CUSTOMER_FILTER_TABLE . ' cf WHERE cf.modelTable="' . stripTblPrefix(FILE_TABLE) . '" AND ' . $mfilter . ' AND (' . $listQuery . ' OR ' . $specificCustomersQuery . ')';
	}

}
