<?php
/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_object
 * @desc    class for tag <we:listview type="object">
 *
 */
class we_listview_object extends we_listview_objectBase{
	private $we_predefinedSQL = '';

	/**
	 * @desc    constructor of class
	 *
	 * @param   name          string - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   offset        integer - start offset of first page
	 * @param   order         string - field name(s) to order by
	 * @param   desc          boolean - set to true, if order should be descendend
	 * @param   cats          string - comma separated categories
	 * @param   catOr         boolean - set to true if it should be an "OR condition"
	 * @param   condition     string - condition string (like SQL)
	 * @param   triggerID     integer - ID of a document which to use for displaying the detail page
	 * @param   cols          ??
	 * @param   seeMode       boolean - value if objects shall be accessible in seeMode (default true)
	 * @param   searchable 	  boolean - if false then show also documents which are not marked as searchable
	 * @param	unknown_type  $calendar
	 * @param	unknown_type  $datefield
	 * @param	unknown_type  $date
	 * @param	unknown_type  $weekstart
	 * @param	string        $categoryids
	 *
	 */
	public function __construct($name, $rows, $offset, $order, $desc, $classID, $cats, $catOr, $condition, $triggerID, $cols, $seeMode, $searchable, $calendar, $datefield, $date, $weekstart, $categoryids, array $workspaceID, $customerFilterType, $docID, $customers, $id, $we_predefinedSQL, $languages, $hidedirindex, $objectseourls, $fromWeObject = false){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id, $triggerID, $seeMode, $searchable, $docID, $languages, $objectseourls, $hidedirindex);

		$this->classID = intval($classID);
		$this->customers = $customers;
		$this->condition = $condition;
		$this->customerArray = [];

		if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		//FIXME: remove this in 8.2
		$this->we_predefinedSQL = $we_predefinedSQL;


		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '');

		$this->fillMatrix($this->classID, true);

		$calendar_select = '';
		$calendar_where = '';

		if($calendar){
			$this->fetchCalendar($this->condition, $calendar_select, $calendar_where, $this->matrix);
		}
		$sqlParts = $this->makeSQLParts($this->classID, $this->order, $this->condition);
		//allways join the file table itself
		$sqlParts['tables'] .= ' JOIN ' . OBJECT_FILES_TABLE . ' `of` ON `of`.ID=ob' . $this->classID . '.OF_ID';

		if($this->customers && $this->customers !== '*'){
			$wsql = ' `of`.WebUserID IN(' . $this->customers . ') ';
			$this->customerArray = $this->DB_WE->getAllFirstq('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . $this->customers . ')', true, MYSQLI_ASSOC);
			$webUserID_tail = '(' . $wsql . ')';
		} else {
			$webUserID_tail = '';
		}

		$condParts = [
			'search' => ($this->searchable ? '`of`.IsSearchable=1' : ''),
			'pid' => ($fromWeObject ? '' : we_objectFile::makePIDTail($GLOBALS['we_doc']->ParentID, $this->classID)),
			'lang' => ($this->languages ? '`of`.Language IN ("' . implode('","', array_map('escape_sql_query', array_filter(array_map('trim', explode(',', $this->languages))))) . '")' : ''),
			'cat' => ($this->cats || $this->categoryids ? we_category_category::getCatSQLTail($this->cats, 'of', $this->catOr, $this->DB_WE, 'Category', $this->categoryids) : ''),
			'publ' => $sqlParts['publ_cond'],
			'cond' => ($sqlParts['cond'] ? '(' . $sqlParts['cond'] . ')' : ''),
			'cal' => $calendar_where,
			'webUser' => $webUserID_tail,
		];

		if($sqlParts['tables'] || $we_predefinedSQL){

			if($we_predefinedSQL){//FIXME: remove this in 8.2
				$this->DB_WE->query($we_predefinedSQL);
				$this->anz_all = $this->DB_WE->num_rows();
				$q = $we_predefinedSQL . (($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : '');
			} else {
				$condParts['id'] = $this->getIdQuery('ob' . $this->classID . '.OF_ID');

				if($this->workspaceID){
					$workspaces = id_to_path($this->workspaceID, OBJECT_FILES_TABLE, $this->DB_WE, true);
					$cond = [];
					foreach($workspaces as $workspace){
						$cond[] = '`of`.Path LIKE "' . $workspace . '/%"';
						$cond[] = '`of`.Path="' . $workspace . '"';
					}
					$condParts['ws'] = empty($cond) ? '' : '(' . implode(' OR ', $cond) . ')';
				}

				$where = implode(' AND ', array_filter($condParts)) .
					(we_modules_info::isActive(we_modules_info::CUSTOMER) ? we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this, $this->classID, $id) : '') .
					$sqlParts['groupBy'];
				$this->DB_WE->query('SELECT `of`.ID ' . $calendar_select . ' FROM ' . $sqlParts['tables'] . ' WHERE ' . $where);
				$this->anz_all = $this->DB_WE->num_rows();
				if($calendar){
					while($this->DB_WE->next_record(MYSQLI_ASSOC)){
						$this->IDs[] = $this->DB_WE->f('ID');
						$this->calendar_struct['storage'][$this->DB_WE->f('ID')] = (int) $this->DB_WE->f('Calendar');
					}
				}
				$q = 'SELECT ' . $sqlParts['fields'] . $calendar_select . ' FROM ' . $sqlParts['tables'] . ' WHERE ' . $where . $sqlParts['order'] . (($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : '');
			}
			$this->DB_WE->query($q);
			$this->anz = $this->DB_WE->num_rows();

			if($this->customers === '*'){
				$idListArray = [];
				while($this->DB_WE->next_record(MYSQLI_ASSOC)){
					if(intval($this->DB_WE->f(self::CUSTOMER_FIELD)) > 0){
						$idListArray[] = $this->DB_WE->f(self::CUSTOMER_FIELD);
					}
				}
				if($idListArray){
					$db = new DB_WE();
					$this->customerArray = $db->getAllFirstq('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . implode(',', array_unique($idListArray)) . ')', true, MYSQLI_ASSOC);
				}
				unset($idListArray);

				$this->DB_WE->seek(0);
			}
		} else {
			$this->anz_all = 0;
			$this->anz = 0;
		}
		if($calendar){
			$this->postFetchCalendar();
		}
	}

}

