<?php
/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class

 *
 */
abstract class we_listview_objectBase extends we_listview_base{
	const FIELD_REPLACEMENTS = [
		'we_filename' => '`of`.Text',
		OBJECT_X_TABLE => 'ob',
		OBJECT_FILES_TABLE => 'of',
		'we_' => self::PROPPREFIX,
		'wedoc_' => self::PROPPREFIX,
	];

	protected $classID = 0; /* ID of a class */
	var $triggerID = 0; /* ID of a document which to use for displaying thr detail page */
	protected $triggerPath = '';
	var $Path = ''; /* internal: Path of document which to use for displaying thr detail page */
	protected $languages = ''; //string of Languages, separated by ,
	var $objectseourls = false;
	//used by we_field
	public $docID = 0;
	protected $matrix = [];

	public function __construct($name = 0, $rows = 999999999, $offset = 0, $order = '', $desc = false, $cats = '', $catOr = false, array $workspaceID = [], $cols = 0, $calendar = '', $datefield = '', $date = '', $weekstart = '', $categoryids = '', $customerFilterType = 'all', $id = 0, $triggerID = 0, $seeMode = false, $searchable = false, $docID = 0, $languages = '', $objectseourls = false, $hidedirindex = false){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id);
		$this->triggerID = $triggerID;
		$this->searchable = $searchable;
		$this->docID = $docID; //Bug #3720
		$this->languages = $languages;
		$this->objectseourls = $objectseourls;
		$this->hidedirindex = $hidedirindex;
		$this->seeMode = $seeMode; //	edit objects in seeMode

		$this->triggerPath = ($this->triggerID ? id_to_path($this->triggerID, FILE_TABLE, $this->DB_WE) : '');
		$this->triggerID = $this->triggerPath ? $this->triggerID : 0;

		$this->Path = ($this->docID ?
			id_to_path($this->docID, FILE_TABLE, $this->DB_WE) :
			($this->triggerID && show_SeoLinks() ?
			id_to_path($this->triggerID, FILE_TABLE, $this->DB_WE) :
			(isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc']->Path : '')
			)
			) ?: (isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc']->Path : '');
	}

	public function next_record(){
		if($this->calendar_struct){
			if($this->count >= $this->anz){
				return false;
			}
			parent::next_record();
			$this->Record = [];
			if(!$this->calendar_struct['forceFetch']){
				return !empty($this->calendar_struct['calendar']);
			}
		}

		if($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$paramName = $this->docID ? 'we_oid' : WEOBJ_PARAM;
			$this->Record = array_merge(array_change_key_case($this->DB_WE->getRecord()), [
				self::CLASS_FIELD => $this->classID,
				self::PATH_FIELD => $this->Path . '?' . $paramName . '=' . $this->DB_WE->f(self::ID_FIELD),
				self::CUSTOMER_PREFIX . 'ID' => $this->DB_WE->f(self::CUSTOMER_FIELD),
				self::SHOPVARIANTS_FIELD => 0]); //check this for global variants

			$path_parts = pathinfo($this->Path);
			if($this->objectseourls && $this->Record[self::URL_FIELD] && show_SeoLinks()){
				if(!$this->triggerID && $this->Record[self::TRIGGER_FIELD]){
					$path_parts = pathinfo(id_to_path($this->Record[self::TRIGGER_FIELD]));
				}
				$this->Record[self::PATH_FIELD] = (!empty($path_parts['dirname']) && $path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
					($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					'' :
					$path_parts['filename'] . '/'
					) . $this->Record[self::URL_FIELD];
			} else {
				$this->Record[self::PATH_FIELD] = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
					$this->Path
					) . '?' . $paramName . '=' . $this->Record[self::ID_FIELD];
			}
			if(($dat = $this->f(we_base_variants::ELEMENT_NAME))){
				$variants = we_unserialize($dat);
				if(is_array($variants) && !empty($variants)){
					$this->Record[self::SHOPVARIANTS_FIELD] = count($variants);
				}
			}
			// for seeMode #5317
			$this->Record[self::LASTPATH_FIELD] = $this->LastDocPath . '?' . $paramName . '=' . $this->Record[self::ID_FIELD];

			if($this->customers && $this->Record[self::CUSTOMER_FIELD] && isset($this->customerArray[$this->Record[self::CUSTOMER_FIELD]])){
				$this->Record = array_merge($this->Record, self::preFixArrayKeys($this->customerArray[$this->Record[self::CUSTOMER_FIELD]]));
			}

			$this->count++;
			return true;
		}

		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return !empty($this->calendar_struct['calendar']);
	}

	function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

	protected function fillMatrix($classID, $withVariant = false){
		$regs = $moreClasses = [];
		$table = OBJECT_X_TABLE . $classID;
		$tableInfo = we_objectFile::getSortedTableInfo($classID, true, $this->DB_WE, $withVariant);
		foreach($tableInfo as $fieldInfo){
			if(!preg_match('/(.+?)_(.*)/', $fieldInfo['name'], $regs)){
				continue;
			}
			list(, $type, $name) = $regs;
			if($type === 'object' && $name != $this->classID){
				if(!empty($this->matrix[self::OBJECT_FIELD_PREFIX . $name]['type'])){
					//class already added
					continue;
				}
				$this->matrix[self::OBJECT_FIELD_PREFIX . $name] = [
					'type' => $type,
					'table' => $table,
					'classID' => $classID,
					'alias' => 'ob' . $classID,
					'aliasf' => 'of' . $name,
					'joinClassID' => $name,
					'join' => OBJECT_X_TABLE . $name,
					'on' => OBJECT_FILES_TABLE . ' of' . $name . ' ON (of' . $name . '.ID=ob' . intval($classID) . '.' . we_object::QUERY_PREFIX . $name . ' AND of' . $name . '.Published>0) LEFT JOIN ' . OBJECT_X_TABLE . $name . ' AS ob' . $name . ' ON (ob' . $name . '.OF_ID=of' . $name . '.ID)'
				];
				$this->fillMatrix($name);
			} else {
				if(!isset($this->matrix[$name])){
					$this->matrix[$name] = [
						'type' => $type,
						'table' => $table,
						'alias' => 'ob' . $classID,
						'aliasf' => 'of' . $classID,
						'classID' => $classID,
					];
				}
			}
		}
	}

	protected function makeSQLParts(){
		if(!$this->classID){
			t_e('no classid given!');
			return [];
		}
		$from = $fromTable = $orderArr = $descArr = $ordertmp = [];
		$publ_cond = [
			'`of`.Published>0'
		];
		$cond = ' ' . preg_replace_callback("/'([^']*)'/", function (array $match){
				$in = $match[1];
				$out = '';
				for($i = 0; $i < strlen($in); $i++){
					$out .= '&' . ord(substr($in, $i, 1)) . ';';
				}
				return "'" . $out . "'";
			}, strtr($this->condition, ['&lt;&gt;' => '!=', '&gt;' => '>', '&lt;' => '<'])) . ' ';

		if($this->order){
			foreach(array_map('trim', explode(',', $this->order)) as $f){
				$g = explode(' ', $f);
				$orderArr[] = $g[0];
				$descArr[] = isset($g[1]) && strtolower(trim($g[1])) === 'desc';
			}
		}

		//get Metadata for class (default title, etc.)
		//BugFix #4629
		$fieldnames = getHash('SELECT DefaultDesc,DefaultTitle,DefaultKeywords FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->classID, $this->DB_WE);
		$fields = ['`of`.ID AS OF_ID'];
		foreach($fieldnames as $key => $val){
			if(!$val || $val === '_'){
				continue;
			}
			switch($key){
				case 'DefaultDesc':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::DESC_FIELD;
					break;
				case 'DefaultTitle':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::TITLE_FIELD;
					break;
				case 'DefaultKeywords':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::KEYWORDS_FIELD;
					break;
			}
		}
		$tableFields = array_keys(getHash('SELECT * FROM ' . OBJECT_FILES_TABLE . ' LIMIT 1'));
		foreach($tableFields as $cur){
			$fields[] = '`of`.' . $cur . ' AS ' . self::PROPPREFIX . $cur;
		}

		$charclass = '[\!\=%&\(\)\*\+\.\/<>\|~, -]';
		//joins need to keep their original order
		foreach($this->matrix as $n => $p){
			if(!isset($fromTable[$p['table']]) && intval($this->classID) === intval($p['classID'])){
				$fromTable[$p['table']] = $p['table'] . ' AS ' . $p['alias'];
			}
			if(!empty($p['join'])){
				$from[$p['join']] = $p['on'];
			}
		}
		//sort keys in reverse order so "Bild" is not matched before "Bild-XX"
		krsort($this->matrix);
		foreach($this->matrix as $n => $p){
			$n2 = $n;
			if(!empty($p['joinClassID'])){
				$n = $p['joinClassID'];
			}

			$fields[] = $p['alias'] . '.`' . $p['type'] . '_' . $n . '` AS `' . $n2 . '`';

			if(($pos = array_search($n, $orderArr)) !== false){
				$ordertmp[$pos] = $p['alias'] . '.`' . $p['type'] . '_' . $n . '`' . ($descArr[$pos] ? ' DESC' : '');
			}
			//some replacements if old conditions may occur
			$cond = strtr($cond, self::FIELD_REPLACEMENTS);
			$cond = preg_replace('/(' . $charclass . ')' . $n . '(' . $charclass . ')/', '${1}' . $p['alias'] . '.`' . $p['type'] . '_' . $n . '`$2', $cond);
		}
		if(count($fromTable) !== 1){
			t_e('we_listview_objectBase::makeSQLParts(): Wrong number of entries in var $fromTable', $fromTable, $from, $this->classID);
		}
		$from = array_slice($fromTable, 0, 1) + $from;

		$cond = preg_replace_callback("/'([^']*)'/", function (array $match){
			return "'" . preg_replace_callback("/&([^;]+);/", function (array $match){
					return chr($match[1]);
				}, $match[1]) . "'";
		}, $cond);

		$order = '';
		foreach($orderArr as $pos => $curOrd){
			switch(strtolower($curOrd)){
				case self::RANDOM_FIELD:
					$ordertmp = [];
					$order = 'RANDOM ';
					break 2;
				case 'we_filename':
					$ordertmp[$pos] = '`of`.Text' . ($descArr[$pos] ? ' DESC' : '');
					break;
				default:
					$ordertmp[$pos] = '`' . strtr($curOrd, self::FIELD_REPLACEMENTS) . '`' . ($descArr[$pos] ? ' DESC' : '');
			}
		}
		if($ordertmp){
			ksort($ordertmp);
			$order = trim(implode(',', $ordertmp));
		}
		if($order === 'RANDOM '){
			$fields[] = 'RAND() AS RANDOM';
		}
		return [
			'fields' => implode(',', $fields),
			'order' => $order ? ' ORDER BY ' . $order : '',
			'tables' => implode(' LEFT JOIN ', $from),
			//FIXME: afaik grouping is not needed
			'groupBy' => (count($from) > 1) ? ' GROUP BY `of`.ID ' : '',
			'publ_cond' => $publ_cond ? ' ( ' . implode(' AND ', $publ_cond) . ' ) ' : '',
			'cond' => trim($cond)
		];
	}

	public function getCustomerRestrictionQuery($specificCustomersQuery, $classID, $mfilter, $listQuery){
		return //at least check only documents of the specified class
			'FROM ' . CUSTOMER_FILTER_TABLE . ' cf JOIN ' . OBJECT_X_TABLE . $classID . ' obx ON (cf.modelId=obx.OF_ID AND cf.modelTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '") WHERE ' . $mfilter . ' AND (' . $listQuery . ' OR ' . $specificCustomersQuery . ')';
	}

	public function getFoundDocument(){
		static $doc = null;
		static $id = 0;
		if($id == ($docID = $this->f(self::ID_FIELD))){
			return $doc;
		}
		$id = $docID;
		$model = new we_objectFile();
		$model->initByID($docID, OBJECT_FILES_TABLE);
		$doc = $model;
		return $doc;
	}

}
