<?php
/**
 * webEdition CMS
 *
 * $Rev: 826eef84bc1f $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-22 20:42 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_object
 * @desc    class for tag <we:listview type="object">
 *
 */
class we_listview_sql extends we_listview_base{

	/**
	 * @desc    constructor of class
	 *
	 * @param   name          string - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   offset        integer - start offset of first page
	 * @param   order         string - field name(s) to order by
	 * @param   desc          boolean - set to true, if order should be descendend
	 * @param   cats          string - comma separated categories
	 * @param   catOr         boolean - set to true if it should be an "OR condition"
	 * @param   condition     string - condition string (like SQL)
	 * @param   triggerID     integer - ID of a document which to use for displaying the detail page
	 * @param   cols          ??
	 * @param   seeMode       boolean - value if objects shall be accessible in seeMode (default true)
	 * @param   searchable 	  boolean - if false then show also documents which are not marked as searchable
	 * @param	unknown_type  $calendar
	 * @param	unknown_type  $datefield
	 * @param	unknown_type  $date
	 * @param	unknown_type  $weekstart
	 * @param	string        $categoryids
	 *
	 */
	public function __construct($name, $rows, $offset, $order, $desc, $select, $from, $where, $predefinedSQL){
		parent::__construct($name, $rows, $offset, $order, $desc);

		if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		$predefinedSQL = $predefinedSQL ?: 'SELECT ' . $select . ' FROM ' . $from . ' WHERE ' . $where;

		$this->DB_WE->query($predefinedSQL);
		$this->anz_all = $this->DB_WE->num_rows();

		$this->DB_WE->query($predefinedSQL .
			($this->order ? ' ORDER BY ' . $this->order : '') .
			(($this->maxItemsPerPage) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(){
		if($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$this->count++;
			$this->Record = $this->DB_WE->getRecord() + [
				we_listview_base::ID_FIELD => $this->count
			];

			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}
		return false;
	}

	function f($key, $stripWE = false){
		return (isset($this->Record[$key]) ? $this->Record[$key] : '');
	}

}
