<?php

/**
 * webEdition CMS
 *
 * $Rev: 152b436a6cad $
 * $Author: mokraemer $
 * $Date: 2020-02-18 16:01 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_newsletter_frames extends we_modules_frame{
	const def_width = 450;
	const TAB_PROPERTIES = 0; //make sure to keep 1
	const TAB_MAILING = 1;
	const TAB_EDIT = 2;
	const TAB_REPORTING = 3;

	private $weAutoCompleter;

	function __construct($frameset){
		parent::__construct($frameset);
		$this->module = we_modules_info::NEWSLETTER;
		$this->View = new we_newsletter_view();
		$this->Tree = new we_tree_newsletter($this->jsCmd);
		$this->weAutoCompleter = &we_gui_suggest::getInstance();
	}

	public function getHTMLDocumentHeader($what = '', $mode = ''){
		switch($what){
			case 'send':
			case 'send_body':
			case 'send_cmd':
			case 'preview':
			case 'black_list':
			case 'newsletter_settings':
			case 'eemail':
			case 'edit_file':
			case 'clear_log':
			case 'export_csv_mes':
			case 'qsend':
				return '';
			default:
				return parent::getHTMLDocumentHeader();
		}
	}

	function getHTML($what = '', $mode = 0, $step = 0){
		switch($what){
			case 'qlog':
				return $this->getHTMLLogQuestion();
			case 'domain_check':
				return $this->getHTMLDCheck();
			case 'show_log':
				return $this->getHTMLLog();
			case 'newsletter_settings':
				return $this->getHTMLSettings();
			case 'print_lists':
				return $this->getHTMLPrintLists();
			case 'qsend':
				return $this->getHTMLSendQuestion();
			case 'eemail':
				return $this->getHTMLEmailEdit();
			case 'preview':
				return $this->getHTMLPreview();
			case 'black_list':
				return $this->getHTMLBlackList();
			case 'upload_black':
				return $this->getHTMLUploadCsv($what);
			case 'upload_csv':
				return $this->getHTMLUploadCsv($what);
			case 'export_csv_mes':
				return $this->getHTMLExportCsvMessage();
			case 'edit_file':
				return $this->getHTMLEditFile($mode);
			case 'clear_log':
				return $this->getHTMLClearLog();
			case 'send':
				return $this->getHTMLSendWait();
			case 'send_frameset':
				return $this->getHTMLSendFrameset();
			case 'send_body':
				return $this->getHTMLSendBody();
			case 'send_cmd':
				return $this->getHTMLSendCmd();
			case 'send_control':
				return $this->getHTMLSendControl();
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	/**
	 * Modul Header
	 *
	 * @param Integer $mode
	 * @return String
	 */
	protected function getHTMLEditorHeader($mode = 0){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return parent::getHTMLEditorHeader(0);
		}

		$group = we_base_request::_(we_base_request::BOOL, 'group');
		$page = ($group ? self::TAB_PROPERTIES : we_base_request::_(we_base_request::INT, 'page', self::TAB_PROPERTIES));

		$textPre = g_l('modules_newsletter', ($group ? '[group]' : '[newsletter][text]'));
		$textPost = we_base_request::_(we_base_request::STRING, "txt", g_l('modules_newsletter', ($group ? '[new_newsletter_group]' : '[new_newsletter]')));

		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, ($page == self::TAB_PROPERTIES), self::TAB_PROPERTIES, ['title' => g_l('modules_newsletter', '[property]')]);

		if(!$group){
			$we_tabs->addTab('', 'fa-list', ($page == self::TAB_MAILING), self::TAB_MAILING, ['title' => sprintf(g_l('modules_newsletter', '[mailing_list]'), "")]);
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_EDIT, ($page == self::TAB_EDIT), self::TAB_EDIT, ['title' => g_l('modules_newsletter', '[edit]')]);
			//if($this->View->newsletter->ID){ // zusaetzlicher tab fuer auswertung
			$we_tabs->addTab('', 'fa-pie-chart', ($page == self::TAB_REPORTING), self::TAB_REPORTING, ['title' => g_l('modules_newsletter', '[reporting][tab]')]);
			//}
		}

		$body = we_html_element::body(['onresize' => "weTabs.setFrameSize()", 'onload' => "weTabs.setFrameSize()", 'id' => 'eHeaderBody'], '<div id="main"><div id="headrow"><b>' . oldHtmlspecialchars($textPre) . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . oldHtmlspecialchars($textPost) . '</b></span></div>' .
				$we_tabs->getHTML() .
				'</div>'
		);
		return $this->getHTMLDocument($body, we_html_element::cssLink(CSS_DIR . 'we_tab.css') . we_html_element::jsScript(JS_DIR . 'initTabs.js') . we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_header.js'));
	}

	/**
	 * Modul Body
	 *
	 * @return String
	 */
	protected function getHTMLEditorBody(){
		return $this->getHTMLProperties();
	}

	/**
	 * Modul Footer
	 *
	 * @param Integer $mode
	 * @return String
	 */
	protected function getHTMLEditorFooter(array $mode = [0], $extraHead = ''){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return parent::getHTMLEditorFooter([]);
		}
		if(empty($mode)){
			$mode = [0];
		}
		$group = we_base_request::_(we_base_request::INT, "group", 0);

		$select = new we_html_select(['name' => 'gview']);

		$main = $optional = '';
		if($mode[0] == 0){
			$main .= ((we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW, we_base_perm::NEWSLETTER_EDIT])) ? we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:we_save()") : '');
			$optional .= ((we_base_permission::hasPerm(we_base_perm::NEWSLETTER_DELETE)) ? we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:top.content.we_cmd('delete_newsletter')") : '');

			if(!$group){
				$main .= $select->getHtml() .
					we_html_forms::checkbox(0, false, "htmlmail_check", g_l('modules_newsletter', '[html_preview]'), false, '', "if(document.we_form.htmlmail_check.checked) { document.we_form.hm.value=1;top.opener.top.nlHTMLMail=true; } else { document.we_form.hm.value=0;top.opener.top.nlHTMLMail=false; }") .
					we_html_button::create_button(we_html_button::PREVIEW, "javascript:we_cmd('popPreview')") .
					(we_base_permission::hasPerm(we_base_perm::NEWSLETTER_SEND) ? we_html_button::create_button(we_html_button::SEND, "javascript:we_cmd('popSend')") : "");
			}
		}

		$body = we_html_element::body(['id' => "footerBody", 'onload' => "afterLoad();"], we_html_element::form([], we_html_element::hidden('hm', 0) .
					we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], $main) . we_html_element::div(['class' => 'optional'], $optional))
				)
		);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/frames_footer.js'));
	}

	private function getHTMLLog(){
		$start = we_base_request::_(we_base_request::INT, 'newsletterStartTime', 0);
		$end = we_base_request::_(we_base_request::INT, 'newsletterEndTime', 0);
		$status = we_base_request::_(we_base_request::INT, 'newsletterStatus');

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => 'we_form', 'method' => 'post'], we_html_tools::dialogLayout(
							we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;margin:5px 10px 15px 10px;"], $this->View->getHTMLLog($status, $start, $end))
							, g_l('modules_newsletter', '[show_log]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();")
						)
					)
		));
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::INT, 'pid')) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_tree_newsletter::getItems($pid)]);

		return $this->getHTMLDocument(we_html_element::body([], we_html_element::form(['name' => 'we_form'], we_html_element::hiddens([
							'mod' => 'newsletter',
							'pnt' => "cmd",
							"ncmd" => '',
							"nopt" => ''])
					)
		));
	}

	private function getHTMLPrintLists(){
		$emails = [];
		$out = '<ul>';
		$count = count($this->View->Model->groups) + 1;

		$c = 0;
		for($k = 1; $k < $count; $k++){
			$out .= '<li>' . we_html_element::b(sprintf(g_l('modules_newsletter', '[mailing_list]'), $k)) . '<ul>';
			$gc = 0;
			if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
				$out .= '<li>' . g_l('modules_newsletter', '[customers]') . '<ul>';
				$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_CUSTOMER, 1);

				foreach($emails as $email){
					$gc++;
					$out .= '<li>' . $email . '</li>';
				}
				$out .= '</ul>';
			}
			$out .= '</ul>
<li>' . g_l('modules_newsletter', '[emails]') . '<ul>';

			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_EMAILS, 1);
			foreach($emails as $email){
				$gc++;
				$out .= '<li>' . $email . '</li>';
			}

			$out .= '</ul><li>' . g_l('modules_newsletter', '[file_email]') . '<ul>';

			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_FILE, 1);
			foreach($emails as $email){
				$gc++;
				$out .= '<li>' . $email . '</li>';
			}

			$c += $gc;
			$out .= '</ul></ul><li>' . we_html_element::b(sprintf(g_l('modules_newsletter', '[sum_group]'), $k) . ':' . $gc) . '</li>';
		}

		$out .= '<li>' . we_html_element::b(g_l('modules_newsletter', '[sum_all]') . ":" . $c) . '</ul>';
		echo self::getHTMLDocument(we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => 'we_form', 'method' => 'post', 'onload' => "self.focus()"], we_html_tools::dialogLayout(
						we_html_element::br() .
						we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;"], $out) .
						we_html_element::br(), g_l('modules_newsletter', '[lists_overview]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();")
					)
		)));
		flush();
	}

	private function getHTMLDCheck(){
		$emails = [];
		$domain = '';
		$count = count($this->View->Model->groups) + 1;

		$out = '<ul>' .
			'<li>' . we_html_element::b(g_l('modules_newsletter', '[domain_check_begins]')) . '<ul>';

		for($k = 1; $k < $count; $k++){
			$out .= '<li>' . sprintf(g_l('modules_newsletter', '[domain_check_list]'), $k) . '<ul>';
			$emails = $this->View->getEmails($k, we_newsletter_view::MAILS_ALL, 1);

			foreach($emails as $email){
				if($this->View->Model->check_email($email)){
					if(!$this->View->Model->check_domain($email, $domain)){
						$out .= '<li>' . sprintf(g_l('modules_newsletter', '[domain_nok]'), $domain) . '</li>';
					}
				} else {
					$out .= '<li>' . sprintf(g_l('modules_newsletter', '[email_malformed]'), $email) . '</li>';
				}
			}
			$out .= '</ul>';
		}
		$out .= '</ul>
<li>' . we_html_element::b(g_l('modules_newsletter', '[domain_check_ends]')) . '</li></ul>';
		echo self::getHTMLDocument(we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => 'we_form', 'method' => 'post', 'onload' => "self.focus()"], we_html_tools::dialogLayout(
						we_html_element::br() .
						we_html_element::div(['class' => "blockWrapper", 'style' => "width: 588px; height: 500px; border:1px #dce6f2 solid;"], $out) .
						we_html_element::br(), g_l('modules_newsletter', '[lists_overview]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();")
					)
		)));
		flush();
	}

	private function getHTMLSettings(){
		$settings = we_newsletter_view::getSettings();

		$closeflag = false;

		if(we_base_request::_(we_base_request::STRING, "ncmd") === 'save_settings'){
			$this->View->processCommands($this->jsCmd);
			$closeflag = true;
			$this->jsCmd->addCmd('settings_close');
		}

		$js = $this->View->getJSProperty();

		$texts = ['send_step', 'send_wait', 'test_account', 'default_sender', 'default_reply', we_newsletter_newsletter::FEMALE_SALUTATION_FIELD, we_newsletter_newsletter::MALE_SALUTATION_FIELD];
		$radios = ['reject_malformed', 'reject_not_verified', 'reject_save_malformed', 'log_sending', 'default_htmlmail', 'isEmbedImages', 'title_or_salutation',
			'use_base_href', 'use_https_refer', 'use_port'];
		$extra_radio_text = ['use_port'];
		$defaults = ['reject_save_malformed' => 1, 'use_https_refer' => 0, 'send_wait' => 0, 'use_port' => 0, 'use_port_check' => 80, 'isEmbedImages' => 0, 'use_base_href' => 1];

		$table = new we_html_table(['class' => 'default withSpace', 'style' => 'margin-bottom:10px'], 1, 2);
		$c = 0;

		foreach($texts as $text){
			if(!isset($settings[$text])){
				$this->View->saveSetting($text, (isset($defaults[$text]) ? $defaults[$text] : 0), $this->db);
				$settings = we_newsletter_view::getSettings();
			}

			$table->setColContent($c, 0, g_l('modules_newsletter', '[' . $text . ']') . ":&nbsp;");
			$table->setColContent($c, 1, we_html_tools::textInput($text, $settings[$text], "", [], "text", 308));

			$c++;
			$table->addRow();
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$custfields = [];

			foreach($this->View->customers_fields as $fv){
				$custfields[$fv] = $fv;
			}

			$table->setColContent($c, 0, g_l('modules_newsletter', '[customer_email_field]') . ":&nbsp;");
			$table->setColContent($c, 1, we_html_tools::select("customer_email_field", $custfields, 1, $settings['customer_email_field'], false, [
					], true, 308));

			$table->setColContent(++$c, 0, g_l('modules_newsletter', '[customer_html_field]') . ':&nbsp;');
			$table->setColContent($c, 1, we_html_tools::select('customer_html_field', $custfields, 1, $settings['customer_html_field'], false, [
					], true, 308));

			$table->setColContent(++$c, 0, g_l('modules_newsletter', '[customer_salutation_field]') . ':&nbsp;');
			$table->setColContent($c, 1, we_html_tools::select('customer_salutation_field', $custfields, 1, $settings['customer_salutation_field'], false, [
					], true, 308));

			$table->setColContent(++$c, 0, g_l('modules_newsletter', '[customer_title_field]') . ':&nbsp;');
			$table->setColContent($c, 1, we_html_tools::select('customer_title_field', $custfields, 1, $settings['customer_title_field'], false, [
					], true, 308));

			$table->setColContent(++$c, 0, g_l('modules_newsletter', '[customer_firstname_field]') . ':&nbsp;');
			$table->setColContent($c, 1, we_html_tools::select('customer_firstname_field', $custfields, 1, $settings['customer_firstname_field'], false, [
					], true, 308));

			$table->setColContent(++$c, 0, g_l('modules_newsletter', '[customer_lastname_field]') . ':&nbsp;');
			$table->setColContent($c, 1, we_html_tools::select('customer_lastname_field', $custfields, 1, $settings['customer_lastname_field'], false, [
					], true, 308));
		}

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:we_cmd('settings_close')");
		$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_settings')");

		$radios_code = '';
		foreach($radios as $radio){
			if(!isset($settings[$radio])){

				$this->View->saveSetting($radio, (isset($defaults[$radio]) ? $defaults[$radio] : 1), $this->db);
				$settings = we_newsletter_view::getSettings();
			}
			if(in_array($radio, $extra_radio_text)){
				$radios_code .= we_html_forms::checkbox($settings[$radio], (($settings[$radio] > 0) ? true : false), $radio . '_check', g_l('modules_newsletter', '[' . $radio . "_check]"), false, '', "if(document.we_form." . $radio . "_check.checked){document.we_form." . $radio . ".value=" . (isset($defaults[$radio . "_check"]) ? $defaults[$radio . "_check"] : 0) . ";}else{document.we_form." . $radio . ".value=0;}");

				$radio_table = new we_html_table(['class' => 'default', 'style' => 'margin-left:25px;'], 1, 2);
				$radio_table->setColContent(0, 0, htmlspecialchars(g_l('modules_newsletter', '[' . $radio . ']')) . ':&nbsp;');
				$radio_table->setCol(0, 1, ['style' => 'padding-left:5px;'], we_html_tools::textInput($radio, $settings[$radio], '', ['onchange' => "if(document.we_form." . $radio . ".value!=0){document.we_form." . $radio . "_check.checked=true;}else{document.we_form." . $radio . "_check.checked=false;}"]));
				$radios_code .= $radio_table->getHtml();
			} else {
				$radios_code .= we_html_forms::checkbox($settings[$radio], (($settings[$radio] == 1) ? true : false), $radio, htmlspecialchars(g_l('modules_newsletter', '[' . $radio . ']')), false, '', "if(document.we_form." . $radio . ".checked){document.we_form." . $radio . ".value=1;}else{document.we_form." . $radio . ".value=0;}") . '<br/>';
			}
		}

		$deselect = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.global_mailing_list.value=''");

		$gml_table = new we_html_table(['class' => 'default withSpace', 'style' => 'width:538px;margin:10px;'], 4, 2);
		$gml_table->setColContent(0, 0, g_l('modules_newsletter', '[global_mailing_list]'));
		$gml_table->setColContent(2, 0, $this->formFileChooser('global_mailing_list', $settings['global_mailing_list']));
		$gml_table->setCol(2, 1, ['style' => 'text-align:right'], $deselect);

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => 'we_form'], $this->View->getHiddens() .
					we_html_tools::dialogLayout(
						$table->getHtml() .
						$radios_code .
						$gml_table->getHtml(), g_l('modules_newsletter', '[settings]'), we_html_button::position_yes_no_cancel($save, $close)
					)
				)
		);

		return $this->getHTMLDocument($body, $js);
	}

	function getHTMLBlockType($name, $selected = 1){
		$values = [we_newsletter_block::DOCUMENT => g_l('modules_newsletter', '[newsletter_type_0]'),
			we_newsletter_block::DOCUMENT_FIELD => g_l('modules_newsletter', '[newsletter_type_1]'),
		];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$values[we_newsletter_block::OBJECT] = g_l('modules_newsletter', '[newsletter_type_2]');
			$values[we_newsletter_block::OBJECT_FIELD] = g_l('modules_newsletter', '[newsletter_type_3]');
		}

		if(we_base_permission::hasPerm(we_base_perm::NEWSLETTER_FILES)){
			$values[we_newsletter_block::FILE] = g_l('modules_newsletter', '[newsletter_type_4]');
		}
		$values[we_newsletter_block::TEXT] = g_l('modules_newsletter', '[newsletter_type_5]');
		$values[we_newsletter_block::ATTACHMENT] = g_l('modules_newsletter', '[newsletter_type_6]');
		$values[we_newsletter_block::URL] = g_l('modules_newsletter', '[newsletter_type_7]');

		return we_html_tools::select($name, $values, 1, $selected, false, ['style' => "width:440px;", 'onchange' => "we_cmd('switchPage','',2);"], true, 315);
	}

	function getHTMLCopy(){
		return we_html_element::hiddens([
				'copyid' => 0,
				'copyid_text' => '']) .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file',document.we_form.elements.copyid.value,WE().consts.tables.NEWSLETTER_TABLE,'copyid','copyid_text','copy_newsletter','','" . get_ws(NEWSLETTER_TABLE) . "')");
	}

	private function getHTMLCustomer($group){
		$out = we_html_forms::checkbox($this->View->Model->groups[$group]->SendAll, (($this->View->Model->groups[$group]->SendAll == 0) ? false : true), 'sendallcheck_' . $group, g_l('modules_newsletter', '[send_all]'), false, '', "we_cmd('switch_sendall',$group);");
		$out .= '(' . g_l('modules_newsletter', '[reporting][mailing_all_emails]') . ' ' . count($this->View->getEmails(($group + 1), we_newsletter_view::MAILS_ALL, 1)) . ')';

		if($this->View->Model->groups[$group]->SendAll == 0){

			$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_customers'," . $group . ")");
			$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_customer_selector','',WE().consts.tables.CUSTOMER_TABLE,'','','add_customer," . $group . "','','','',1)");

			$cats = new we_chooser_multiDir(self::def_width, $this->View->Model->groups[$group]->Customers, 'del_customer', $delallbut . $addbut, '', '"' . we_base_ContentTypes::CUSTOMER . '"', CUSTOMER_TABLE);
			$cats->extraDelFn = "document.we_form.ngroup.value=$group";
			$out .= $cats->get($this->jsCmd);
		}

		$out .= $this->getHTMLCustomerFilter($group);

		return $out;
	}

	private function getHTMLExtern($group){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_files'," . $group . ")");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('browse_server','fileselect',WE().consts.contentTypes.TEXT,'/','add_file,top.currentID," . $group . "','',1);");

		$buttons = $delallbut . (we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT)) ? $addbut : '';

		$cats = new we_chooser_multiFile(self::def_width, $this->View->Model->groups[$group]->Extern, 'del_file', $buttons, 'edit_file');

		$cats->extraDelFn = 'document.we_form.ngroup.value=' . $group;
		return we_html_element::hiddens(['fileselect' => '']) .
			$cats->get($this->jsCmd);
	}

	private function getHTMLCustomerFilter($group){
		$custfields = [];
		foreach($this->View->customers_fields as $fv){
			switch($fv){
				case 'ParentID':
				case 'IsFolder':
				case 'Path':
				case 'Text' :
					continue 2;
				default:
					$custfields[$fv] = $fv;
			}
		}

		$operators = [
			we_newsletter_newsletter::OP_EQ => '=',
			we_newsletter_newsletter::OP_NEQ => '<>',
			we_newsletter_newsletter::OP_LE => '<',
			we_newsletter_newsletter::OP_LEQ => '<=',
			we_newsletter_newsletter::OP_GE => '>',
			we_newsletter_newsletter::OP_GEQ => '>=',
			we_newsletter_newsletter::OP_CONTAINS => g_l('modules_newsletter', '[operator][contains]'),
			we_newsletter_newsletter::OP_STARTS => g_l('modules_newsletter', '[operator][startWith]'),
			we_newsletter_newsletter::OP_ENDS => g_l('modules_newsletter', '[operator][endsWith]'),
			we_newsletter_newsletter::OP_LIKE => 'LIKE',
		];
		$logic = ['AND' => g_l('modules_newsletter', '[logic][and]'), 'OR' => g_l('modules_newsletter', '[logic][or]')];
		$hours = [];
		for($i = 0; $i < 24; $i++){
			$hours[] = ($i <= 9 ? '0' : '') . $i;
		}
		$minutes = [];
		for($i = 0; $i < 60; $i++){
			$minutes[] = ($i <= 9 ? '0' : '') . $i;
		}

		$filter = $this->View->Model->groups[$group]->getFilter();

		$colspan = 6;
		$table = new we_html_table(['class' => 'default'], 1, $colspan);
		$table->setCol(0, 0, (($filter) ? ['colspan' => $colspan] : []), we_html_forms::checkbox(($filter ? 1 : 0), ($filter ? true : false), "filtercheck_$group", g_l('modules_newsletter', '[filter]'), false, '', "if(document.we_form.filtercheck_" . $group . ".checked){ we_cmd('add_filter'," . $group . ",0); }else{ we_cmd('del_all_filters'," . $group . ");}"));

		$k = 0;
		$c = 1;
		if($filter){
			foreach($filter as $k => $v){
				if($k != 0){
					$table->addRow();
					$table->setCol($c, 0, ['colspan' => $colspan], we_html_tools::select('filter_logic_' . $group . '_' . $k, $logic, 1, $v['logic'], false, [], true, 70));
					$c++;
				}

				$table->addRow();
				$table->setColContent($c, 0, we_html_tools::select('filter_fieldname_' . $group . '_' . $k, $custfields, 1, $v['fieldname'], false, ['onchange' => 'top.content.hot=true;changeFieldValue(this.val,\'filter_fieldvalue_' . $group . '_' . $k . '\');'], true, 170));
				$table->setColContent($c, 1, we_html_tools::select('filter_operator_' . $group . '_' . $k, $operators, 1, $v['operator'], false, ['onchange' => "top.content.hot=true;"], true, 80));
				switch($v['fieldname']){
					case 'MemberSince':
					case 'LastLogin':
					case 'LastAccess':
						$table->setCol($c, 2, ['id' => 'td_value_fields_' . $group . '_' . $k], we_html_tools::getDateSelector('filter_fieldvalue_' . $group . '_' . $k, '_from_' . $group . '_' . $k, !empty($v['fieldvalue']) ? !stristr($v['fieldvalue'], '.') ? date('d.m.Y', $v['fieldvalue']) : $v['fieldvalue'] : ''));
						$table->setColContent($c, 3, we_html_tools::select('filter_hours_' . $group . '_' . $k, $hours, 1, isset($v['hours']) ? $v['hours'] : "", false, ['onchange' => 'top.content.hot=true;']) . '&nbsp;h :');

						$table->setColContent($c, 4, we_html_tools::select('filter_minutes_' . $group . '_' . $k, $minutes, 1, isset($v['minutes']) ? $v['minutes'] : '', false, [
								'onchange' => "top.content.hot=true;"]) . '&nbsp;m');
						break;
					default:
						$table->setCol($c, 2, ['colspan' => 3, 'id' => 'td_value_fields_' . $group . '_' . $k], we_html_tools::textInput('filter_fieldvalue_' . $group . '_' . $k, isset($v['fieldvalue']) ? $v['fieldvalue'] : '', '', [
								'onkeyup' => "top.content.hot=true;"], 'text', 200));
				}
				$table->setColContent($c, 5, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_filter'," . $group . "," . ($k + 1) . ")") .
					we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_filter'," . $group . "," . $k . ")"));
				$c++;
			}
		}
		return we_html_element::hiddens(['filter_' . $group => count($filter)]) .
			$table->getHtml();
	}

	private function htmlSelectEmailList($name, $values, $size = 1, $selectedIndex = '', $multiple = false, $attribs = '', $compare = 'value', $width = '', $cls = 'defaultfont'){
		reset($values);
		$ret = '<select class="' . $cls . '" name="' . trim($name) . '" size=' . abs($size) . ' ' . ($multiple ? " multiple" : "") . ($attribs ? " $attribs" : "") . ($width ? ' style="width: ' . $width . 'px"' : '') . '>';
		$selIndex = makeArrayFromCSV($selectedIndex);
		foreach($values as $value => $text){
			$ret .= '<option value="' . oldHtmlspecialchars($value) . '"' . (in_array((($compare === 'value') ? $value : $text), $selIndex) ? ' selected="selected"' : '') . (we_check_email($text) ? ' class="markValid"' : ' class="markNotValid"') . '>' . $text . "</option>";
		}
		$ret .= '</select>';
		return $ret;
	}

	/**
	 * Mailing list - block Emails
	 *
	 *
	 * @param integer $group
	 * @return string
	 */
	function getHTMLEmails($group){
		$arr = we_newsletter_base::getEmailsFromList(oldHtmlspecialchars($this->View->Model->groups[$group]->Emails), 1);
		// Buttons to handle the emails in  the email list
		$buttons_table = new we_html_table(['class' => 'default withSpace'], 4, 1);
		$buttons_table->setColContent(0, 0, we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('add_email', " . $group . ");"));
		$buttons_table->setColContent(1, 0, we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('edit_email', " . $group . ");"));
		$buttons_table->setColContent(2, 0, we_html_button::create_button(we_html_button::DELETE, "javascript:deleteit(" . $group . ")"));
		$buttons_table->setColContent(3, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:deleteall(" . $group . ")"));

		// Dialog table for the email block
		$table = new we_html_table(['class' => 'default withSpace'], 3, 3);

		// 1. ROW: select status
		$selectStatus = we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
				we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
				we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
				we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
				], '', we_base_request::_(we_base_request::INT, 'weEmailStatus', 0), '', [
				'onchange' => "weShowMailsByStatus(this.value, $group);",
				'id' => 'weViewByStatus'], true, 150);
		$table->setCol(0, 0, ['style' => 'vertical-align:middle;', 'colspan' => 3,], $selectStatus);

		// 2. ROW: Mail list with handling buttons
		$table->setCol(1, 0, ['style' => 'vertical-align:top;'], $this->htmlSelectEmailList('we_recipient' . $group, $arr, 10, '', false, 'style="width:' . (self::def_width - 110) . 'px; height:140px" id="we_recipient' . $group . '"', 'value', 600));
		$table->setCol(1, 1, ['style' => 'vertical-align:middle;width:10px;']);
		$table->setCol(1, 2, ['style' => 'vertical-align:top;'], $buttons_table->getHtml());

		// 3. ROW: Buttons for email import and export
		$importbut = we_html_button::create_button(we_html_button::IMPORT, "javascript:we_cmd('set_import'," . $group . ")");
		$exportbut = we_html_button::create_button(we_html_button::EXPORT, "javascript:we_cmd('set_export'," . $group . ")");

		$table->setCol(2, 0, ['colspan' => 3], $importbut . $exportbut);

		// Import dialog
		if($this->View->getShowImportBox() == $group){
			$ok = we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('import_csv')");
			$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('reset_import');");

			$import_options = new we_html_table(['class' => 'default withSpace'], 7, 3);

			$import_options->setColContent(0, 0, g_l('modules_newsletter', '[csv_delimiter]') . ":&nbsp;");
			$import_options->setColContent(0, 1, we_html_tools::textInput("csv_delimiter" . $group, ","));
			$import_options->setColContent(1, 0, g_l('modules_newsletter', '[csv_col]') . ":&nbsp;");
			$import_options->setColContent(1, 1, we_html_tools::textInput("csv_col" . $group, 1));
			$import_options->setColContent(2, 0, g_l('modules_newsletter', '[csv_hmcol]') . ":&nbsp;");
			$import_options->setColContent(2, 1, we_html_tools::textInput("csv_hmcol" . $group, 2));
			$import_options->setCol(2, 2, ['class' => "lowContrast"], "&nbsp;" . g_l('modules_newsletter', '[csv_html_explain]'));
			$import_options->setColContent(3, 0, g_l('modules_newsletter', '[csv_salutationcol]') . ":&nbsp;");
			$import_options->setColContent(3, 1, we_html_tools::textInput("csv_salutationcol" . $group, 3));
			$import_options->setCol(3, 2, ['class' => "lowContrast"], "&nbsp;" . g_l('modules_newsletter', '[csv_salutation_explain]'));
			$import_options->setColContent(4, 0, g_l('modules_newsletter', '[csv_titlecol]') . ":&nbsp;");
			$import_options->setColContent(4, 1, we_html_tools::textInput("csv_titlecol" . $group, 4));
			$import_options->setCol(4, 2, ['class' => 'lowContrast'], "&nbsp;" . g_l('modules_newsletter', '[csv_title_explain]'));
			$import_options->setColContent(5, 0, g_l('modules_newsletter', '[csv_firstnamecol]') . ":&nbsp;");
			$import_options->setColContent(5, 1, we_html_tools::textInput("csv_firstnamecol" . $group, 5));
			$import_options->setCol(5, 2, ['class' => "lowContrast"], "&nbsp;" . g_l('modules_newsletter', '[csv_firstname_explain]'));
			$import_options->setColContent(6, 0, g_l('modules_newsletter', '[csv_lastnamecol]') . ":&nbsp;");
			$import_options->setColContent(6, 1, we_html_tools::textInput("csv_lastnamecol" . $group, 6));
			$import_options->setCol(6, 2, ['class' => "lowContrast"], "&nbsp;" . g_l('modules_newsletter', '[csv_lastname_explain]'));


			$import_box = new we_html_table(['class' => 'default withSpace', 'style' => 'margin-top:10px;'], 4, 1);
			$import_box->setColContent(0, 0, $this->formFileChooser('csv_file' . $group, '/', ''));
			$import_box->setColContent(1, 0, we_html_button::create_button(we_html_button::UPLOAD, "javascript:we_cmd('upload_csv',$group)"));
			$import_box->setColContent(2, 0, $import_options->getHtml());
			$import_box->setColContent(3, 0, $ok . $cancel);

			$table->setCol(5, 0, ['colspan' => 3], we_html_element::hiddens(['csv_import' => $group]) . $import_box->getHtml());
		}

		// Export dialog
		if($this->View->getShowExportBox() == $group){
			$ok = we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('export_csv')");
			$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('reset_import');");

			$export_box = new we_html_table(['class' => 'default withSpace', 'style' => 'margin-top:10px;'], 2, 1);

			$export_box->setColContent(1, 0, $this->formFileChooser('csv_dir' . $group, '/', '', we_base_ContentTypes::FOLDER));
			$export_box->setColContent(2, 0, $ok . $cancel);

			$table->setCol(5, 0, ['colspan' => 3], we_html_element::hiddens(['csv_export' => $group]) . $export_box->getHtml());
		}

		return $table->getHtml();
	}

	private function formWeChooser($table = FILE_TABLE, $width = '', $rootDirID = 0, $IDName = 'ID', $IDValue = 0, $Pathname = 'Path', $Pathvalue = '/', $cmd = '', $open_doc = '', $acObject = null, $contentType = ''){
		$Pathvalue = $Pathvalue ?: f('SELECT Path FROM ' . $this->db->escape($table) . ' WHERE ID=' . intval($IDValue), '', $this->db);

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $IDName . "'].value,'" . $table . "','" . $IDName . "','" . $Pathname . "','" . $cmd . "','','" . $rootDirID . "','','" . $open_doc . "')");
		if(is_object($acObject)){

			$weSuggest = $acObject;
			$weSuggest->setAcId($IDName);
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::TEMPLATE]);
			$weSuggest->setInput($Pathname, $Pathvalue);
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult($IDName, $IDValue);
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setTable($table);
			$weSuggest->setWidth($width);
			$weSuggest->setSelectButton($button);
			return $weSuggest->getHTML();
		}
		return we_html_tools::formElementTable('', we_html_tools::textInput($Pathname, $Pathvalue, '', ['onchange' => 'top.content.hot=true;', 'readonly' => 'readonly'], 'text', $width, 0), we_html_element::hidden(trim($IDName), oldHtmlspecialchars($IDValue)), $button);
	}

	private function formWeDocChooser($table = FILE_TABLE, $width = '', $rootDirID = 0, $IDName = 'ID', $IDValue = 0, $Pathname = 'Path', $Pathvalue = '/', $cmd = '', $filter = we_base_ContentTypes::WEDOCUMENT, $acObject = null){
		$Pathvalue = $Pathvalue ?: f('SELECT Path FROM ' . $this->db->escape($table) . ' WHERE ID=' . intval($IDValue));

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $IDName . "'].value,'" . $table . "','" . $IDName . "','" . $Pathname . "','" . $cmd . "','','" . $rootDirID . "','" . $filter . "'," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ")");
		if(is_object($acObject)){
			$weSuggest = $acObject;
			$weSuggest->setAcId($IDName);
			$weSuggest->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER]));
			$weSuggest->setInput($Pathname, $Pathvalue);
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult($IDName, $IDValue);
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setTable($table);
			$weSuggest->setWidth($width);
			$weSuggest->setSelectButton($button);
			return $weSuggest->getHTML();
		}
		return we_html_tools::formElementTable('', we_html_tools::textInput($Pathname, $Pathvalue, "", ['onchange' => 'top.content.hot=true;', 'readonly' => 'readonly'], "text", $width, 0), we_html_element::hidden(trim($IDName), oldHtmlspecialchars($IDValue)), $button);
	}

	private function getHTMLNewsletterBlocks(){
		$counter = 0;

		$parts = [[
			'headline' => '',
			'html' => we_html_element::hiddens(['blocks' => count($this->View->Model->blocks)]),
			'space' => we_html_multiIconBox::SPACE_MED2,
			'noline' => true]
		];

		foreach($this->View->Model->blocks as $block){
			$content = we_html_tools::formElementTable(g_l('modules_newsletter', '[name]'), $this->getHTMLBlockType('block' . $counter . '_Type', $block->Type));

			$values = [];
			$count = count($this->View->Model->groups) + 1;

			for($i = 1; $i < $count; $i++){
				$values[$i] = sprintf(g_l('modules_newsletter', '[mailing_list]'), $i);
			}

			$selected = $block->Groups ?: "1";
			$content .= we_html_element::hiddens(['block' . $counter . '_Groups' => $selected,
					'block' . $counter . '_Pack' => $block->Pack]) .
				we_html_tools::formElementTable(g_l('modules_newsletter', '[block_lists]'), we_html_tools::select('block' . $counter . '_GroupsSel', $values, 1, $selected, true, [
						'class' => 'newSelect', 'style' => 'width:440px;',
						'onchange' => "populateMultipleVar(document.we_form.block" . $counter . "_GroupsSel,document.we_form.block" . $counter . "_Groups);top.content.hot=true"]));

			switch($block->Type){
				case we_newsletter_block::DOCUMENT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document]'), $this->formWeDocChooser(FILE_TABLE, 320, 0, "block" . $counter . "_LinkID", $block->LinkID, "block" . $counter . "_LinkPath", '', 'setHot', we_base_ContentTypes::WEDOCUMENT, $this->weAutoCompleter)) .
						we_html_tools::formElementTable('', we_html_forms::checkbox((($block->Field) ? 0 : 1), (($block->Field) ? false : true), "block" . $counter . "_use_def_template", g_l('modules_newsletter', '[use_default]'), false, '', "top.content.hot=true;if(document.we_form.block" . $counter . "_use_def_template.checked){ document.we_form.block" . $counter . "_Field.value=0; document.we_form.block" . $counter . "_FieldPath.value='';}")) .
						we_html_tools::formElementTable(g_l('modules_newsletter', '[block_template]'), $this->formWeChooser(TEMPLATES_TABLE, 320, 0, 'block' . $counter . '_Field', (!is_numeric($block->Field) ? 0 : $block->Field), 'block' . $counter . "_FieldPath", '', 'blocks_selectTemplateCallback,' . $counter, '', $this->weAutoCompleter, 'folder,' . we_base_ContentTypes::TEMPLATE));
					break;

				case we_newsletter_block::DOCUMENT_FIELD:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document]'), $this->formWeChooser(FILE_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->LinkID, "block" . $counter . "_LinkPath", "", 'switchPage,2,setHot', "", $this->weAutoCompleter, "folder," . we_base_ContentTypes::WEDOCUMENT));

					if($block->LinkID){
						$values = $this->View->getFields($block->LinkID, FILE_TABLE);

						$content .= (!empty($values) ?
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_tools::select('block' . $counter . '_Field', $values, 1, $block->Field, '', [
									'style' => 'width:440px;',
									'onkeyup' => 'top.content.hot=true;'])) :
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_element::div(['class' => "lowContrast"], g_l('modules_newsletter', '[none]')))
							);
					}
					break;

				case we_newsletter_block::OBJECT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object]'), $this->formWeChooser(OBJECT_FILES_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->LinkID, "block" . $counter . "_LinkPath", '', 'setHot', (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1), $this->weAutoCompleter, we_base_ContentTypes::FOLDER . "," . we_base_ContentTypes::OBJECT_FILE)) .
						we_html_tools::formElementTable(g_l('modules_newsletter', '[block_template]'), $this->formWeChooser(TEMPLATES_TABLE, 320, 0, 'block' . $counter . "_Field", (!is_numeric($block->Field) ? 0 : $block->Field), "block" . $counter . "_FieldPath", '', 'setHot', '', $this->weAutoCompleter, 'folder,' . we_base_ContentTypes::TEMPLATE));
					break;

				case we_newsletter_block::OBJECT_FIELD:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object]'), $this->formWeChooser(OBJECT_FILES_TABLE, 320, 0, "block" . $counter . "_LinkID", $block->LinkID, "block" . $counter . "_LinkPath", "", 'switchPage,2,setHot', (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1), $this->weAutoCompleter, we_base_ContentTypes::FOLDER . "," . we_base_ContentTypes::OBJECT_FILE));

					if($block->LinkID){
						$values = $this->View->getFields($block->LinkID, OBJECT_FILES_TABLE);

						$content .= (!empty($values) ?
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_object_field]'), we_html_tools::select('block' . $counter . '_Field', $values, 1, $block->Field, false, [
									'OnChange' => "top.content.hot=true;"])) :
							we_html_tools::formElementTable(g_l('modules_newsletter', '[block_document_field]'), we_html_element::div(['class' => "lowContrast"], g_l('modules_newsletter', '[none]')))
							);
					}
					break;

				case we_newsletter_block::FILE:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_file]'), $this->formFileChooser('block' . $counter . '_Field', (is_numeric($block->Field) ? '' : ((substr($block->Field, 0, 1) != '/') ? '' : $block->Field))));
					break;

				case we_newsletter_block::TEXT:
					$attribs = ["wysiwyg" => "on",
						'width' => 430,
						'height' => 400,
						"rows" => 10,
						"cols" => 40,
						'style' => "width:440px;",
						"inlineedit" => "true",
						"bgcolor" => "white",
					];
					$blockHtml = preg_replace(
						['/(href=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/',
							'/(src=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/'
						], '${1}${3}${5}', stripslashes($block->Html));


					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_plain]'), we_html_element::textArea(['cols' => 40, "rows" => 10, 'name' => "block" . $counter . "_Source",
								'onchange' => "top.content.hot=true;",
								'style' => "width:440px;"], oldHtmlspecialchars($block->Source))) .
						we_html_tools::formElementTable(g_l('modules_newsletter', '[block_html]'), we_html_forms::weTextarea('block' . $counter . '_Html', $blockHtml, $attribs, false, '', true, true, false, true, $this->View->Model->Charset));
					break;

				case we_newsletter_block::ATTACHMENT:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_attachment]'), $this->formWeChooser(FILE_TABLE, 320, 0, 'block' . $counter . '_LinkID', $block->LinkID, 'block' . $counter . '_LinkPath', '', '', '', $this->weAutoCompleter, implode(',', array_merge(we_base_ContentTypes::LINK_ABLE, [
								we_base_ContentTypes::FOLDER]))));
					break;

				case we_newsletter_block::URL:
					$content .= we_html_tools::formElementTable(g_l('modules_newsletter', '[block_url]'), we_html_tools::textInput('block' . $counter . '_Field', (is_numeric($block->Field) ? '' : $block->Field), '', [
								'style' => 'width:440px;'], 'text', 0, 0, 'top.content'));
					break;
			}

			$buttons = '<div style="margin-left:440px;">' .
				we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addBlock','" . $counter . "')") .
				(count($this->View->Model->blocks) > 1 ?
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('delBlock','" . $counter . "')") :
				''
				) . '</div>';

			$parts[] = ['headline' => sprintf(g_l('modules_newsletter', '[block]'), ($counter + 1)), 'html' => $content, 'space' => we_html_multiIconBox::SPACE_MED];
			$parts[] = ['headline' => '', 'html' => $buttons, 'space' => we_html_multiIconBox::SPACE_MED];

			$counter++;
		}

		return we_html_multiIconBox::getHTML("newsletter_header", $parts, true);
	}

	private function getHTMLNewsletterGroups(){
		$count = count($this->View->Model->groups);
		$out = we_html_element::jsScript(JS_DIR . 'multiIconBox.js');

		for($i = 0; $i < $count; $i++){
			$parts = [we_modules_info::isActive(we_modules_info::CUSTOMER) ? [
				'headline' => g_l('modules_newsletter', '[customers]'),
				'html' => $this->getHTMLCustomer($i),
				'icon' => we_html_multiIconBox::PROP_CUSTOMER
				] : null,
				[
					'headline' => g_l('modules_newsletter', '[file_email]'),
					'html' => $this->getHTMLExtern($i),
					'icon' => we_html_multiIconBox::PROP_DOC
				],
				['headline' => g_l('modules_newsletter', '[emails]'),
					'html' => $this->getHTMLEmails($i),
					'icon' => we_html_multiIconBox::PROP_MAIL
				]
			];


			$buttons = ($i == $count - 1 ? we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addGroup')") : null) .
				($count > 1 ? we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('delGroup'," . $i . ")") : null);

			$out .= we_html_multiIconBox::getHTML('newsletter_group_box_' . $i, $parts, true, "", 0, "", "", ($count < 2), sprintf(g_l('modules_newsletter', '[mailing_list]'), ($i + 1))) .
				we_html_element::br() .
				'<div style="margin-right:30px;">' . $buttons . '</div>';
		}

		return $out;
	}

	private function formNewsletterDirChooser($width = '', $rootDirID = 0, $IDName = 'ID', $IDValue = 0, $Pathname = 'Path', $Pathvalue = '/', $cmd = '', $acObject = null){
		if(!$Pathvalue){
			$Pathvalue = f('SELECT Path FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . intval($IDValue), '', $this->db);
		}

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_newsletter_dirSelector',document.we_form.elements['" . $IDName . "'].value,'" . $IDName . "','" . $Pathname . "','" . $cmd . "','','" . $rootDirID . "')");
		if(is_object($acObject)){
			$weSuggest = $acObject;
			$weSuggest->setAcId('PathGroup');
			$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
			$weSuggest->setInput($Pathname, str_replace('\\', '/', $Pathvalue));
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult($IDName, $IDValue);
			$weSuggest->setSelector(we_gui_suggest::DirSelector);
			$weSuggest->setTable(NEWSLETTER_TABLE);
			$weSuggest->setWidth($width);
			$weSuggest->setSelectButton($button);

			return $weSuggest->getHTML();
		}
		return we_html_tools::formElementTable('', we_html_tools::textInput($Pathname, $Pathvalue, '', ['readonly' => "readonly", 'id' => "yuiAcInputPathGroup"], 'text', $width, 0), we_html_element::hidden(trim($IDName), oldHtmlspecialchars($IDValue)), $button
		);
	}

	function getHTMLNewsletterHeader(){
		$table = new we_html_table(['class' => 'default withSpace'], 2, 1);
		$table->setColContent(0, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[name]'), we_html_tools::textInput('Text', stripslashes($this->View->Model->Text), "", [
					'onkeyup' => "we_cmd('setHot')",
					'id' => 'yuiAcInputPathName', 'onblur' => "we_cmd('syncNewsletterTitle');"], 'text', self::def_width)));
		$table->setColContent(2, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[dir]'), $this->formNewsletterDirChooser((self::def_width - 120), 0, "ParentID", $this->View->Model->ParentID, "Path", dirname($this->View->Model->Path), 'setHot', $this->weAutoCompleter)));

		//$table->setCol(2,0,[],we_html_tools::htmlFormElementTable($this->formWeDocChooser(NEWSLETTER_TABLE,320,0,"ParentID",$this->View->newsletter->ParentID,"Path",dirname($this->View->newsletter->Path),"opener.top.content.hot=true;",we_base_ContentTypes::FOLDER),g_l('modules_newsletter','[dir]')));
		$parts = [
			[
				'headline' => g_l('modules_newsletter', '[path]'),
				'html' => $table->getHtml(),
				'icon' => we_html_multiIconBox::PROP_PATH
			],
		];

		if(!$this->View->Model->IsFolder){
			$table = new we_html_table(['class' => 'default withSpace'], 4, 1);
			$table->setColContent(0, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[subject]'), we_html_tools::textInput("Subject", stripslashes($this->View->Model->Subject), "", [
						'onkeyup' => 'top.content.hot=true;'], 'text', self::def_width)));
			$table->setColContent(1, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[sender]'), we_html_tools::textInput("Sender", $this->View->Model->Sender, "", [
						'onkeyup' => 'top.content.hot=true;'], 'text', self::def_width)));

			$chk = ($this->View->Model->Sender == $this->View->Model->Reply ?
				we_html_element::input(['type' => "checkbox", 'value' => 1, 'checked' => null, 'name' => "reply_same", 'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked) document.we_form.Reply.value=document.we_form.Sender.value"]) :
				we_html_element::input(['type' => "checkbox", 'value' => 0, 'name' => "reply_same", 'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked) document.we_form.Reply.value=document.we_form.Sender.value"])
				);
			$table->setColContent(2, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[reply]'), we_html_tools::textInput("Reply", $this->View->Model->Reply, '', [
						'onkeyup' => 'top.content.hot=true;']) . "&nbsp;&nbsp;" . $chk . "&nbsp;" . we_html_element::label([
						'onclick' => "top.content.hot=true;if(document.we_form.reply_same.checked){document.we_form.reply_same.checked=false;}else{document.we_form.Reply.value=document.we_form.Sender.value;document.we_form.reply_same.checked=true;}"], g_l('modules_newsletter', '[reply_same]'))));
			$table->setColContent(3, 0, we_html_tools::formElementTable(g_l('modules_newsletter', '[test_email]'), we_html_tools::textInput("Test", $this->View->Model->Test, "", [
						'onkeyup' => 'top.content.hot=true;'])));

			// FIXME: inline js
			$inputAttribs = ['type' => "checkbox", 'value' => 1, 'name' => "isEmbedImagesChk", 'onclick' => "top.content.hot=true;if(document.we_form.isEmbedImagesChk.checked){document.we_form.isEmbedImages.value=1;}else{document.we_form.isEmbedImages.value=0;}"];
			$embedImagesChk = we_html_element::input(array_merge($inputAttribs, ($this->View->Model->isEmbedImages ? ['checked' => 'checked'] : [])));
			$embedImagesHid = we_html_element::hidden("isEmbedImages", $this->View->Model->isEmbedImages);
			$embedImagesLab = we_html_element::label(['onclick' => "top.content.hot=true;if(document.we_form.isEmbedImagesChk.checked){ document.we_form.isEmbedImagesChk.checked=false; document.we_form.isEmbedImages.value=0; }else{document.we_form.isEmbedImagesChk.checked=true;document.we_form.isEmbedImages.value=1;}"], g_l('modules_newsletter', '[isEmbedImages]'));

			$table->setColContent(4, 0, we_html_tools::formElementTable('', $embedImagesHid . $embedImagesChk . '&nbsp;' . $embedImagesLab));

			$parts[] = [
				'headline' => g_l('modules_newsletter', '[newsletter][text]'),
				'html' => $table->getHtml(),
				'icon' => we_html_multiIconBox::PROP_NEWSLETTER,
			];
			$parts[] = [
				'headline' => g_l('modules_newsletter', '[charset]'),
				'html' => $this->getHTMLCharsetTable(),
				'icon' => we_html_multiIconBox::PROP_CHARSET
			];
			$parts[] = [
				'headline' => g_l('modules_newsletter', '[copy_newsletter]'),
				'html' => $this->getHTMLCopy(),
				'icon' => we_html_multiIconBox::PROP_COPY,
				'noline' => true
			];
		}

		return we_html_multiIconBox::getHTML("newsletter_header", $parts, true) .
			we_html_element::br();
	}

	/**
	 * Generates the body for modul frame
	 *
	 * @return string
	 */
	function getHTMLProperties(){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return $this->View->getHomeScreen();
		}
		$js = $this->View->getJSProperty();
		$this->jsCmd->addCmd('setFocus');

		$out = $this->View->getHiddens() .
			$this->View->newsletterHiddens() .
			$this->View->getHiddensProperty();

		switch($this->View->page){
			case we_newsletter_frames::TAB_PROPERTIES:
				$out .= we_html_element::hiddens(['home' => 0, 'fromPage' => we_newsletter_frames::TAB_PROPERTIES]);

				if($this->View->Model->IsFolder == 0){
					$out .= $this->View->getHiddensMailingPage() .
						$this->View->getHiddensContentPage();
				}

				$out .= $this->getHTMLNewsletterHeader();
				break;
			case we_newsletter_frames::TAB_MAILING:
				$out .= $this->View->getHiddensPropertyPage() .
					$this->View->getHiddensContentPage() .
					we_html_element::hiddens(['fromPage' => we_newsletter_frames::TAB_MAILING, "ncustomer" => '', 'nfile' => '', 'ngroup' => '', 'noCustFilter' => '']) .
					$this->getHTMLNewsletterGroups();
				break;
			case we_newsletter_frames::TAB_EDIT:
				$out .= JQUERY .
					$this->View->getHiddensMailingPage() .
					$this->View->getHiddensPropertyPage() .
					we_html_element::hiddens(['fromPage' => we_newsletter_frames::TAB_EDIT, 'blockid' => 0]) .
					$this->getHTMLNewsletterBlocks();
				$js .= we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLHeader(false, true) : '';
				break;
			default:
				$out .= $this->View->getHiddensPropertyPage() .
					$this->View->getHiddensMailingPage() .
					$this->View->getHiddensContentPage() .
					we_html_element::hiddens(['fromPage' => we_newsletter_frames::TAB_REPORTING, 'blockid' => 0]) .
					we_html_multiIconBox::getHTML('', $this->View->getHTMLReporting(), true);
		}

		$body = we_html_element::body(['onload' => "self.loaded=true;if(self.doScrollTo){self.doScrollTo();}; setHeaderTitle();", 'class' => "weEditorBody", 'onunload' => "doUnload()"], we_html_element::form([
					'name' => 'we_form', 'method' => 'post', 'onsubmit' => "return false;"], $out
				)
		);

		return $this->getHTMLDocument($body, $js);
	}

	private function getHTMLEmailEdit(){
		$type = we_base_request::_(we_base_request::INT, 'etyp', 0);
		$htmlmail = we_base_request::_(we_base_request::STRING, 'htmlmail', f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"', '', $this->db));
		$id = we_base_request::_(we_base_request::INT, 'eid', 0);
		$email = we_base_request::_(we_base_request::EMAIL, 'email', '');
		$group = we_base_request::_(we_base_request::STRING, 'grp', '');
		$salutation = rawurldecode(str_replace('[:plus:]', '+', we_base_request::_(we_base_request::STRING, 'salutation', '')));
		$title = rawurldecode(str_replace('[:plus:]', '+', we_base_request::_(we_base_request::STRING, 'title', '')));
		$firstname = rawurldecode(str_replace('[:plus:]', '+', we_base_request::_(we_base_request::STRING, 'firstname', '')));
		$lastname = rawurldecode(str_replace('[:plus:]', '+', we_base_request::_(we_base_request::STRING, 'lastname', '')));

		$row = 0;
		$table = new we_html_table([], 12, 3);

		$table->setColContent($row, 0, g_l('modules_newsletter', '[email]'));
		$table->setColContent($row++, 1, we_html_tools::textInput("emailfield", $email, "", [], 'text', 280));


		$table->setColContent($row++, 1, we_html_forms::checkbox($htmlmail, (($htmlmail) ? true : false), "htmlmail", g_l('modules_newsletter', '[edit_htmlmail]'), false, '', "if(document.we_form.htmlmail.checked) document.we_form.htmlmail.value=1; else document.we_form.htmlmail.value=0;"));

		$salut_select = new we_html_select(['name' => 'salutation', 'style' => "width: 280px"]);
		$salut_select->addOption('', '');
		if(!empty($this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD])){
			$salut_select->addOption($this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], $this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD]);
		}
		if(!empty($this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD])){
			$salut_select->addOption($this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], $this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD]);
		}
		$salut_select->selectOption($salutation);

		$table->setColContent($row, 0, g_l('modules_newsletter', '[salutation]'));
		$table->setColContent($row++, 1, $salut_select->getHtml());


		$table->setColContent($row, 0, g_l('modules_newsletter', '[title]'));
		$table->setColContent($row++, 1, we_html_tools::textInput('title', ($GLOBALS['we']['Charset']['Backend'] != 'UTF-8' ? utf8_decode($title) : $title), "", [
				], 'text', 280));


		$table->setColContent($row, 0, g_l('modules_newsletter', '[firstname]'));
		$table->setColContent($row++, 1, we_html_tools::textInput('firstname', ($GLOBALS['we']['Charset']['Backend'] != 'UTF-8' ? utf8_decode($firstname) : $firstname), '', [
				], 'text', 280));


		$table->setColContent($row, 0, g_l('modules_newsletter', '[lastname]'));
		$table->setColContent($row++, 1, we_html_tools::textInput('lastname', ($GLOBALS['we']['Charset']['Backend'] != 'UTF-8' ? utf8_decode($lastname) : $lastname), '', [
				], 'text', 280));


		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();");
		$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('editEmails_save', " . intval($type) . ")");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "we_cmd('editEmails_onload')"], we_html_element::form([
					'name' => 'we_form', 'onsubmit' => "we_cmd('editEmails_save', " . intval($type) . ");return false;"], we_html_element::hidden("group", $group) .
					($type ?
					we_html_element::hidden('id', $id) :
					''
					) .
					we_html_tools::dialogLayout(
						$table->getHtml(), g_l('modules_newsletter', $type ? '[edit_email]' : '[add_email]'), we_html_button::position_yes_no_cancel($save, $close)
					)
				)
		);

		return $this->getHTMLDocument($body, $this->View->getJSProperty());
	}

	private function getHTMLPreview(){
		$gview = we_base_request::_(we_base_request::INT, "gview", 0);
		$hm = we_base_request::_(we_base_request::INT, "hm", 0);

		$content = '';
		$count = count($this->View->Model->blocks);
		for($i = 0; $i < $count; $i++){
			$content .= $this->View->getContent($i, $gview, $hm);
		}

		header("Pragma: no-cache;");
		header("Cache-Control: post-check=0, post-check=0, false");
		we_html_tools::headerCtCharset('text/html', ($this->View->Model->Charset ?: $GLOBALS['we']['Charset']['Backend']));


		if(!$hm){
			echo '<html><head></head><body><form>
							<textarea name="foo" style="width:100%;height:95%">' .
			oldHtmlspecialchars(trim($content)) .
			'</textarea></form></body></html>';
		} else {
			echo $content;
		}
	}

	private function getHTMLBlackList(){
		$this->View->settings['black_list'] = we_base_request::_(we_base_request::STRING, "black_list", $this->View->settings['black_list']);

		if(($ncmd = we_base_request::_(we_base_request::STRING, 'ncmd'))){
			if($ncmd === 'save_black'){
				$this->View->processCommands($this->jsCmd);
			}
		}

		$js = $this->View->getJSProperty() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', 'self.focus();');

		switch(we_base_request::_(we_base_request::STRING, 'ncmd')){
			case 'import_black':
				$filepath = we_base_request::_(we_base_request::FILE, "csv_file");
				$delimiter = we_base_request::_(we_base_request::RAW_CHECKED, "csv_delimiter");
				$col = we_base_request::_(we_base_request::INT, "csv_col", 0);

				if($col){
					$col--;
				}

				if(strpos($filepath, '..') !== false){
					$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
				} else {
					$fh = @fopen($_SERVER['DOCUMENT_ROOT'] . $filepath, 'rb');
					if($fh){
						while(($dat = fgetcsv($fh, 1000, $delimiter))){
							$alldat = implode('', $dat);
							if(str_replace(' ', '', $alldat) === ''){
								continue;
							}
							$row[] = $dat[$col];
						}

						fclose($fh);

						if(!empty($row)){
							if($this->View->settings['black_list'] === ''){
								$this->View->settings['black_list'] = implode(',', $row);
							} else {
								$this->View->settings['black_list'] .= "," . implode(',', $row);
							}
						}
					} else {
						$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				break;
			case 'export_black':
				$fname = rtrim(we_base_request::_(we_base_request::FILE, "csv_dir", ''), '/') . '/blacklist_export_' . time() . '.csv';
				we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $fname, str_replace(",", "\n", $this->View->settings['black_list']));
				$this->jsCmd->addCmd('export_csv_window', $fname);
				break;
		}

		$arr = array_filter(explode(',', $this->View->settings['black_list']));

		$buttons_table = new we_html_table(['class' => 'default withSpace'], 4, 1);
		$buttons_table->setColContent(0, 0, we_html_button::create_button(we_html_button::ADD, "javascript:addBlack();"));
		$buttons_table->setColContent(1, 0, we_html_button::create_button(we_html_button::EDIT, "javascript:editBlack();"));
		$buttons_table->setColContent(2, 0, we_html_button::create_button(we_html_button::DELETE, "javascript:deleteBlack()"));
		$buttons_table->setColContent(3, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:deleteallBlack()"));

		$table = new we_html_table(['class' => 'default'], 5, 3);
		$table->setCol(0, 0, ['style' => 'vertical-align:middle;'], we_html_tools::select("blacklist_sel", $arr, 10, "", false, ['style' => "width:388px"], true, 600));
		$table->setCol(0, 1, ['style' => 'vertical-align:top;padding-left:15px;'], $buttons_table->getHtml());

		$importbut = we_html_button::create_button(we_html_button::IMPORT, "javascript:set_import(1)");
		$exportbut = we_html_button::create_button(we_html_button::EXPORT, "javascript:set_export(1)");

		$table->setCol(2, 0, ['colspan' => 3, 'style' => 'padding-top:10px;'], $importbut . $exportbut);

		$sib = we_base_request::_(we_base_request::RAW, "sib", 0);
		$seb = we_base_request::_(we_base_request::RAW, "seb", 0);

		if($sib){
			$import_options = new we_html_table(['class' => 'default withSpace'], 2, 2);
			$import_options->setColContent(0, 0, g_l('modules_newsletter', '[csv_delimiter]') . ":&nbsp;");
			$import_options->setColContent(0, 1, we_html_tools::textInput("csv_delimiter", ","));
			$import_options->setColContent(1, 0, g_l('modules_newsletter', '[csv_col]') . ":&nbsp;");
			$import_options->setColContent(1, 1, we_html_tools::textInput("csv_col", 1));

			$import_box = new we_html_table(['class' => 'default withSpace', 'style' => 'padding-top:10px;'], 4, 1);
			$import_box->setColContent(0, 0, $this->formFileChooser('csv_file', '/', ''));
			$import_box->setColContent(1, 0, we_html_button::create_button(we_html_button::UPLOAD, "javascript:we_cmd('upload_black')"));
			$import_box->setColContent(2, 0, $import_options->getHtml());
			$import_box->setColContent(3, 0, we_html_button::create_button(we_html_button::OK, "javascript:document.we_form.sib.value=0;we_cmd('import_black');") . we_html_button::create_button(we_html_button::CANCEL, "javascript:set_import(0);"));

			$table->setCol(3, 0, ['colspan' => 3], we_html_element::hiddens(['csv_import' => 1]) . $import_box->getHtml());
		} elseif($seb){
			$export_box = new we_html_table(['class' => 'default withSpace', 'style' => 'padding-top:10px;'], 2, 1);
			$export_box->setColContent(0, 0, $this->formFileChooser('csv_dir', '/', '', we_base_ContentTypes::FOLDER));
			$export_box->setColContent(1, 0, we_html_button::create_button(we_html_button::OK, "javascript:document.we_form.seb.value=0;we_cmd('export_black');") . we_html_button::create_button(we_html_button::CANCEL, "javascript:set_export(0);"));

			$table->setCol(3, 0, ['colspan' => 3], we_html_element::hiddens(['csv_export' => 1]) . $export_box->getHtml());
		}

		$body = we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => 'we_form', 'onsubmit' => "save();return false;"], $this->View->getHiddens() .
					we_html_element::hiddens([
						'black_list' => $this->View->settings['black_list'],
						'sib' => $sib,
						'seb' => $seb]) .
					we_html_tools::dialogLayout(
						$table->getHtml(), g_l('modules_newsletter', '[black_list]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_black')"), null, we_html_button::create_button(we_html_button::CANCEL, "javascript:self.close();"))
					)
				)
		);

		return $this->getHTMLDocument($body, $js);
	}

	private function getHTMLUploadCsv($what){
		$weFileupload = new we_fileupload_ui_base('we_File', [we_base_ContentTypes::TEXT]);

		$pb = new we_gui_progressBar(0, 120, 'fu_extProgress_we_File');

		$footerTable = new we_html_table(['class' => 'default', 'style' => 'width:100%;'], 1, 2);
		$footerTable->setColContent(0, 0, we_html_element::div(['id' => 'progressbar', 'style' => 'display:none;padding-left:10px'], $pb->getHTML()));
		$footerTable->setCol(0, 1, ['style' => 'text-align:right'], we_html_button::create_button(we_html_button::IMPORT, "javascript:we_cmd('do_" . $what . "');", '', '', false, false, '_footer') . we_html_button::create_button(we_html_button::CANCEL, "javascript:top.close();"));

		$js = $this->View->getJSProperty() . $weFileupload->getJs() . we_gui_progressBar::getJSCode();

		$table = new we_html_table(['class' => 'default withBigSpace'], 1, 1);
		//$table->setCol(0, 0, ['style' => "padding-right:30px"], $weFileupload->getHtmlAlertBoxes());
		$table->setCol(0, 0, ['style' => 'vertical-align:middle;'], $weFileupload->getHTML());

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => 'we_form', 'method' => 'post',
					'enctype' => 'multipart/form-data'], we_html_element::center(
						$this->View->getHiddens() .
						(($grp = we_base_request::_(we_base_request::STRING, 'grp')) !== false ? we_html_element::hiddens(['group' => $grp]) : '') .
						we_html_element::hiddens(['MAX_FILE_SIZE' => we_fileupload::getMaxUploadSizeB()]) .
						we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[csv_upload]'), $footerTable->getHTML())
					)
				)
		);

		return $this->getHTMLDocument($body, $weFileupload->getCSS() . $js);
	}

	private function getHTMLExportCsvMessage($allowClear = false){
		$link = we_base_request::_(we_base_request::URL, 'lnk');
		if($link === false){
			return '';
		}

		$table = new we_html_table(['class' => 'default'], 3, 1);
		$table->setCol(0, 0, ['style' => 'padding-top:5px;'], sprintf(g_l('modules_newsletter', '[csv_export]'), $link));
		$table->setCol(1, 0, ['style' => 'padding-top:5px;'], we_backup_wizard::getDownloadLinkText());
		$table->setCol(2, 0, ['style' => 'padding:5px 0;'], we_html_element::a(['href' => getServerUrl(true) . $link, 'download' => basename($link)], g_l('modules_newsletter', '[csv_download]')));

		if($allowClear){
			$table->setCol(3, 0, ['style' => 'padding:5px 0;'], we_html_element::b(g_l('modules_newsletter', '[clearlog_note]')));
		}

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => 'we_form', 'method' => 'post'], we_html_element::hidden('group', '') .
					($allowClear ?
					we_html_element::hiddens(['pnt' => 'clear_log', 'ncmd' => 'do_clear_log']) .
					we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[clear_log]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:clearLog();"), null, we_html_button::create_button(we_html_button::CANCEL, "javascript:self.close();"))) :
					we_html_tools::dialogLayout($table->getHtml(), g_l('modules_newsletter', '[csv_download]'), we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();")))
					)
				)
		);

		return ($allowClear ? $body : $this->getHTMLDocument($body));
	}

	/**
	 * Edit csv mail list
	 *
	 * @param String $open_file
	 * @return String
	 */
	private function getHTMLEditFile($open_file = ''){
		$out = '';
		$content = [];

		$order = we_base_request::_(we_base_request::STRING, 'order', '');
		for($i = 0; $i < 14; $i = $i + 2){
			$sorter_code[$i] = "<br/>" . ($order == $i ?
				we_html_element::input(['type' => 'radio', 'value' => $i, 'name' => "order", 'checked' => true, 'onclick' => "submitForm('edit_file')"]) . "&darr;" :
				we_html_element::input(['type' => 'radio', 'value' => $i, 'name' => "order", 'onclick' => "submitForm('edit_file')"]) . "&darr;"
				);
			$sorter_code[$i + 1] = ($order == $i + 1 ?
				we_html_element::input(['type' => 'radio', 'value' => $i + 1, 'name' => "order", 'checked' => true, 'onclick' => "submitForm('edit_file')"]) . "&uarr;" :
				we_html_element::input(['type' => 'radio', 'value' => $i + 1, 'name' => "order", 'onclick' => "submitForm('edit_file')"]) . "&uarr;"
				);
		}

		$headlines = [
			'ID' . $sorter_code[0] . $sorter_code[1],
			g_l('modules_newsletter', '[email]') . $sorter_code[2] . $sorter_code[3],
			g_l('modules_newsletter', '[edit_htmlmail]') . $sorter_code[4] . $sorter_code[5],
			g_l('modules_newsletter', '[salutation]') . $sorter_code[6] . $sorter_code[7],
			g_l('modules_newsletter', '[title]') . $sorter_code[8] . $sorter_code[9],
			g_l('modules_newsletter', '[firstname]') . $sorter_code[10] . $sorter_code[11],
			g_l('modules_newsletter', '[lastname]') . $sorter_code[12] . $sorter_code[13],
			g_l('modules_newsletter', '[edit]'),
			g_l('modules_newsletter', '[status]'),
		];

		$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', '');
		$emails = [];
		$emailkey = [];
		if(strpos($csv_file, '..') === false && is_readable($_SERVER['DOCUMENT_ROOT'] . $csv_file)){
			if($csv_file){
				$emails = we_newsletter_newsletter::getEmailsFromExtern2($csv_file, we_base_request::_(we_base_request::INT, 'weEmailStatus', 0), $emailkey);
			}
		} else {
			$this->jsCmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
		}

		$offset = max(we_base_request::_(we_base_request::INT, 'offset', 0), 0);
		$numRows = we_base_request::_(we_base_request::INT, 'numRows', 15);
		$anz = count($emails);
		$endRow = min($offset + $numRows, $anz);

		switch($order){
			case 2:
			case 3:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[0], $b[0]);
				});
				break;
			case 4:
			case 5:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[1], $b[1]);
				});
				break;
			case 6:
			case 7:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[2], $b[2]);
				});
				break;
			case 8:
			case 9:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[3], $b[3]);
				});
				break;
			case 10:
			case 11:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[4], $b[4]);
				}
				);
				break;
			case 12:
			case 13:
				uasort($emails, function ($a, $b){
					return strnatcasecmp($a[5], $b[5]);
				});
				break;
		}

		switch($order){
			case 0:
			case 2:
			case 4:
			case 6:
			case 8:
			case 10:
			case 12:
				$emails = array_reverse($emails, true);
		}
		$counter = 0;
		foreach($emails as $k => $cols){
			if($k >= $offset && $k < $endRow){

				$edit = we_html_button::create_button(we_html_button::EDIT, "javascript:editEmailFile(" . $emailkey[$k] . ",'" . $cols[0] . "','" . $cols[1] . "','" . $cols[2] . "','" . $cols[3] . "','" . $cols[4] . "','" . $cols[5] . "')");
				$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:delEmailFile(" . $emailkey[$k] . ",'" . $cols[0] . "')");

				$content[$counter] = [['dat' => we_html_element::div(['class' => 'middlefont'], $k),
					'height' => '',
					'align' => '',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], ($cols[0] ?: '&nbsp;')),
						'height' => '',
						'align' => '',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], g_l('modules_newsletter', ($cols[1] ? '[yes]' : '[no]'))),
						'height' => '',
						'align' => '',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], ($cols[2] ?: '&nbsp;')),
						'height' => '',
						'align' => 'right',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], ($cols[3] ?: '&nbsp;')),
						'height' => '',
						'align' => 'left',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], ($cols[4] ?: '&nbsp;')),
						'height' => '',
						'align' => 'left',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], ($cols[5] ?: '&nbsp;')),
						'height' => '',
						'align' => 'left',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], $edit . $trash),
						'height' => '',
						'align' => 'left',
					],
					['dat' => we_html_element::div(['class' => 'middlefont'], '<i class="fa fa-lg ' . (we_check_email($cols[0]) ? 'fa-ok fa-check' : 'fa-cancel fa-close' ) . '"></i>'),
						'height' => '',
						'align' => "center",
					]
				];
				$counter++;
			}
		}

		$js = $this->View->getJSProperty();

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close()");
		$edit = we_html_button::create_button(we_html_button::SAVE, "javascript:listFile()");
		$nextprev = new we_html_table(['class' => 'default'], 1, 4);

		$nextprev->setCol(0, 0, ['style' => 'padding-right:10px;'], ($offset ?
				we_html_button::create_button(we_html_button::BACK, "javascript:document.we_form.offset.value=" . ($offset - $numRows) . ";submitForm('edit_file');") :
				we_html_button::create_button(we_html_button::BACK, "#", '', '', true)));

		$nextprev->setCol(0, 1, ['style' => 'padding-right:10px;'], we_html_element::b(( $anz ? $offset + 1 : 0 ) . ' - ' . (($anz - $offset) < $numRows ? $anz : $offset + $numRows) . ' ' .
				g_l('global', '[from]') . ' ' .
				$anz));

		$nextprev->setColContent(0, 2, (($offset + $numRows) < $anz ?
				we_html_button::create_button(we_html_button::NEXT, "javascript:document.we_form.offset.value=" . ($offset + $numRows) . ";submitForm('edit_file');") :
				we_html_button::create_button(we_html_button::NEXT, "#", '', '', true)
		));

		if(!empty($emails)){
			$add = we_html_button::create_button(we_html_button::PLUS, "javascript:editEmailFile(" . count($emails) . ",'','','','','','')");
			$end = $nextprev->getHtml();

			$nextprev->addCol(3);

			$nextprev->setCol(0, 3, ['style' => 'padding-left:20px;'], we_html_element::b(g_l('modules_newsletter', '[show]')) . ' ' . we_html_tools::textInput("numRows", $numRows)
			);
			$selectStatus = we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
					we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
					we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
					we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
					], '', we_base_request::_(we_base_request::INT, 'weEmailStatus', 0), '', ['onchange' => 'listFile();'], true, 150);
			$nextprev->setCol(0, 4, ['style' => 'padding-left:20px;'], $selectStatus);
			$nextprev->setCol(0, 5, ['style' => 'padding-left:20px;'], $add);

			$out = $nextprev->getHtml() .
				we_html_tools::dialogBorder(850, $content, $headlines) .
				$end;
		} else {
			if(empty($csv_file) || strlen($csv_file) < 4){
				$nlMessage = g_l('modules_newsletter', '[no_file_selected]');
				$selectStatus2 = '';
			} elseif(($emailStatus = we_base_request::_(we_base_request::INT, 'weEmailStatus', 0)) == we_newsletter_newsletter::CSV_STATUS_INVALID){
				$nlMessage = g_l('modules_newsletter', '[file_all_ok]');
				$selectStatus2 = '<br/>' . we_html_element::b(g_l('modules_newsletter', '[status]')) . ' ' . we_html_tools::select('weEmailStatus', [
						we_newsletter_newsletter::CSV_STATUS_ALL => g_l('modules_newsletter', '[statusAll]'),
						we_newsletter_newsletter::CSV_STATUS_INVALID => g_l('modules_newsletter', '[statusInvalid]'),
						we_newsletter_newsletter::CSV_STATUS_VALID => g_l('modules_newsletter', '[statusValid]')
						], '', $emailStatus, '', ['onchange' => 'listFile();'], true, 150);
			} else {
				$nlMessage = g_l('modules_newsletter', '[file_all_ok]');
				$selectStatus2 = '';
			}

			$out = we_html_element::div(['class' => "middlefont lowContrast", 'style' => 'text-align:center;padding-bottom:2em;'], '--&nbsp;' . $nlMessage . '&nbsp;--' . $selectStatus2) .
				we_html_button::create_button(we_html_button::PLUS, "javascript:editEmailFile(" . count($emails) . ",'','','','','','')");
		}


		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => ($open_file ? "submitForm('edit_file')" : "" )], we_html_element::form(['name' => 'we_form'], we_html_element::hiddens([
						'ncmd' => 'edit_file',
						'pnt' => 'edit_file',
						'order' => $order,
						'offset' => $offset,
						'nrid' => '',
						'email' => '',
						'htmlmail' => '',
						'salutation' => '',
						'title' => '',
						'firstname' => '',
						'lastname' => '',
						'etyp' => '',
						'eid' => '']) .
					we_html_tools::dialogLayout(
						we_html_element::div(['style' => 'margin-top:10px;'], $this->formFileChooser('csv_file', ($open_file ?: ($csv_file ?: '/')), 'listFile'))
						. '<br/>' . $out, g_l('modules_newsletter', '[select_file]'), $close . $edit)
				)
		);

		return $this->getHTMLDocument($body, $js);
	}

	function getHTMLClearLog(){
		if(we_base_request::_(we_base_request::STRING, 'ncmd') === 'do_clear_log'){
			$this->db->query('TRUNCATE TABLE ' . NEWSLETTER_LOG_TABLE);
			$this->jsCmd->addMsg(g_l('modules_newsletter', '[log_is_clear]'), we_base_util::WE_MESSAGE_NOTICE);
			return $this->getHTMLDocument(we_html_element::body());
		}

		$csv = '';
		$this->db->query('SELECT n.Text as NewsletterName, l.* FROM ' . NEWSLETTER_TABLE . ' n JOIN ' . NEWSLETTER_LOG_TABLE . ' l ON n.ID=l.NewsletterID');
		while($this->db->next_record()){
			$csv .= $this->db->f('NewsletterName') . ',' . date(g_l('weEditorInfo', '[date_format]'), intval($this->db->f('LogTime'))) . ',' . (g_l('modules_newsletter', '[' . $this->db->f('Log') . ']') !== false ? (sprintf(g_l('modules_newsletter', '[' . $this->db->f('Log') . ']'), $this->db->f('Param'))) : $this->db->f('Log')) . "\n";
		}

		$link = BACKUP_DIR . 'download/log_' . time() . '.csv';
		if(we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $link, $csv)){
			we_base_file::insertIntoCleanUp($link, 3600);
		} else {
			$link = '';
		}

		$_REQUEST['lnk'] = $link;

		return $this->getHTMLDocument($this->getHTMLExportCsvMessage(true), we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js'));
	}

	private function getHTMLSendWait(){
		$nid = we_base_request::_(we_base_request::INT, 'nid', 0);
		$test = we_base_request::_(we_base_request::BOOL, 'test');


		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();setTimeout(function (doc) {doc.we_form.submit();},200,document);"], we_html_element::form([
						'name' => 'we_form', 'action' => WEBEDITION_DIR . 'we_showMod.php', 'method' => 'post'], we_html_element::hiddens(['mod' => 'newsletter',
							'pnt' => "send_frameset",
							'nid' => $nid,
							'test' => $test]) .
						we_html_element::center('<i class="fa fa-2x fa-spinner fa-pulse"></i>' .
							we_html_element::br() .
							we_html_element::br() .
							we_html_element::div(['class' => 'header_small'], g_l('modules_newsletter', '[prepare_newsletter]'))
						)
					)
				)
		);
	}

	private function getHTMLSendFrameset(){
		$nid = we_base_request::_(we_base_request::INT, 'nid', 0);
		$test = we_base_request::_(we_base_request::BOOL, 'test');

		$this->View->Model = new we_newsletter_newsletter($nid);
		$ret = $this->View->cacheNewsletter();

		$offset = ($this->View->Model->Offset) ? ($this->View->Model->Offset + 1) : 0;
		$step = $this->View->Model->Step;

		if($this->View->settings['send_step'] <= $offset){
			$step++;
			$offset = 0;
		}
		$this->jsCmd->addCmd(
			(($this->View->Model->Step != 0 || $this->View->Model->Offset != 0) ?
				'ask_camp' :
				'cancel_camp')
		);

		$head = we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', '', ['id' => 'loadVarNewsletter', 'data-newsletter' => setDynamicVar([
					'start' => $this->View->Model->Step,
					'group' => $this->View->Model->Offset,
					'nid' => $nid,
					'test' => $test,
					'blockcache' => $ret['blockcache'],
					'emailcache' => $ret['emailcache'],
					'ecount' => $ret['ecount'],
					'gcount' => $ret['gcount']
			])]) . $this->jsCmd->getCmds();

		$body = we_html_element::iframe('send_body', WEBEDITION_DIR . 'we_showMod.php?mod=newsletter&pnt=send_body', 'position:absolute;top:0;bottom:0;left:0;right:0;', '', '', false) .
			we_html_element::iframe('send_cmd', WEBEDITION_DIR . 'we_showMod.php?mod=newsletter&pnt=send_cmd', 'position:absolute;width:0;height:0;', '', '', false) .
			we_html_element::iframe('send_control', WEBEDITION_DIR . 'we_showMod.php?mod=newsletter&pnt=send_control&nid=' . $nid . '&test=' . $test . '&blockcache=' . $ret['blockcache'] . '&emailcache=' . $ret['emailcache'] . '&ecount=' . $ret['ecount'] . '&gcount=' . $ret['gcount'], 'position:absolute;width:0;height:0;', '', '', false);

		return $this->getHTMLDocument(we_html_element::body([], $body), $head);
	}

	private function getHTMLSendBody(){
		$pb = new we_gui_progressBar(we_base_request::_(we_base_request::INT, "pro", 0), 400);
		$pb->addText(g_l('modules_newsletter', '[sending]'), we_gui_progressBar::TOP, "title");

		$footer = '<table style="width:580px;" class="default"><tr><td style="text-align:left">' .
			$pb->getHTML() . '</td><td style="text-align:right">' .
			we_html_button::create_button(we_html_button::CLOSE, "javascript:top.close();") .
			'</td></tr></table>';

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => 'we_form', 'method' => 'post'], we_html_tools::dialogLayout(we_html_element::textArea([
								'name' => "details",
								'style' => "width:530px;height:280px;"
								], g_l('modules_newsletter', (we_base_request::_(we_base_request::BOOL, "test") ? '[test_no_mail]' : '[sending]')) . "\n" .
								g_l('modules_newsletter', '[campaign_starts]')
							), g_l('modules_newsletter', '[details]'), $footer)
					)
				), we_gui_progressBar::getJSCode());
	}

	private function getHTMLSendCmd(){
		$nid = we_base_request::_(we_base_request::INT, 'nid');
		if($nid === false){
			return '';
		}

		$test = we_base_request::_(we_base_request::BOOL, 'test');
		$start = we_base_request::_(we_base_request::INT, 'start', 0);

		// to calc progress ------------------
		// total number of emails
		$ecount = we_base_request::_(we_base_request::INT, 'ecount', 0);
		// counter
		$ecs = we_base_request::_(we_base_request::INT, 'ecs', 0);
		//-----------------------------------

		$blockcache = we_base_request::_(we_base_request::RAW, 'blockcache', 0);

		// emails cache -----------------------
		$emailcache = we_base_request::_(we_base_request::RAW, 'emailcache', 0);
		//
		$egc = we_base_request::_(we_base_request::INT, 'egc', 0);
		//
		$gcount = we_base_request::_(we_base_request::INT, 'gcount', 0);
		//-----------------------------------

		$reload = we_base_request::_(we_base_request::BOOL, 'reload');
		$retry = we_base_request::_(we_base_request::BOOL, 'retry');


		$this->View->Model = new we_newsletter_newsletter($nid);
		if($retry){
			$egc = $this->View->Model->Step;
			$start = $this->View->Model->Offset;
			if($start){
				$start++;
			}
			$this->View->Model->addLog(we_newsletter_newsletter::LOG_RETRY);
		}

		$body = we_html_element::body(['style' => 'margin:10px;', 'onload' => "initControl()"], we_html_element::form([
					'name' => 'we_form',
					'method' => 'post'], we_html_element::hiddens([
						'mod' => 'newsletter',
						'nid' => $nid,
						'pnt' => 'send_cmd',
						'test' => $test,
						'blockcache' => $blockcache,
						'emailcache' => $emailcache,
						'ecount' => $ecount,
						'value' => $ecount,
						'gcount' => $gcount,
						'egc' => $egc + 1,
						'ecs' => $ecs,
						'reload' => 1])
				)
		);
		$progress_update = [
			'log' => [],
			'text' => '',
			'percent' => 0
		];

		if($gcount <= $egc){
			we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $blockcache);
			$progress_update = [
				'log' => [g_l('modules_newsletter', '[campaign_ends]')],
				'text' => '<span style="color:#006699;font-weight:bold;">' . g_l('modules_newsletter', '[finished]') . '</span>',
				'percent' => 100
			];

			$this->db->query('UPDATE ' . NEWSLETTER_TABLE . ' SET Step=0,Offset=0 WHERE ID=' . $this->View->Model->ID);
			if(!$test){
				$this->View->Model->addLog(we_newsletter_newsletter::LOG_END_SEND);
			}
			$this->jsCmd->addCmd('updateLog', $progress_update);
			return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', 'self.focus();'));
		}

		if($start && !$test && !$reload){
			$this->View->Model->addLog(we_newsletter_newsletter::LOG_CONTINUE_SEND);
		} else if(!$test && !$reload){
			$this->View->Model->addLog(we_newsletter_newsletter::LOG_START_SEND);
		}

		$content = '';

		$emails = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $egc);
		if($emails){
			$end = count($emails);
			for($j = $start; $j < $end; $j++){
				$email = trim($emails[$j][0]);

				$user_groups = explode(',', $emails[$j][6]);
				$user_blocks = $emails[$j][7];

				sort($user_blocks);
				$user_blocks = array_unique($user_blocks);

				$htmlmail = isset($emails[$j][1]) ? str_replace(["\r", "\n"], '', $emails[$j][1]) : '';
				$salutation = isset($emails[$j][2]) ? str_replace(["\r", "\n"], '', $emails[$j][2]) : '';
				$title = isset($emails[$j][3]) ? str_replace(["\r", "\n"], '', $emails[$j][3]) : '';
				$firstname = isset($emails[$j][4]) ? str_replace(["\r", "\n"], '', $emails[$j][4]) : '';
				$lastname = isset($emails[$j][5]) ? str_replace(["\r", "\n"], '', $emails[$j][5]) : '';
				$customerid = isset($emails[$j][8]) ? str_replace(["\r", "\n"], '', $emails[$j][8]) : '';
				$iscustomer = (isset($emails[$j][9]) && $emails[$j][9] === 'customer' ? 'C' : '');

				$contentDefault = $content_plainDefault = $contentF = $contentF_plain = $contentM = $contentM_plain = $contentTFL = $contentTFL_plain = $contentTL = $contentTL_plain = $contentFL = $contentFL_plain = $contentLN = $contentLN_plain = $contentFN = $contentFN_plain = '';

				$inlines = $atts = [];

				foreach($user_groups as $user_group){
					$atts = array_merge($atts, $this->View->getAttachments($user_group));
				}

				foreach($user_blocks as $user_block){
					$html_block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_h_' . $user_block);
					$plain_block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_p_' . $user_block);

					$contentDefault .= $html_block['default' . $iscustomer];
					$content_plainDefault .= $plain_block['default' . $iscustomer];

					$contentF .= $html_block['female' . $iscustomer];
					$contentF_plain .= $plain_block['female' . $iscustomer];

					$contentM .= $html_block['male' . $iscustomer];
					$contentM_plain .= $plain_block['male' . $iscustomer];

					$contentTFL .= $html_block['title_firstname_lastname' . $iscustomer];
					$contentTFL_plain .= $plain_block['title_firstname_lastname' . $iscustomer];

					$contentTL .= $html_block['title_lastname' . $iscustomer];
					$contentTL_plain .= $plain_block['title_lastname' . $iscustomer];

					$contentFL .= $html_block['firstname_lastname' . $iscustomer];
					$contentFL_plain .= $plain_block['firstname_lastname' . $iscustomer];

					$contentLN .= $html_block['lastname' . $iscustomer];
					$contentLN_plain .= $plain_block['lastname' . $iscustomer];

					$contentFN .= $html_block['firstname' . $iscustomer];
					$contentFN_plain .= $plain_block['firstname' . $iscustomer];

					$inlines = array_merge($inlines, $html_block['inlines']);
				}

				if($salutation && $lastname && ($salutation == $this->View->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD]) && ((!$this->View->settings['title_or_salutation']) || (!$title))){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title,];
					$content = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentF) : $contentF), $rep);
					$content_plain = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentF_plain) : $contentF_plain), $rep);
				} else if($salutation && $lastname && ($salutation == $this->View->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD]) && ((!$this->View->settings['title_or_salutation']) || (!$title))){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];

					$content = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentM) : $contentM), $rep);
					$content_plain = strtr(($title ? preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentM_plain) : $contentM_plain), $rep);
				} else if($title && $firstname && $lastname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];
					$content = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTFL), $rep);
					$content_plain = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTFL_plain), $rep);
				} else if($title && $lastname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
						we_newsletter_base::TITLE_REPLACE_TEXT => $title
					];
					$content = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTL), $rep);
					$content_plain = strtr(preg_replace('|([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '|', '${1} ' . $title, $contentTL_plain), $rep);
				} else if($lastname && $firstname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid,
					];

					$content = strtr($contentFL, $rep);
					$content_plain = strtr($contentFL_plain, $rep);
				} else if($firstname){
					$rep = [
						we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $firstname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid
					];
					$content = strtr($contentFN, $rep);
					$content_plain = strtr($contentFN_plain, $rep);
				} else if($lastname){
					$rep = [
						we_newsletter_base::LASTNAME_REPLACE_TEXT => $lastname,
						we_newsletter_base::CUSTOMERID_REPLACE_TEXT => $customerid
					];

					$content = strtr($contentLN, $rep);
					$content_plain = strtr($contentLN_plain, $rep);
				} else {

					$content = $contentDefault;
					$content_plain = $content_plainDefault;
				}

				$secEmail = sha1($email . md5(SECURITY_ENCRYPTION_KEY . $_SERVER['DOCUMENT_ROOT'])); //use md5, doc root so we will never expose the key
				$content_plain = strtr($content_plain, [we_newsletter_base::EMAIL_REPLACE_TEXT => $email, we_newsletter_base::EMAILTOKEN_REPLACE_TEXT => $secEmail]);
				$content = strtr($content, [we_newsletter_base::EMAIL_REPLACE_TEXT => $email, we_newsletter_base::EMAILTOKEN_REPLACE_TEXT => $secEmail]);

				// damd: Newsletter Platzhalter ersetzten
				$this->replacePlaceholder($content, $content_plain, $emails[$j]);

				//$clean = $this->View->getCleanMail($this->View->newsletter->Reply);

				$not_black = !$this->View->isBlack($email); //Bug #5791 Prüfung muss vor der aufbereitung der Adresse erfolgen
				if($lastname && $firstname || $title && $lastname){
					$emailName = ($title ? $title . ' ' : '') .
						($firstname ? $firstname . ' ' : '') .
						$lastname . '<' . $email . '>';
					//$email = $emailName;
				} else {
					$emailName = $email;
				}
				$phpmail = new we_mail_mail($this->View->Model->Charset ?: $GLOBALS['language']['charset'], $emailName, $this->View->Model->Subject, $this->View->Model->Sender, $this->View->Model->Reply, $this->View->Model->isEmbedImages);

				if($htmlmail){
					$phpmail->addHTMLPart($content);
					$phpmail->addTextPart(trim($content_plain));
				} else {
					$phpmail->addTextPart(trim($content_plain));
				}

				if(!$this->View->settings['use_base_href']){
					$phpmail->setIsUseBaseHref($this->View->settings['use_base_href']);
				}

				foreach($atts as $att){
					$phpmail->doaddAttachment($att);
				}
//FIXME: what about inlines?
				$domain = '';
				$not_malformed = ($this->View->settings['reject_malformed']) ? $this->View->Model->check_email($email) : true;
				$verified = ($this->View->settings['reject_not_verified']) ? $this->View->Model->check_domain($email, $domain) : true;

				if($verified && $not_malformed && $not_black){
					if(!$test){
						$phpmail->buildMessage();
						if($phpmail->send()){
							if($this->View->settings['log_sending']){
								$this->View->Model->addLog(we_newsletter_newsletter::LOG_MAIL_SENT, $email);
							}
						} else {
							if($this->View->settings['log_sending']){
								$this->View->Model->addLog(we_newsletter_newsletter::LOG_MAIL_FAILED, $email);
							}
							$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[error]') . ': ' . g_l('modules_newsletter', '[mail_failed]'), $email);
						}
						$this->db->query('UPDATE ' . NEWSLETTER_TABLE . ' SET Step=' . intval($egc) . ',Offset=' . intval($j) . ' WHERE ID=' . $this->View->Model->ID);
					}
				} elseif(!$not_malformed){
					if(!$test && $this->View->settings['log_sending']){
						$this->View->Model->addLog(we_newsletter_newsletter::LOG_MAIL_MALFORMED, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[error]') . ': ' . g_l('modules_newsletter', '[email_malformed]'), $email);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				} elseif(!$verified){
					if(!$test && $this->View->settings['log_sending']){
						$this->View->Model->addLog(we_newsletter_newsletter::LOG_DOMAIN_NOK, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[warning]') . ': ' . g_l('modules_newsletter', '[domain_nok]'), $domain);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				} elseif(!$not_black){
					if(!$test && $this->View->settings['log_sending']){
						$this->View->Model->addLog(we_newsletter_newsletter::LOG_MAIL_BLACK, $email);
					}
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[warning]') . ': ' . g_l('modules_newsletter', '[email_is_black]'), $email);
					$progress_update['log'][] = sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $email);
				}
				$ecs++;
			}
		}
		$progress_update['percent'] = ($ecount ? ($ecs / $ecount) * 100 : 0);

		we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $egc, true);

		$this->jsCmd->addCmd('updateLog', $progress_update);
		$this->jsCmd->addCmd('nextMails', (!empty($this->View->settings['send_wait']) && intval($this->View->settings['send_wait']) && $egc && intval($this->View->settings['send_step']) && $egc < ceil($ecount / $this->View->settings['send_step']) ? intval($this->View->settings['send_wait']) : 0), $ecs);
		//$laststep = ceil(we_base_request::_(we_base_request::INT, "ecount", 0) / $this->View->settings['send_step']);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', 'self.focus();'));
	}

	private function getHTMLSendControl(){
		$nid = we_base_request::_(we_base_request::INT, "nid", 0);
		$gcount = we_base_request::_(we_base_request::INT, "gcount", 0);
		$ecount = we_base_request::_(we_base_request::INT, "ecount", 0);
		$blockcache = we_base_request::_(we_base_request::RAW, "blockcache", 0);
		$ecs = we_base_request::_(we_base_request::RAW, "ecs", 0);
		$emailcache = we_base_request::_(we_base_request::RAW, "emailcache", 0);

		$to = (is_numeric($this->View->settings['send_wait']) ? $this->View->settings['send_wait'] : 0) + 40000;

		echo $this->getHTMLDocument(we_html_element::body(['style' => 'margin:10px', 'onload' => "startTimeout();"], we_html_element::form([
					'name' => 'we_form',
					'method' => 'post',
					'target' => 'send_cmd',
					'action' => $this->frameset], we_html_element::hiddens(['mod' => 'newsletter',
						'nid' => $nid,
						'pnt' => "send_cmd",
						'retry' => 1,
						'test' => 0,
						'blockcache' => $blockcache,
						'emailcache' => $emailcache,
						'ecount' => $ecount,
						'gcount' => $gcount,
						'ecs' => $ecs,
						'reload' => 0])
				)
			), we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/sendControl.js', 'self.focus();', ['id' => 'loadVarSendControl', 'data-control' => setDynamicVar([
					'to' => $to
		])]));
		flush();
	}

	/**
	 * returns	a select menu within a html table. to ATTENTION this function is also used in classes object and objectFile !!!!
	 * 			when $withHeadline is true, a table with headline is returned, default is false
	 *
	 * @return	select menue to determine charset
	 * @param	string
	 */
	private function getHTMLCharsetTable(){
		$value = (isset($this->View->Model->Charset) ? $this->View->Model->Charset : '');

		$charsets = we_base_charsetHandler::inst()->getCharsetsForTagWizzard();
		asort($charsets);
		reset($charsets);

		$table = new we_html_table([], 1, 2);
		$table->setColContent(0, 0, we_html_tools::textInput('Charset', $value, '', [], 'text', 100));
		$table->setColContent(0, 1, we_html_tools::select('CharsetSelect', $charsets, 1, $value, false, ['onblur' => "document.forms[0].elements.Charset.value=this.options[this.selectedIndex].value;",
				'onchange' => "document.forms[0].elements.Charset.value=this.options[this.selectedIndex].value;"], true, (self::def_width - 120), false));

		return $table->getHtml();
	}

	/**
	 * Ersetzt die Newsletter Platzthalter
	 *
	 * @author damd
	 * @param String $content
	 * @param String $content_plain
	 * @param Array $customerInfos
	 */
	private function replacePlaceholder(&$content, &$content_plain, $customerInfos){
		$placeholderfieldsmatches = [];
		preg_match_all('/' . we_newsletter_util::PLACEHOLDER_PREFIX . '(.[^#]{1,200})####/', $content, $placeholderfieldsmatches);
		$placeholderfields = $placeholderfieldsmatches[1];
		unset($placeholderfieldsmatches);

		$fromCustomer = (is_array($customerInfos) && isset($customerInfos[8]) && isset($customerInfos[9]) && $customerInfos[9] === 'customer' ?
			array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . intval($customerInfos[8])), we_customer_customer::getEncryptedFields()) :
			[]);

		foreach($placeholderfields as $phf){
			$placeholderReplaceValue = $fromCustomer && isset($fromCustomer[$phf]) ? $fromCustomer[$phf] : "";
			$content = str_replace(we_newsletter_util::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $content);
			$content_plain = str_replace(we_newsletter_util::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $content_plain);
		}
	}

}
