<?php

/**
 * webEdition CMS
 *
 * $Rev: 8b369bcc9479 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:15 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_newsletter_util{
	//set to unset
	public static $REMOVENEWSLETTER_STATUS = null;
	//set to unset
	public static $WRITENEWSLETTER_STATUS = null;
	public static $MAIL = '';
	public static $TITLE = '';
	public static $SALUTATION = '';
	public static $FIRSTNAME = '';
	public static $LASTNAME = '';
	public static $CONFIRMLINK = '';
	public static $HTMLMAIL = false;
	public static $CUSTOMERID = 0;

	const PLACEHOLDER_PREFIX = '####PLACEHOLDER:DB::CUSTOMER_TABLE:';

	static function unsubscribeNewsletter(we_database_base $db, $customer, $emailField, $abos, $paths, $unsubscribe_mail){
		self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
		$unsubscribe_mail = strtolower(preg_replace("|[\r\n,]|", '', $unsubscribe_mail));
		self::$MAIL = $unsubscribe_mail;
		if(!we_check_email($unsubscribe_mail)){
			self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_INVALID; // E-Mail ungueltig
			return false;
		}

		$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE subscribe_mail="' . $db->escape($unsubscribe_mail) . '"');

		$emailExists = ($customer ?
			self::removeFromDB($db, $emailField, $unsubscribe_mail, $abos) :
			self::removeFromFile($paths, $unsubscribe_mail));

		if($emailExists){
			return true;
		}
		self::$REMOVENEWSLETTER_STATUS = self::$REMOVENEWSLETTER_STATUS ?: we_newsletter_base::STATUS_EMAIL_EXISTS;
		return false;
	}

	private static function removeFromDB(we_database_base $db, $emailField, $unsubscribe_mail, array $abos){
		$meta = $db->metadata(CUSTOMER_TABLE);

		$tmp = [];
		foreach($abos as $abo){
			$tmp[] = '"' . $db->escape($abo) . '"';
		}
		$where = ' WHERE ' . $emailField . '="' . $db->escape($unsubscribe_mail) . '"';
		$hash = getHash('SELECT ' . implode(',', $tmp) . ' FROM ' . CUSTOMER_TABLE . $where, $db);
		unset($tmp);
		$update = [];
		if(!$hash){
			return false;
		}
		$emailExists = false;
		foreach($abos as $abo){
			$fieldDefaults = explode(',', (isset($meta[$abo]) ? $meta[$abo]['default'] : ''));
			$aboNeg = count($fieldDefaults) > 1 ? $fieldDefaults[0] : '';

			$dbAbo = $hash[$abo];
			if(!empty($dbAbo) || $dbAbo != $aboNeg){
				$update[$abo] = $aboNeg;
				$emailExists = true;
			}
		}
		if($emailExists){
			$fields = ['ModifyDate' => sql_function('UNIX_TIMESTAMP()'),
				'ModifiedBy' => 'frontend',
			];
			$hook = new we_hook_base('customer_preSave', '', ['customer' => &$fields, 'from' => 'tag', 'type' => 'modify', 'tagname' => 'addDelNewsletterEmail', 'isSubscribe' => false,
				'isUnsubscribe' => true]);
			$hook->executeHook();
			$db->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter(array_merge($update, $fields)) . ' ' . $where);
		}
		return $emailExists;
	}

	private static function removeFromFile(array $paths, $unsubscribe_mail){
		$emailExists = false;
		foreach($paths as $path){
			$path = ($_SERVER['DOCUMENT_ROOT'] . '/' . ltrim($path, '/'));

			if(!file_exists(dirname($path))){
				t_e('file ' . $path . ' doesn\'t exist');
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return false;
			}

			// #4158
			while(($lock = we_base_file::lock(basename(__FILE__))) == false){
				usleep(500000);
			}

			$file = file($path);
			if(!$file){
				we_base_file::unlock($lock);
				continue;
			}

			$fileChanged = false;
			$regex = '|' . preg_quote($unsubscribe_mail, '|') . ',|i';
			foreach($file as $i => $line){
				if(preg_match($regex, $line)){
					$emailExists = true;
					unset($file[$i]);
					$fileChanged = true;
				}
			}

			if($fileChanged && !we_base_file::save($path, implode("\n", array_map('trim', $file)) . "\n")){
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return false;
			}
			we_base_file::unlock($lock);
		}
		return $emailExists;
	}

	static function getNewsletterFields($confirmid, &$errorcode, $mail, $subscribe_mail){
		$errorcode = we_newsletter_base::STATUS_SUCCESS;
		if($confirmid){
			if(preg_match('|^([a-f0-9]){32}$|', $confirmid)){
				$h = getHash('SELECT * FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE confirmID=x\'' . $GLOBALS['DB_WE']->escape($confirmid) . '\' AND subscribe_mail="' . $GLOBALS['DB_WE']->escape(strtolower($mail)) . '"');
			} else {
				t_e('confirmid was not valid hexstring of length 32', $confirmid);
				$h = [];
			}
			if(!$h){
				$errorcode = we_newsletter_base::STATUS_CONFIRM_FAILED;
			}
			return $h;
		}

		$subscribe_mail = trim(preg_replace("|[\r\n,]|", '', $subscribe_mail));
		if(!$subscribe_mail){
			$errorcode = we_newsletter_base::STATUS_EMAIL_INVALID;
			return [];
		}

		if(!we_check_email($subscribe_mail)){
			$errorcode = we_newsletter_base::STATUS_EMAIL_INVALID; // E-Mail ungueltig
			return [];
		}

		return [
			'subscribe_mail' => $subscribe_mail,
			'subscribe_html' => we_base_request::_(we_base_request::BOOL, 'we_subscribe_html__'),
			'subscribe_salutation' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::HTML, 'we_subscribe_salutation__', ''))),
			'subscribe_title' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::HTML, 'we_subscribe_title__', ''))),
			'subscribe_firstname' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::HTML, 'we_subscribe_firstname__', ''))),
			'subscribe_lastname' => trim(preg_replace("|[\r\n,]|", '', we_base_request::_(we_base_request::HTML, 'we_subscribe_lastname__', '')))
		];
	}

	static function mailNewSuccessfullNewsletterActiviation($adminmailid, $adminemail, $adminsubject, $charset, array $f, $includeimages){
		if(!($adminmailid && $adminemail)){//inform admin of the new account
			return;
		}
		$db = $GLOBALS['DB_WE'];
		$phpmail = new we_mail_mail($charset, $adminemail, $adminsubject, $f['subscribe_mail'], $f['subscribe_mail']);

		$adminmailtextHTML = strtr(
			(($adminmailid > 0) && we_base_file::isWeFile($adminmailid, FILE_TABLE, $db) ? we_getDocumentByID($adminmailid, '', $charset) : '')
			, [
			'###MAIL###' => $f['subscribe_mail'],
			'###SALUTATION###' => $f['subscribe_salutation'],
			we_newsletter_base::TITLE_REPLACE_TEXT => $f['subscribe_title'],
			we_newsletter_base::FIRSTNAME_REPLACE_TEXT => $f['subscribe_firstname'],
			we_newsletter_base::LASTNAME_REPLACE_TEXT => $f['subscribe_lastname'],
			'###HTML###' => $f['subscribe_html'],
		]);
		$phpmail->addHTMLPart($adminmailtextHTML);
		$phpmail->setIsEmbedImages($includeimages);
		$phpmail->buildMessage();
		$phpmail->send();
	}

	public static function addDoubleOptIn(we_database_base $db, $type, $customer_email_field, $f, array $abos, array $paths, $mailid, $includeimages, $expires, $confirmDocumentID, $subject, $from, array $toCC, array $toBCC){
		if(!$mailid){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			return;
		}

		$confirmID = md5(uniqid($f['subscribe_mail'], true));
		$lists = [];
		$emailExistsInOneOfTheLists = false;
		switch($type){
			case 'customer':
				// #5589 start
				if(empty($customer_email_field)){
					t_e('Error in newsletter settings: The field for the customer mail-address is not set');
					return;
				}
				if(($hash = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customer_email_field . '="' . $db->escape($f['subscribe_mail']) . '"', $db))){
					foreach($abos as $cAbo){
						if(empty($hash[$cAbo])){
							$lists[] = $cAbo;
						}
					}
					if(empty($lists)){// subscriber exists in all lists
						$emailExistsInOneOfTheLists = true;
					}
					// #5589 end
				} else {
					$lists = $abos; //#9002
				}

				break;
			case 'csv':
				foreach($paths as $p){
					$realPath = realpath((substr($p, 0, 1) === '/') ? ($_SERVER['DOCUMENT_ROOT'] . $p) : ($_SERVER['DOCUMENT_ROOT'] . '/' . $p));
					if(!file_exists($realPath)){
						t_e('newsletter file not found', $p, $realPath);
						self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						return;
					}
					$file = we_base_file::load($realPath);
					if($file === false){// #5135
						t_e('newsletter file unable to read', $p, $realPath);
						self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
						return;
					}
					if(!preg_match("%[\r\n]" . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file) && !preg_match('%^' . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file)){
						$lists[] = $p;
					}
				}
				if(empty($lists)){// #5135 subscriber exists in all lists
					$emailExistsInOneOfTheLists = true;
				}
				break;
			case 'emailonly':
				$lists = $abos;
				break;
		}
		if($emailExistsInOneOfTheLists){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
			return;
		}

		$db->query('REPLACE INTO ' . NEWSLETTER_CONFIRM_TABLE . ' SET ' .
			we_database_base::arraySetter([
				'confirmID' => sql_function('x\'' . $confirmID . '\''),
				'subscribe_mail' => strtolower($f['subscribe_mail']),
				'subscribe_html' => $f['subscribe_html'],
				'subscribe_salutation' => $f['subscribe_salutation'],
				'subscribe_title' => $f['subscribe_title'],
				'subscribe_firstname' => $f['subscribe_firstname'],
				'subscribe_lastname' => $f['subscribe_lastname'],
				'lists' => implode(',', $lists),
				'expire' => sql_function('NOW() + INTERVAL ' . $expires . ' MINUTE ') // in secs
		]));


		$use_https_refer = f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="use_https_refer"', '', $db);
		$protocol = ($use_https_refer ? 'https://' : 'http://');

		$port = $use_https_refer ? 443 : 80;
		$basehref = $protocol . $_SERVER['SERVER_NAME'] . ':' . $port;

		$cnt = 0;
		$confirmLink = ($confirmDocumentID ? id_to_path($confirmDocumentID, FILE_TABLE) : getScriptName()) . '?' . http_build_query(['confirmID' => $confirmID, 'mail' => $f['subscribe_mail']]);
		if(($urlReplace = we_folder::getUrlReplacements($GLOBALS['DB_WE'], true, true))){
			$confirmLink = str_replace('//', $protocol, preg_replace($urlReplace, array_keys($urlReplace), $confirmLink, -1, $cnt));
		}

		self::$MAIL = $f['subscribe_mail'];
		self::$TITLE = we_newsletter_base::TITLE_REPLACE_TEXT;
		self::$SALUTATION = $f['subscribe_salutation'];
		self::$FIRSTNAME = $f['subscribe_firstname'];
		self::$LASTNAME = $f['subscribe_lastname'];
		self::$CONFIRMLINK = ($cnt == 0 ? $protocol . $_SERVER['SERVER_NAME'] . (($port && ($port != 80)) ? ':' . $port : '') : '') . $confirmLink;

		if($f['subscribe_html']){
			self::$HTMLMAIL = true;

			if(isset($GLOBALS['we_doc'])){
				$mywedoc = $GLOBALS['we_doc'];
				unset($GLOBALS['we_doc']);
			}
			$mailtextHTML = $mailid && we_base_file::isWeFile($mailid, FILE_TABLE, $GLOBALS['DB_WE']) ? we_getDocumentByID($mailid) : '';
			$mailtextHTML = str_replace(we_newsletter_base::TITLE_REPLACE_TEXT, $f['subscribe_title'], ($f['subscribe_title'] ? preg_replace('%([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '%', '${1} ' . $f['subscribe_title'], $mailtextHTML) : $mailtextHTML));
		}

		self::$HTMLMAIL = false;

		if(isset($GLOBALS['we_doc'])){
			if(!isset($mywedoc)){
				$mywedoc = $GLOBALS['we_doc'];
			}
			unset($GLOBALS['we_doc']);
		}
		$charset = '';
		$mailtext = $mailid && we_base_file::isWeFile($mailid, FILE_TABLE, $db) ? we_getDocumentByID($mailid, '', $charset) : '';
		$mailtext = str_replace(we_newsletter_base::TITLE_REPLACE_TEXT, $f['subscribe_title'], ($f['subscribe_title'] ? preg_replace('%([^ ])' . we_newsletter_base::TITLE_REPLACE_TEXT . '%', '${1} ' . $f['subscribe_title'], $mailtext) : $mailtext));

		$placeholderfieldsmatches = [];
		if(preg_match_all('/' . self::PLACEHOLDER_PREFIX . '(.[^#]{1,200})####/', $mailtext, $placeholderfieldsmatches)){
			$placeholderfields = $placeholderfieldsmatches[1];
			unset($placeholderfieldsmatches);
		}
		$placeholderReplaceValue = '';
		if($type === 'customer'){
			$customerHash = array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customer_email_field . '="' . $db->escape($f['subscribe_mail']) . '"', $db), we_customer_customer::getEncryptedFields());
		}
		if(!empty($placeholderfields)){
			foreach($placeholderfields as $phf){
				$placeholderReplaceValue = ($type === 'customer' && isset($customerHash[$phf])) ? $customerHash[$phf] : '';
				$mailtext = str_replace(self::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $mailtext);
				$mailtextHTML = str_replace(self::PLACEHOLDER_PREFIX . $phf . '####', $placeholderReplaceValue, $mailtextHTML);
			}
		}
		$we_recipientCC = [];
		foreach($toCC as $cc){
			if(strpos($cc, '@') === false){
				if(!empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser'][$cc]) && strpos($_SESSION['webuser'][$cc], '@') !== false){ //wenn man registrierten Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$cc])){
						$we_recipientCC[] = $_SESSION['webuser'][$cc];
					}
				} else if(isset($_REQUEST[$cc]) && strpos($_REQUEST[$cc], '@') !== false){ //email to friend test
					if(we_check_email(($cc = we_base_request::_(we_base_request::EMAIL, $cc)))){
						$we_recipientCC[] = $cc;
					}
				}
			} elseif(we_check_email($cc)){
				$we_recipientCC[] = $cc;
			}
		}
		$we_recipientBCC = [];
		foreach($toBCC as $bcc){
			if(strpos($bcc, '@') === false){
				if(!empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser'][$bcc]) && strpos('@', $_SESSION['webuser'][$bcc]) !== false){ //wenn man registrierte Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$bcc])){
						$we_recipientBCC[] = $_SESSION['webuser'][$bcc];
					}
				} else if(isset($_REQUEST[$bcc]) && strpos('@', $_REQUEST[$bcc]) !== false){ //email to friend test
					if(we_check_email(($bcc = we_base_request::_(we_base_request::EMAIL, $bcc)))){
						$we_recipientBCC[] = $bcc;
					}
				}
			} elseif(we_check_email($bcc)){
				$we_recipientBCC[] = $bcc;
			}
		}
		$phpmail = new we_mail_mail($charset, $f['subscribe_mail'], $subject, $from, $from);
		if($includeimages){
			$phpmail->setIsEmbedImages($includeimages);
		} else {
			$phpmail->setBaseDir($basehref);
		}
		if(!empty($we_recipientCC)){
			$phpmail->setCC($we_recipientCC);
		}
		if(!empty($we_recipientBCC)){
			$phpmail->setBCC($we_recipientBCC);
		}

		if($f['subscribe_html']){
			$phpmail->addHTMLPart($mailtextHTML);
		} else {
			$phpmail->addTextPart(trim($mailtext));
		}
		//what about attachments/inlines??
		$phpmail->buildMessage();
		$phpmail->send();
		we_tag_addDelNewsletterEmail::$isDoubleOptIn = true;

		if(isset($mywedoc)){
			$GLOBALS['we_doc'] = $mywedoc;
		}
	}

	static function addToFile(array $paths, $f){
		$emailwritten = false;
		foreach($paths as $p){
			$realPath = realpath($_SERVER['DOCUMENT_ROOT'] . '/' . ltrim($p, '/'));
			if(!file_exists(dirname($realPath)) || strpos(realpath($realPath), realpath($_SERVER['DOCUMENT_ROOT'])) === false){
				self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				self::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
				return;
			}

			while(($lock = we_base_file::lock(basename(__FILE__))) == false){
				usleep(500000);
			}

			if(($file = we_base_file::load($realPath)) && (
				preg_match("%[\r\n]" . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file) ||
				preg_match('%^' . $f['subscribe_mail'] . ",[^\r\n]+[\r\n]%i", $file))){
				// E-Mail schon vorhanden => Nix tun
			} else {
				$row = $f['subscribe_mail'] . ',' . $f['subscribe_html'] . ',' . $f['subscribe_salutation'] . ',' . $f['subscribe_title'] . ',' . $f['subscribe_firstname'] . ',' . $f['subscribe_lastname'] . "\n";
				if(!we_base_file::save($realPath, $row, 'ab+')){
					we_base_file::unlock($lock);
					t_e('save of file ' . $p . ' failed');
					self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR; // FATAL ERROR
					return;
				}
				$emailwritten = true;
				we_base_file::unlock($lock);
			}
		}
		if(!$emailwritten){
			self::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_EMAIL_EXISTS;
		}
	}

	static function addToDB(we_database_base $db, array $customerFieldPrefs, array $f, array $abos){
		$uid = f('SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE ' . $customerFieldPrefs['customer_email_field'] . '="' . $db->escape($f['subscribe_mail']) . '"', '', $db);
		if(!$uid){
			$pass = new we_customer_generatePassword(12, ['caps' => 3, 'small' => 4, 'nums' => 3, 'specs' => 2]);
			$GLOBALS['WE_NEWSUBSCRIBER_PASSWORD'] = $pass->PassGen();
			$GLOBALS['WE_NEWSUBSCRIBER_USERNAME'] = $f['subscribe_mail'];
		}
		$fields = (!$uid ? [
			'Username' => $f['subscribe_mail'],
			'Password' => we_customer_customer::cryptPassword($GLOBALS['WE_NEWSUBSCRIBER_PASSWORD']),
			'MemberSince' => time(),
			'LoginDenied' => 0,
			'LastLogin' => 0,
			'LastAccess' => 0,
			($customerFieldPrefs['customer_salutation_field'] != 'ID' ? $customerFieldPrefs['customer_salutation_field'] : '') => $f['subscribe_salutation'],
			($customerFieldPrefs['customer_title_field'] != 'ID' ? $customerFieldPrefs['customer_title_field'] : '') => $f['subscribe_title'],
			($customerFieldPrefs['customer_firstname_field'] != 'ID' ? $customerFieldPrefs['customer_firstname_field'] : '') => $f['subscribe_firstname'],
			($customerFieldPrefs['customer_lastname_field'] != 'ID' ? $customerFieldPrefs['customer_lastname_field'] : '') => $f['subscribe_lastname'],
			($customerFieldPrefs['customer_email_field'] != 'ID' ? $customerFieldPrefs['customer_email_field'] : '') => $f['subscribe_mail'],
			($customerFieldPrefs['customer_html_field'] != 'ID' ? $customerFieldPrefs['customer_html_field'] : '') => $f['subscribe_html'],
			] : [
			'ModifyDate' => time(),
			'ModifiedBy' => 'frontend',
		]);
		$hook = new we_hook_base('customer_preSave', '', ['customer' => &$fields, 'from' => 'tag', 'type' => (!$uid ? 'new' : 'modify'), 'tagname' => 'addDelNewsletterEmail',
			'isSubscribe' => true, 'isUnsubscribe' => false]);
		$hook->executeHook();

		$db->query($uid ?
				'UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($fields) . ' WHERE ID=' . $uid :
				'INSERT INTO ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($fields));

		$set = [];
		$meta = $db->metadata(CUSTOMER_TABLE);
		foreach($abos as $abo){
			if(!empty($meta[$abo]['default'])){
				$setVals = explode(',', $meta[$abo]['default']);
			} else if(!empty($meta['Newsletter_Ok']['default'])){
				$setVals = explode(',', $meta['Newsletter_Ok']['default']);
			} else {
				$setVals = ['', '1'];
			}

			list($setDefault, $setVal) = (count($setVals) > 1 ? $setVals : ['', $setVals[0]]);

			if(!$db->isColExist(CUSTOMER_TABLE, $abo)){
				$db->addCol(CUSTOMER_TABLE, $abo, 'VARCHAR(200) DEFAULT "' . $db->escape($setDefault) . '"', 'Username');
			}
			$set[$abo] = $setVal;
		}

		if($customerFieldPrefs['customer_html_field'] === 'ID'){
			t_e('warning', 'missing newsletter customer settings', 'no customer html field found in settings: field "ID" is not allowed');
		} else {
			$set[$customerFieldPrefs['customer_html_field']] = $f['subscribe_html'];
		}
		if(!empty($customerFieldPrefs['customer_email_field'])){
			$db->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set) . ' WHERE ' . $customerFieldPrefs['customer_email_field'] . '="' . $db->escape($f['subscribe_mail']) . '"');
		} else {
			t_e('warning', 'missing newsletter customer settings', 'Email address field not set');
		}
	}

}
