<?php

/**
 * webEdition CMS
 *
 * $Rev: 152b436a6cad $
 * $Author: mokraemer $
 * $Date: 2020-02-18 16:01 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */

class we_newsletter_view extends we_modules_view{
	const MAILS_ALL = 0;
	const MAILS_CUSTOMER = 1;
	const MAILS_EMAILS = 2;
	const MAILS_FILE = 3;

	// settings array; format settings[setting_name]=settings_value
	var $settings = [];
	//wat page is currentlly displayed 0-properties(default);1-overview;
	var $page = 0;
	var $get_import = 0;
	var $hiddens = ['ID'];
	var $customers_fields;
	protected $show_import_box = -1;
	protected $show_export_box = -1;

	public function __construct(){
		parent::__construct();

		$this->Model = new we_newsletter_newsletter();

		$this->settings = self::getSettings();
		//FIXME: add types for settings

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->customers_fields = [];
			$this->db->query('SHOW FIELDS FROM ' . CUSTOMER_TABLE);
			while($this->db->next_record()){
				$this->customers_fields[] = $this->db->f('Field');
			}
		}
		$this->Model->Text = g_l('modules_newsletter', '[new_newsletter]');
		$this->Model->Sender = $this->settings['default_sender'];
		$this->Model->Reply = $this->settings['default_reply'];
		$this->Model->Test = $this->settings['test_account'];
		$this->Model->isEmbedImages = $this->settings['isEmbedImages'];
	}

	function getHiddens($predefs = []){
		return we_html_element::hiddens(['mod' => 'newsletter',
				'ncmd' => (isset($predefs['ncmd']) ? $predefs['ncmd'] : 'new_newsletter'),
				'we_cmd[0]' => 'show_newsletter',
				'nid' => (isset($predefs['nid']) ? $predefs['nid'] : $this->Model->ID),
				'pnt' => (isset($predefs['pnt']) ? $predefs['pnt'] : we_base_request::_(we_base_request::STRING, 'pnt')),
				'page' => (isset($predefs['page']) ? $predefs['page'] : $this->page),
				'gview' => (isset($predefs['gview']) ? $predefs['gview'] : 0),
				'hm' => (isset($predefs['hm']) ? $predefs['hm'] : 0),
				'ask' => (isset($predefs['ask']) ? $predefs['ask'] : 1),
				'test' => (isset($predefs['test']) ? $predefs['test'] : 0)
		]);
	}

	function newsletterHiddens(){
		$out = '';
		foreach($this->hiddens as $val){
			$out .= we_html_element::hidden($val, (isset($this->Model->persistents[$val]) ? $this->Model->$val : $this->$val)
			);
		}

		return $out;
	}

	function getHiddensProperty(){
		$val = $out = '';
		$counter = 0;

		foreach($this->Model->groups as $group){
			foreach(array_keys($group->persistents) as $per){
				$val = $group->$per;
				$out .= we_html_element::hidden('group' . $counter . '_' . $per, $val);
			}

			$counter++;
		}

		$out .= we_html_element::hiddens(['groups' => $counter,
				'Step' => $this->Model->Step,
				'Offset' => $this->Model->Offset,
				'IsFolder' => $this->Model->IsFolder]
		);
		return $out;
	}

	function getHiddensPropertyPage(){
		return we_html_element::hiddens(['Text' => $this->Model->Text,
				'Subject' => $this->Model->Subject,
				'ParentID' => $this->Model->ParentID,
				'Sender' => $this->Model->Sender,
				'Reply' => $this->Model->Reply,
				'Test' => $this->Model->Test,
				'Charset' => $this->Model->Charset,
				'isEmbedImages' => $this->Model->isEmbedImages
		]);
	}

	function getHiddensMailingPage(){
		$out = '';

		$fields_names = ['fieldname', 'operator', 'fieldvalue', 'logic', 'hours', 'minutes'];
		foreach($this->Model->groups as $g => $group){
			$filter = $group->getFilter();
			if($filter){
				$out .= we_html_element::hidden('filter_' . $g, count($filter));

				foreach($filter as $k => $v){
					foreach($fields_names as $field){
						if(isset($v[$field])){
							$out .= we_html_element::hidden('filter_' . $field . '_' . $g . '_' . $k, $v[$field]);
						}
					}
				}
			}
		}

		return $out;
	}

	function getHiddensContentPage(){
		$out = '';
		$counter = 0;

		foreach($this->Model->blocks as $bk => $bv){

			foreach(array_keys($this->Model->blocks[$bk]->persistents) as $per){
				$out .= we_html_element::hidden('block' . $counter . '_' . $per, $bv->$per);
			}

			$counter++;
		}

		$out .= we_html_element::hidden('blocks', $counter);

		return $out;
	}

	function getFields($id, $table){
		$ct = f('SELECT ContentType FROM ' . $this->db->escape($table) . ' WHERE ID=' . intval($id), '', $this->db);
		$foo = [];

		if($ct){
			$ent = we_base_ContentTypes::getObject($ct);
			$ent->initByID($id, $table);
			$tmp = array_keys($ent->elements);

			foreach($tmp as $v){
				$foo[$v] = $v;
			}
		}

		return $foo;
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::NEWSLETTER, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', '', ['id' => 'loadVarNewsletter_property', 'data-nlView' => setDynamicVar(array_merge($jsVars, [
					'checkMail' => !empty($this->settings['reject_save_malformed']),
					'uid' => (isset($this->uid) ? $this->uid : ''),
					'testMail' => $this->Model->Test,
				]))]
		);
	}

	function processCommands(we_base_jsCmd $jscmd){
		switch(($ncmd = we_base_request::_(we_base_request::STRING, 'ncmd'))){
			case 'new_newsletter':
				$this->Model = new we_newsletter_newsletter();
				$this->Model->Text = g_l('modules_newsletter', '[new_newsletter]');
				$this->Model->Sender = $this->settings['default_sender'];
				$this->Model->Reply = $this->settings['default_reply'];
				$this->Model->Test = $this->settings['test_account'];
				$this->Model->isEmbedImages = $this->settings['isEmbedImages'];
				$page = we_base_request::_(we_base_request::INT, "page");
				$jscmd->addCmd('loadHeadFooter', ['page' => $page, 'txt' => $this->Model->Text, 'folder' => 0]);

				break;
			case 'new_newsletter_group':
				$this->page = 0;
				$this->Model = new we_newsletter_newsletter();
				$this->Model->IsFolder = "1";
				$this->Model->Text = g_l('modules_newsletter', '[new_newsletter_group]');
				$jscmd->addCmd('loadHeadFooter', ['page' => $page, 'txt' => $this->Model->Text, 'folder' => 1]);
				break;
			case 'add_customer':
				$ngroup = we_base_request::_(we_base_request::STRING, 'ngroup');
				if($ngroup !== false){
					$arr = array_filter(explode(',', $this->Model->groups[$ngroup]->Customers));
					if(($ncust = we_base_request::_(we_base_request::STRING_LIST, "ncustomer") ) !== false){

						foreach($ncust as $id){
							if($id && (!in_array($id, $arr))){
								$arr[] = $id;
							}
						}

						$this->Model->groups[$ngroup]->Customers = implode(',', $arr);
					}
				}
				break;

			case 'del_customer':
				$arr = [];
				$ngroup = we_base_request::_(we_base_request::STRING, 'ngroup');
				if($ngroup !== false){
					$arr = array_filter(explode(',', $this->Model->groups[$ngroup]->Customers));

					if(($ncust = we_base_request::_(we_base_request::STRING, "ncustomer") ) !== false){
						foreach($arr as $k => $v){
							if($v == $ncust){
								unset($arr[$k]);
							}
						}
						$this->Model->groups[$ngroup]->Customers = implode(',', $arr);
					}
				}
				break;

			case 'add_file':
				$arr = [];
				if(($ngroup = we_base_request::_(we_base_request::STRING, 'ngroup')) !== false){
					$arr = explode(',', $this->Model->groups[$ngroup]->Extern);
					if(($nfile = we_base_request::_(we_base_request::FILE, 'nfile')) !== false){
						$sd = str_replace("\\", "/", $_SERVER['DOCUMENT_ROOT']);
						$arr[] = str_replace($sd, (substr($sd, -1) === '/' ? '/' : ''), $nfile);
						$this->Model->groups[$ngroup]->Extern = implode(',', $arr);
					}
				}
				break;

			case 'del_file':
				$arr = [];
				if(($ngroup = we_base_request::_(we_base_request::STRING, 'ngroup')) !== false){
					$arr = explode(',', $this->Model->groups[$ngroup]->Extern);
					if(($nfile = we_base_request::_(we_base_request::FILE, 'nfile')) !== false){
						if(($pos = array_search($nfile, $arr, false)) !== false){
							unset($arr[$pos]);
						}
						$this->Model->groups[$ngroup]->Extern = implode(',', $arr);
					}
				}
				break;
			case 'del_all_files':
				if(($ngroup = we_base_request::_(we_base_request::STRING, 'ngroup')) !== false){
					$this->Model->groups[$ngroup]->Extern = '';
				}
				break;
			case 'del_all_customers':
				if(($ngroup = we_base_request::_(we_base_request::STRING, 'ngroup')) !== false){
					$this->Model->groups[$ngroup]->Customers = '';
				}
				break;

			case 'reload':
				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text, 'folder' => $this->Model->IsFolder]);
				break;

			case 'newsletter_edit':
				if(($nid = we_base_request::_(we_base_request::INT, 'nid'))){
					$this->Model = new we_newsletter_newsletter($nid);
				}
				if($this->Model->IsFolder){
					$this->page = 0;
				}
				$_REQUEST['ncmd'] = 'reload';
				$this->processCommands($jscmd);
				break;

			case 'switchPage':
				$this->page = we_base_request::_(we_base_request::INT, "page", $this->page);
				break;

			case 'save_newsletter':
				$nid = we_base_request::_(we_base_request::INT, 'nid');
				if($nid === false){
					break;
				}
				$newone = false;

				if($this->Model->filenameNotValid()){
					$jscmd->addMsg(g_l('modules_newsletter', '[we_filename_notValid]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				if($this->Model->ParentID){
					$folder = f('SELECT IsFolder FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . intval($this->Model->ParentID));
					if(empty($folder)){
						$jscmd->addMsg(g_l('modules_newsletter', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
				}

				if(isset($_REQUEST['blocks'])){
					for($i = 0; $i < $_REQUEST['blocks']; $i++){
						switch(we_base_request::_(we_base_request::INT, 'block' . $i . '_Type')){
							case we_newsletter_block::DOCUMENT:
							case we_newsletter_block::DOCUMENT_FIELD:
								$acTable = FILE_TABLE;
								$acErrorField = g_l('modules_newsletter', '[block_document]');
								break;
							case we_newsletter_block::OBJECT:
							case we_newsletter_block::OBJECT_FIELD:
								$acTable = OBJECT_FILES_TABLE;
								$acErrorField = g_l('modules_newsletter', '[block_object]');
								break;
							default:
								$acTable = '';
								$acErrorField = '';
						}
						if($acTable){

							$folder = f('SELECT IsFolder FROM ' . $acTable . ' WHERE ID=' . we_base_request::_(we_base_request::INT, 'block' . $i . '_LinkID'));
							if($folder === '' || $folder == 1){
								$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[blockFieldError]'), ($i + 1), $acErrorField), we_base_util::WE_MESSAGE_ERROR);
								return;
							}
							if(($field = we_base_request::_(we_base_request::INT, 'block' . $i . '_Field'))){
								$folder = f('SELECT IsFolder FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($field));
								if($folder === '' || $folder == 1){
									$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[blockFieldError]'), $i, g_l('modules_newsletter', '[block_template]')), we_base_util::WE_MESSAGE_ERROR);
									return;
								}
							}
						}
					}
				}

				if(!$this->Model->ID){
					$newone = true;
				}

				if(!$newone && we_base_request::_(we_base_request::BOOL, 'ask')){
					$h = getHash('SELECT Step,Offset FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . intval($this->Model->ID), $this->db);

					if($h['Step'] != 0 || $h['Offset'] != 0){
						$jscmd->addCmd("save_newsletter_question");
						break;
					}
				}

				if(!$this->Model->Sender){
					$this->Model->Sender = $this->settings['default_sender'];
				}

				if(!$this->Model->Reply){
					$this->Model->Reply = $this->settings['default_reply'];
				}

				if(!$this->Model->Test){
					$this->Model->Test = $this->settings['test_account'];
				}
				if(!$this->Model->isEmbedImages){
					$this->Model->isEmbedImages = $this->settings['isEmbedImages'];
				}

				$double = intval(f('SELECT COUNT(1) FROM ' . NEWSLETTER_TABLE . ' WHERE Path="' . $this->db->escape($this->Model->Path) . '"' . ($newone ? '' : ' AND ID!=' . $this->Model->ID), '', $this->db));

				if(!we_base_permission::hasPerm([we_base_perm::NEWSLETTER_EDIT, we_base_perm::NEWSLETTER_NEW])){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if($newone && !we_base_permission::hasPerm(we_base_perm::NEWSLETTER_NEW)){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(!$newone && !we_base_permission::hasPerm(we_base_perm::NEWSLETTER_EDIT)){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				if($double){
					$jscmd->addMsg(g_l('modules_newsletter', '[double_name]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				$message = '';

				$ret = $this->Model->saveNewsletter($message, (isset($this->settings['reject_save_malformed']) ? $this->settings['reject_save_malformed'] : true));
				switch($ret){
					default:
						$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_group]'), $ret, $message), we_base_util::WE_MESSAGE_ERROR);
						break;
					case we_newsletter_newsletter::MALFORMED_SENDER:
						$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_sender]'), $message), we_base_util::WE_MESSAGE_ERROR);
						break;
					case we_newsletter_newsletter::MALFORMED_REPLY:
						$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_reply]'), $message), we_base_util::WE_MESSAGE_ERROR);
						break;
					case we_newsletter_newsletter::MALFORMED_TEST:
						$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_test]'), $message), we_base_util::WE_MESSAGE_ERROR);
						break;
					case we_newsletter_newsletter::SAVE_PATH_NOK:
						$jscmd->addMsg($message, we_base_util::WE_MESSAGE_ERROR);
						break;
					case 0:
						if($newone){
							$jscmd->addCmd('makeTreeEntry', [
								'id' => $this->Model->ID,
								'parentid' => $this->Model->ParentID,
								'text' => $this->Model->Text,
								'open' => false,
								'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER : we_base_ContentTypes::NEWSLETTER),
								'table' => NEWSLETTER_TABLE
							]);
						} else {
							$jscmd->addCmd('updateTreeEntry', [
								'id' => $this->Model->ID,
								'parentid' => $this->Model->ParentID,
								'text' => $this->Model->Text,
							]);
						}
						$jscmd->addCmd('drawTree');
						$jscmd->addCmd('unsetHot');
						$jscmd->addMsg(g_l('modules_newsletter', ($this->Model->IsFolder == 1 ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);
						break;
				}

				break;

			case 'delete_newsletter':
				$nid = we_base_request::_(we_base_request::INT, 'nid');
				if($nid === false){
					$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::_(we_base_request::BOOL, 'IsFolder') ? '[delete_group_nok]' : '[delete_nok]')), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(!$nid){
					$jscmd->addMsg(g_l('modules_newsletter', '[delete_nok]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(!we_base_permission::hasPerm(we_base_perm::NEWSLETTER_DELETE)){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$this->Model = new we_newsletter_newsletter($nid);

				if(!$this->Model->delete()){
					$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::_(we_base_request::BOOL, 'IsFolder') ? '[delete_group_nok]' : '[delete_nok]')), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				$this->Model = new we_newsletter_newsletter();
				$jscmd->addCmd('deleteTreeEntry', [$nid, "file"]);
				$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::_(we_base_request::BOOL, 'IsFolder') ? '[delete_group_ok]' : '[delete_ok]')), we_base_util::WE_MESSAGE_NOTICE);
				$_REQUEST['home'] = 1;
				$_REQUEST['pnt'] = 'edbody';
				break;

			case 'reload_table':
				$this->page = 1;
				break;

			case 'set_import':
				$this->show_import_box = we_base_request::_(we_base_request::STRING, 'ngroup');
				break;

			case 'set_export':
				$this->show_export_box = we_base_request::_(we_base_request::STRING, 'ngroup');
				break;

			case 'reset_import':
				$this->show_import_box = -1;
				break;

			case 'reset_export':
				$this->show_export_box = -1;
				break;

			case 'addBlock':
				if(($bid = we_base_request::_(we_base_request::INT, 'blockid')) !== false){
					$this->Model->addBlock($bid + 1);
				}
				break;

			case 'delBlock':
				if(($bid = we_base_request::_(we_base_request::INT, 'blockid')) !== false){
					$this->Model->removeBlock($bid);
				}
				break;

			case 'addGroup':
				$this->Model->addGroup();
				$jscmd->addCmd('groups_changed');
				break;

			case 'delGroup':
				if(($ngroup = we_base_request::_(we_base_request::STRING, 'ngroup')) !== false){
					$this->Model->removeGroup($ngroup);
					$jscmd->addCmd('groups_changed');
				}
				break;

			case 'send_test':
				if(!we_base_permission::hasPerm(we_base_perm::NEWSLETTER_SEND_TEST_EMAIL)){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$this->sendTestMail(we_base_request::_(we_base_request::INT, 'gview', 0), we_base_request::_(we_base_request::BOOL, "hm"));
				$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[test_mail_sent]'), $this->Model->Test), we_base_util::WE_MESSAGE_NOTICE);
				break;

			case 'add_filter':
				$this->Model->groups[we_base_request::_(we_base_request::STRING, 'ngroup')]->addFilter($this->customers_fields[0], we_base_request::_(we_base_request::INT, 'noCustFilter'));
				break;

			case 'del_filter':
				$this->Model->groups[we_base_request::_(we_base_request::STRING, 'ngroup')]->delFilter(we_base_request::_(we_base_request::INT, 'noCustFilter'));
				break;

			case 'del_all_filters':
				$this->Model->groups[we_base_request::_(we_base_request::STRING, 'ngroup')]->delallFilter();
				break;

			case 'copy_newsletter':
				if(($cid = we_base_request::_(we_base_request::INT, 'copyid'))){
					$id = $this->Model->ID;
					$this->Model = new we_newsletter_newsletter($cid);
					$this->Model->ID = $id;
					$this->Model->Text .= "_" . g_l('modules_newsletter', '[copy]');
				}
				break;

			case 'save_settings':
				foreach($this->settings as $k => $v){
					$this->settings[$k] = we_base_request::_(we_base_request::RAW, $k, 0);
				}
				$this->saveSettings();
				break;

			case 'import_csv':
				if(($importno = we_base_request::_(we_base_request::INT, 'csv_import')) !== false){
					$filepath = we_base_request::_(we_base_request::FILE, 'csv_file' . $importno);
					$delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter' . $importno);
					$col = max(0, we_base_request::_(we_base_request::INT, 'csv_col' . $importno, 1) - 1);

					$imports = ['hmcol' => [],
						'salutationcol' => [],
						'titlecol' => [],
						'firstnamecol' => [],
						'lastnamecol' => [],
					];
					foreach($imports as $key => &$vals){
						$vals['val'] = we_base_request::_(we_base_request::INT, 'csv_' . $key . $importno, 0);
						$vals['import'] = ($vals['val'] > 0);
						if($vals['val']){
							$vals['val']--;
						}
					}

					if(strpos($filepath, '..') !== false){
						$jscmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
					} else {
						$row = [];
						$control = [];
						$fh = @fopen($_SERVER['DOCUMENT_ROOT'] . $filepath, 'rb');
						$mailListArray = [];
						if($fh){
							$mailListArray = explode("\n", $this->Model->groups[$importno]->Emails);
							foreach($mailListArray as $line){
								$mailListArray[] = substr($line, 0, strpos($line, ','));
							}

							while(($dat = fgetcsv($fh, 1000, $delimiter))){
								if(!isset($control[$dat[$col]])){
									$alldat = implode('', $dat);
									if(str_replace(' ', '', $alldat) === ''){
										continue;
									}
									$mailrecip = (str_replace(' ', '', $dat[$col]) === '') ? '--- ' . g_l('modules_newsletter', '[email_missing]') . ' ---' : $dat[$col];
									if(!empty($mailrecip) && !in_array($mailrecip, $mailListArray)){
										$row[] = $mailrecip . ',' .
											( ($imports['hmcol']['import'] && isset($dat[$imports['hmcol']['val']])) ? $dat[$imports['hmcol']['val']] : '') . "," .
											( ($imports['salutationcol']['import'] && isset($dat[$imports['salutationcol']['val']])) ? $dat[$imports['salutationcol']['val']] : '') . "," .
											( ($imports['titlecol']['import'] && isset($dat[$imports['titlecol']['val']])) ? $dat[$imports['titlecol']['val']] : '') . "," .
											( ($imports['firstnamecol']['import'] && isset($dat[$imports['firstnamecol']['val']])) ? $dat[$imports['firstnamecol']['val']] : '') . "," .
											( ($imports['lastnamecol']['import'] && isset($dat[$imports['lastnamecol']['val']])) ? $dat[$imports['lastnamecol']['val']] : '');
										$control[$dat[$col]] = 1;
									}
								}
							}
							fclose($fh);
							$this->Model->groups[$importno]->Emails .= ($this->Model->groups[$importno]->Emails ? "\n" : '') . implode("\n", $row);
						} else {
							$jscmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
						}
					}
				}
				break;

			case 'export_csv':
				if(($exportno = we_base_request::_(we_base_request::INT, 'csv_export')) !== false){
					$fname = rtrim(we_base_request::_(we_base_request::FILE, 'csv_dir' . $exportno), '/') . '/emails_export_' . time() . '.csv';

					we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $fname, $this->Model->groups[$exportno]->Emails);
					$jscmd->addCmd('export_csv_window', $fname);
				}
				break;

			case 'save_black':
				$this->saveSetting('black_list', $this->settings['black_list'], new DB_WE());
				$jscmd->addCmd('close');
				break;

			case 'do_upload_csv':
			case 'do_upload_black':
				$group = we_base_request::_(we_base_request::INT, 'group', 0);

				//set header we avoided when sending JSON only
				we_html_tools::headerCtCharset('text/html', $GLOBALS['we']['Charset']['Backend']);
				echo we_html_tools::getHtmlTop('newsletter');

				$tempName = '';
				$fileUploader = new we_fileupload_resp_base();
				if(!($tempName = $fileUploader->commitUploadedFile())){
					//some reaction on upload failure
				}

				//print next command
				$jscmd->addCmd('processAfterUpload', ['cmd' => $ncmd, 'name' => $tempName, 'group' => $group]);
				break;

			case 'save_email_file':
				$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', '');
				$nrid = we_base_request::_(we_base_request::INT, 'nrid', '');
				$email = we_base_request::_(we_base_request::EMAIL, 'email', '');
				$htmlmail = we_base_request::_(we_base_request::BOOL, 'htmlmail', '');
				$salutation = we_base_request::_(we_base_request::STRING, 'salutation', '');
				$title = we_base_request::_(we_base_request::STRING, 'title', '');
				$firstname = we_base_request::_(we_base_request::STRING, 'firstname', '');
				$lastname = we_base_request::_(we_base_request::STRING, 'lastname', '');

				$emails = ($csv_file ? we_newsletter_newsletter::getEmailsFromExtern($csv_file) : []);

				$emails[$nrid] = [$email, $htmlmail, $salutation, $title, $firstname, $lastname];
				$emails_out = '';
				foreach($emails as $email){
					$emails_out .= implode(',', array_slice($email, 0, 6)) . "\n";
				}

				if($csv_file){
					we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $csv_file, $emails_out);
				}

				break;

			case 'delete_email_file':
				$nrid = we_base_request::_(we_base_request::INT, 'nrid', false);
				$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', '');
				$emails = ($csv_file ? we_newsletter_newsletter::getEmailsFromExtern($csv_file, 2) : []);

				if($nrid !== false){
					unset($emails[$nrid]);
					$emails_out = '';
					foreach($emails as $email){
						$emails_out .= implode(',', $email) . "\n";
					}

					if($csv_file){
						we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $csv_file, $emails_out);
					}
				}
				break;
			case 'popSend':
				$jscmd->addCmd('popSend_do_cont', [$this->Model->ID, empty(trim($this->Model->Subject)), we_base_request::_(we_base_request::BOOL, 'test')]);
				break;
			default:
		}
	}

	private function processVariablesGroups($groups){
		$this->Model->groups = [];

		if($groups == 0){
			$this->Model->addGroup();
		}

		for($i = 0; $i < $groups; $i++){
			$this->Model->addGroup();
		}

		$fields_names = ['fieldname', 'operator', 'fieldvalue', 'logic', 'hours', 'minutes'];

		foreach($this->Model->groups as $gkey => &$gval){
			// persistens
			$gval->NewsletterID = $this->Model->ID;

			foreach($gval->persistents as $per => $type){
				$varname = 'group' . $gkey . '_' . $per;
				$gval->$per = we_base_request::_($type, $varname, $gval->$per);
			}

			// Filter
			$count = (isset($_REQUEST['filter_' . $gkey]) ? $_REQUEST['filter_' . $gkey]++ : 0);
			if($count){
				for($i = 0; $i < $count; $i++){
					$new = [];

					foreach($fields_names as $field){
						$varname = 'filter_' . $field . '_' . $gkey . '_' . $i;

						if(($tmp = we_base_request::_(we_base_request::RAW_CHECKED, $varname)) !== false){
							$new[$field] = $tmp;
						}
					}

					if($new){
						$gval->appendFilter($new);
						$gval->preserveFilter();
					}
				}
			}
		}
		unset($gval);
	}

	private function processVariablesBlocks($blocks){
		$this->Model->blocks = [];

		if($blocks == 0){
			$this->Model->addBlock();
		}

		for($i = 0; $i < $blocks; $i++){
			$this->Model->addBlock();
		}

		foreach($this->Model->blocks as $skey => &$sval){
			$sval->NewsletterID = $this->Model->ID;

			foreach($sval->persistents as $per => $type){
				$varname = 'block' . $skey . '_' . $per;
				$sval->$per = we_base_request::_($type, $varname, $sval->$per);
			}
		}
	}

	function processVariables(){
		if(($uid = we_base_request::_(we_base_request::STRING, 'wname'))){
			$this->uid = $uid;
		}

		foreach($this->Model->persistents as $val => $type){
			$this->Model->$val = we_base_request::_($type, $val, $this->Model->$val);
		}

		if($this->Model->ParentID){
			$this->Model->Path = f('SELECT Path FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . $this->Model->ParentID, '', $this->db) . '/' . $this->Model->Text;
		} elseif(!$this->Model->filenameNotValid()){
			$this->Model->Path = '/' . $this->Model->Text;
		}

		$this->page = we_base_request::_(we_base_request::INT, 'page', $this->page);
		if(($groups = we_base_request::_(we_base_request::INT, 'groups')) !== false){
			$this->processVariablesGroups($groups);
		}
		if(($blocks = we_base_request::_(we_base_request::INT, 'blocks')) !== false){
			$this->processVariablesBlocks($blocks);
		}
	}

	/**
	 * Newsletter printing functions
	 */
	private function initDocByObject($we_objectID){
		$we_obj = new we_objectFile();
		$we_obj->initByID($we_objectID, OBJECT_FILES_TABLE);

		$we_doc = $this->initDoc();
		$we_doc->initByObj($we_obj);
		return $we_doc;
	}

	private function initDoc($id = 0){
		$we_doc = new we_webEditionDocument();

		if($id){
			$we_doc->initByID($id);
		}
		return $we_doc;
	}

	function getContent($pblk = 0, $gview = 0, $hm = 0, $salutation = '', $title = '', $firstname = '', $lastname = '', $customerid = 0){
		if(!isset($this->Model->blocks[$pblk])){
			return '';
		}
		$block = $this->Model->blocks[$pblk];
		$groups = makeArrayFromCSV($block->Groups);
		if(!(in_array($gview, $groups) || $gview == 0)){
			return '';
		}

		$content = $GLOBALS['we_doc'] = '';

		we_newsletter_util::$MAIL = we_newsletter_base::EMAIL_REPLACE_TEXT;
		we_newsletter_util::$HTMLMAIL = $hm;
		we_newsletter_util::$TITLE = $title;
		we_newsletter_util::$SALUTATION = $salutation;
		we_newsletter_util::$FIRSTNAME = $firstname;
		we_newsletter_util::$LASTNAME = $lastname;
		we_newsletter_util::$CUSTOMERID = $customerid;
		$patterns = [];

		switch($block->Type){
			case we_newsletter_block::DOCUMENT:
				$path = (!empty($block->Field) ?
					TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', id_to_path($block->Field, TEMPLATES_TABLE)) :
					($block->LinkID ?
					TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', f('SELECT t.Path FROM ' . FILE_TABLE . ' f LEFT JOIN ' . TEMPLATES_TABLE . ' t ON f.TemplateID=t.ID WHERE f.ID=' . intval($block->LinkID), '', $this->db)) :
					''));

				if($block->LinkID && $path){
					$content = $block->LinkID && we_base_file::isWeFile($block->LinkID, FILE_TABLE, $this->db) ? we_getDocumentByID($block->LinkID, $path) : 'No such File';
				}
				break;
			case we_newsletter_block::DOCUMENT_FIELD:
				if($block->LinkID){
					$we_doc = $this->initDoc($block->LinkID);
					$content = $we_doc->getElement($block->Field);
				}
				break;
			case we_newsletter_block::OBJECT:
				$path = (!empty($block->Field) ? TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', id_to_path($block->Field, TEMPLATES_TABLE)) : '');

				if($block->LinkID && $path){
					$content = self::we_getObjectFileByID($block->LinkID, $path);
				}

				break;
			case we_newsletter_block::OBJECT_FIELD:
				if($block->LinkID){
					$we_doc = $this->initDocByObject($block->LinkID);
					$content = $we_doc->getElement($block->Field);
				}
				break;
			case we_newsletter_block::TEXT:
				$blockHtml = $block->Html ? preg_replace(['/(href=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/',
						'/(src=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/'], '${1}${3}${5}', stripslashes($block->Html)) : '';

				if($hm){
					$content = $blockHtml ?
						$blockHtml :
						strtr($block->Source, [
							"\r\n" => '<br/>',
							"\r" => '<br/>',
							'&' => '&amp;',
							'<' => '&lt;',
							'>' => '&gt;',
							"\n" => '<br/>',
							"\t" => str_repeat('&nbsp;', 3),
					]);
					break;
				}
				$content = ($block->Source ?
					$block->Source :
					str_ireplace(['&nbsp;', '&lt;', "&gt;", "&quot;", "&amp;",], [' ', "<", ">", '"', "&",], preg_replace("|&nbsp;(&nbsp;)+|i", "\t", trim(strip_tags(preg_replace("|<br\s*/?\s*>|i", "\n", $blockHtml))))));
				//TODO: we should preserve img- and link-pathes: "text text linktext (path) text"

				break;
			case we_newsletter_block::FILE:
				$content = we_base_file::load($_SERVER['DOCUMENT_ROOT'] . $block->Field);
				if(!$content){
					echo g_l('modules_newsletter', '[cannot_open]') . ": " . $_SERVER['DOCUMENT_ROOT'] . $block->Field;
				}
				break;
			case we_newsletter_block::URL:
				if($block->Field){
					if(substr(trim($block->Field), 0, 4) != 'http'){
						$block->Field = 'http://' . $block->Field;
					}

					$url = parse_url($block->Field);
					$st = '';
					$content = getHTTP($url['host'], (isset($url['path']) ? $url['path'] : '/'), $st, '', defined('HTTP_USERNAME') ? HTTP_USERNAME : '', defined('HTTP_PASSWORD') ? HTTP_PASSWORD : '');

					$patterns[] = "/<(img\s*[^>]+src\s*[\=\"|\=\'|\=\\\\|\=]*\s*)([^\'\"> ? \\\]*)([^\"\' \\\\>]*)(\s*[^>]*)>/si";
					$patterns[] = "/<(link\s*[^>]+href\s*[\=\"|\=\'|\=\\\\|\=]*\s*)([^\'\"> ? \\\]*)([^\"\' \\\\>]*)(\s*[^>]*)>/si";
					$match = [];

					foreach($patterns as $pattern){
						if(preg_match_all($pattern, $content, $match)){
							$unique = array_unique($match[2]);
							foreach($unique as $name){
								$src = parse_url($name);

								if(!isset($src['host'])){

									if(isset($src['path'])){
										$path = (dirname($src['path']) ?
											dirname($src['path']) . '/' :
											(isset($url['path']) ?
											dirname($url['path']) . '/' :
											''));
									}
									$newname = $url['scheme'] . '://' . preg_replace('|/+|', '/', $url['host'] . '/' . $path . basename($name));
									$content = str_replace($name, $newname, $content);
								}
							}
						}
					}
				}
				break;
			case we_newsletter_block::ATTACHMENT:
				break;
		}

		$port = (!empty($this->settings['use_port'])) ? ':' . $this->settings['use_port'] : '';
		$protocol = (!empty($this->settings['use_https_refer']) ? 'https://' : 'http://');

		if($hm){
			if($block->Type != we_newsletter_block::URL){
				$spacer = '\s*';

				$content = we_contents_document::parseInternalLinks($content, 0);

				$urlReplace = we_folder::getUrlReplacements($this->db, false, true);
				if($urlReplace){
					$content = preg_replace('-(["\'])//-', '${1}' . $protocol, preg_replace($urlReplace, array_keys($urlReplace), $content));
				}
				return preg_replace(['-(<[^>]+src' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(<[^>]+href' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(<[^>]+background' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(background' . $spacer . ':' . $spacer . '[^url]*url' . $spacer . '\\([\'"]?)(/)-i',
					'+(background-image' . $spacer . ':' . $spacer . '[^url]*url' . $spacer . '\\([\'"]?)(/)+i',
					], ['${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					], $content);
			}
			return $content;
		}
		$urlReplace = we_folder::getUrlReplacements($this->db, true, true);
		if($urlReplace){
			$content = str_replace('//', $protocol, preg_replace($urlReplace, array_keys($urlReplace), $content));
		}
		$newplain = preg_replace(['|<br */? *>|', '|<title>.*</title>|i', "|&nbsp;(&nbsp;)+|i"], ["\n", '', "\t"], $content);
		if($block->Type != we_newsletter_block::TEXT){
			$newplain = strip_tags($newplain);
		}
		return str_ireplace(['&nbsp;', '&lt;', '&gt;', '&quot;', '&amp;',], [' ', '<', '>', '"', '&'], $newplain);
	}

	function getGroupBlocks($group){
		$content = [];
		$count = count($this->Model->blocks);
		if($group == 0){
			for($i = 0; $i < $count; $i++){
				$content[] = $i;
			}
		} else {
			foreach($this->Model->blocks as $kblock => $block){
				if(in_array($group, $block->GroupsA)){
					$content[] = $kblock;
				}
			}
		}
		return $content;
	}

	function getGroupsForEmail($email){
		$ret = [];

		if(is_array($this->Model->groups)){
			$keys = array_keys($this->Model->groups);
			foreach($keys as $gk){
				$emails = $this->getEmails($gk + 1, self::MAILS_ALL, 1);

				if(in_array($email, $emails)){
					$ret[] = $gk + 1;
				}
			}
		}

		return $ret;
	}

	function getAttachments($group){
		return $this->db->getAllq('SELECT CONCAT("' . $this->db->escape($_SERVER['DOCUMENT_ROOT']) . '",f.Path) FROM ' . NEWSLETTER_BLOCK_TABLE . ' b JOIN ' . FILE_TABLE . ' f ON (f.ID=b.LinkID) WHERE b.NewsletterID=' . $this->Model->ID . ' AND b.Type=' . we_newsletter_block::ATTACHMENT . ($group ? ' AND FIND_IN_SET("' . $this->db->escape($group) . '",b.Groups)' : ''), true);
	}

	function sendTestMail($group, $hm){
		$plain = $content = '';
		$inlines = [];

		$ret = $this->cacheNewsletter($this->Model->ID, false);
		$blocks = $this->getGroupBlocks($group);
		foreach($blocks as $i){
			if($hm){
				$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_h_' . $i);
				$inlines = array_merge($inlines, $block['inlines']);
				$content .= $block['default'];
				$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_p_' . $i);
				$plain .= $block['default'];
			} else {
				$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_p_' . $i);
				$content .= $block['default'];
				$plain .= $block['default'];
			}
		}

		$atts = $this->getAttachments($group);
		//$clean = $this->getCleanMail($this->newsletter->Reply);
		$phpmail = new we_mail_mail($this->Model->Charset ?: $GLOBALS['we']['Charset']['Backend'], $this->Model->Test, $this->Model->Subject, $this->Model->Sender, $this->Model->Reply, $this->Model->isEmbedImages);
		if(!$this->settings['use_base_href']){
			$phpmail->setIsUseBaseHref($this->settings['use_base_href']);
		}
		if($hm){
			$phpmail->addHTMLPart($content);
		}
		$phpmail->addTextPart(trim($plain));
		foreach($atts as $att){
			$phpmail->doaddAttachment($att);
		}
		//FIXME: what abpout inlines???

		$phpmail->buildMessage();
		$phpmail->send();

		we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache']);
	}

	function getFilterSQL($filter){
		$filterSQL = $filter['fieldname'];
		switch($filter['fieldname']){
			case 'MemberSince':
			case 'LastLogin':
			case 'LastAccess':
				if(stristr($filter['fieldvalue'], '.')){
					$date = explode('.', $filter['fieldvalue']);
					$day = $date[0];
					$month = $date[1];
					$year = $date[2];
					$hour = $filter['hours'];
					$minute = $filter['minutes'];
					$filter['fieldvalue'] = mktime($hour, $minute, 0, $month, $day, $year);
				}
		}

		switch($filter['operator']){
			case we_newsletter_newsletter::OP_EQ:
				return $filterSQL . '="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_NEQ:
				return $filterSQL . '!="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LE:
				return $filterSQL . '<"' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LEQ:
				return $filterSQL . '<="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_GE:
				return $filterSQL . '>"' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_GEQ:
				return $filterSQL . '>="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LIKE:
				return $filterSQL . ' LIKE "' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_CONTAINS:
				return $filterSQL . ' LIKE "%' . $filter['fieldvalue'] . '%"';
			case we_newsletter_newsletter::OP_STARTS:
				return $filterSQL . ' LIKE "' . $filter['fieldvalue'] . '%"';
			case we_newsletter_newsletter::OP_ENDS:
				return $filterSQL . ' LIKE "%' . $filter['fieldvalue'] . '"';
			default:
				return $filterSQL;
		}
	}

	function getEmails($group, $select = self::MAILS_ALL, $emails_only = 0){

		update_time_limit(0);

		$extern = ($select == self::MAILS_ALL || $select == self::MAILS_FILE) ?
			we_newsletter_base::getEmailsFromExtern($this->Model->groups[$group - 1]->Extern, $emails_only, $group, $this->getGroupBlocks($group)) :
			[];

		if($select == self::MAILS_FILE){
			return $extern;
		}

		$list = ($select == self::MAILS_ALL || $select == self::MAILS_EMAILS) ?
			we_newsletter_base::getEmailsFromList($this->Model->groups[$group - 1]->Emails, $emails_only, $group, $this->getGroupBlocks($group)) :
			[];
		if($select == self::MAILS_EMAILS){
			return $list;
		}

		$customer_mail = $customers = [];

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$filterarr = [];
			$filtera = $this->Model->groups[$group - 1]->getFilter();
			if($filtera){
				foreach($filtera as $k => $filter){
					$filterarr[] = ($k ? (' ' . $filter['logic'] . ' ') : ' ') . $this->getFilterSQL($filter);
				}
			}

			$filtersql = implode(' ', $filterarr);

			$customers = ($this->Model->groups[$group - 1]->SendAll ?
				'SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE ' . ($filtersql !== '' ? $filtersql : 1) :
				implode(',', array_map('intval', explode(',', $this->Model->groups[$group - 1]->Customers))));

			if(!empty($customers) || !empty($filtersql)){ //Fix #10898
				$default_html = f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"', '', $this->db);

				$selectX = array_filter(array_unique([
					($email_field = $this->settings['customer_email_field']),
					($html_field = ($this->settings['customer_html_field'] === 'ID' ? '' : $this->settings['customer_html_field'])),
					($salutation_field = ($this->settings['customer_salutation_field'] === 'ID' ? '' : $this->settings['customer_salutation_field'])),
					($title_field = ($this->settings['customer_title_field'] === 'ID' ? '' : $this->settings['customer_title_field'])),
					($firstname_field = $this->settings['customer_firstname_field']),
					($lastname_field = $this->settings['customer_lastname_field'])
				]));

				if($emails_only){
					$customer_mail = $email_field ? $this->db->getAllq('SELECT ' . $email_field . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . $customers . ') AND ' . $email_field . '!="" ' . ($filtersql ? ' AND (' . $filtersql . ')' : ''), true) : [];
				} elseif($this->db->query('SELECT ID,' . $this->db->escape(implode(',', $selectX)) . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . $customers . ') AND ' . $email_field . '!="" ' . ($filtersql ? ' AND (' . $filtersql . ')' : ''))){
					while($this->db->next_record()){
						$customer_mail[] = [
							trim($this->db->f($email_field)),
							($html_field && ($tmp = trim($this->db->f($html_field))) != '' ? $tmp : $default_html),
							($salutation_field ? $this->db->f($salutation_field) : ''),
							($title_field ? $this->db->f($title_field) : ''),
							$this->db->f($firstname_field),
							$this->db->f($lastname_field),
							$group,
							$this->getGroupBlocks($group),
							$this->db->f('ID'),
							'customer'
						];
					}
				}
			}

			if($select == self::MAILS_CUSTOMER){
				return $customer_mail;
			}
		}
		return array_merge($customer_mail, $list, $extern);
	}

	function getEmailsNum(){
		$out = 0;
		$count = count($this->Model->groups);
		for($i = 0; $i < $count; $i++){
			$out += count($this->getEmails($i + 1, self::MAILS_ALL, 1));
		}
		return $out;
	}

	/**
	 * Static function - Settings
	 */
	static function getSettings(){
		$db = new DB_WE();
		$domainName = str_replace('www.', '', $_SERVER['SERVER_NAME']);

		$ret = array_merge([
			'black_list' => '',
			'customer_email_field' => '',
			'customer_firstname_field' => 'Forename',
			'customer_html_field' => 'ID',
			'customer_lastname_field' => 'Surname',
			'customer_salutation_field' => '',
			'customer_title_field' => '',
			'default_htmlmail' => 0,
			'isEmbedImages' => 0,
			'default_reply' => 'replay@' . $domainName,
			'default_sender' => 'mailer@' . $domainName,
			we_newsletter_newsletter::FEMALE_SALUTATION_FIELD => g_l('modules_newsletter', '[default][female]'),
			'global_mailing_list' => '',
			'log_sending' => 1,
			we_newsletter_newsletter::MALE_SALUTATION_FIELD => g_l('modules_newsletter', '[default][male]'),
			'reject_malformed' => 1,
			'reject_not_verified' => 1,
			'send_step' => 20,
			'send_wait' => 0,
			'test_account' => 'test@' . $domainName,
			'title_or_salutation' => 0,
			'use_port' => 0,
			'use_https_refer' => 0,
			'use_base_href' => 1
			], $db->getAllFirstq('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter"', false));
		//make sure blacklist is correct
		$ret['black_list'] = implode(',', array_map('trim', explode(',', $ret['black_list'])));

		return $ret;
	}

	private function saveSettings(){
		$db = new DB_WE();
		foreach($this->settings as $key => $value){
			if($key != 'black_list'){
				$this->saveSetting($key, $value, $db);
			}
		}
	}

	public function saveSetting($name, $value, we_database_base $db){
		$db->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter(['tool' => 'newsletter', 'pref_name' => $name, 'pref_value' => $value]));
	}

	function isBlack($email){
		static $black = [];
		if(!$black){
			if(empty($this->settings['black_list'])){
				return false;
			}
			//remove whitespaces
			$black = explode(',', strtolower($this->settings['black_list']));
			foreach($black as &$b){
				$b = trim($b, " \t\n\r\n"); //intentionally duplicate \n!
			}
		}
		return in_array(trim(strtolower($email), " \t\n\r\n"), $black);
	}

	/**
	 * Write newsletter and mailing lists temp files
	 *
	 * @param Integer $nid
	 * @param Boolean $cachemails
	 * @return Array
	 */
	function cacheNewsletter($nid = 0, $cachemails = true){
		$GLOBALS['we']['WE_SendMail'] = true;
		$ret = [];
		if($nid){
			$this->Model = new we_newsletter_newsletter($nid);
		}

		if($cachemails){
			// BEGIN cache emails groups
			$emailcache = we_base_file::getUniqueId();
			$groupcount = count($this->Model->groups) + 1;

			$ret['emailcache'] = $emailcache;
			$buffer = [];

			for($groupid = 1; $groupid < $groupcount; $groupid++){
				$tmp = $this->getEmails($groupid);
				foreach($tmp as $curEntry){
					if(isset($curEntry[0]) && !empty($curEntry[7])){
						$index = strtolower($curEntry[0]);
						if(isset($buffer[$index])){
							if(!in_array($curEntry[6], explode(',', $buffer[$index][6]))){
								$buffer[$index][6] .= ',' . $curEntry[6];
							}
							$buffer[$index][7] = array_merge($buffer[$index][7], $curEntry[7]);
						} else {
							$buffer[$index] = $curEntry;
						}
					}
				}
			}

			$cc = 0;
			foreach($buffer as $k => $one){
				$buffer[$cc] = $one;
				unset($buffer[$k]);
				$cc++;
			}

			$ret['ecount'] = count($buffer);

			$groups = 0;
			$tmp = [];
			$go = true;
			$offset = 0;


			while($go){
				$tmp = array_slice($buffer, $offset, $this->settings['send_step']);
				if(!empty($tmp)){
					$offset += $this->settings['send_step'];
					$groups++;
					we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $groups, $tmp, we_base_util::SECONDS_DAY);
				} else {
					$go = false;
				}
			}

			$ret['gcount'] = $groups + 1;
		}

		// END cache emails groups
		// BEGIN cache newlsetter blocks
		$blockcache = we_base_file::getUniqueId();
		$blockcount = count($this->Model->blocks);

		$ret['blockcache'] = $blockcache;

		for($blockid = 0; $blockid < $blockcount; $blockid++){

			we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_p_' . $blockid, [
				'defaultC' => $this->getContent($blockid, 0, 0, '', '', '', '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'femaleC' => $this->getContent($blockid, 0, 0, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'maleC' => $this->getContent($blockid, 0, 0, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_firstname_lastnameC' => $this->getContent($blockid, 0, 0, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_lastnameC' => $this->getContent($blockid, 0, 0, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstname_lastnameC' => $this->getContent($blockid, 0, 0, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstnameC' => $this->getContent($blockid, 0, 0, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'lastnameC' => $this->getContent($blockid, 0, 0, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'default' => $this->getContent($blockid, 0, 0, '', '', '', '', ''),
				'female' => $this->getContent($blockid, 0, 0, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'male' => $this->getContent($blockid, 0, 0, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_firstname_lastname' => $this->getContent($blockid, 0, 0, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_lastname' => $this->getContent($blockid, 0, 0, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname_lastname' => $this->getContent($blockid, 0, 0, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname' => $this->getContent($blockid, 0, 0, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', ''),
				'lastname' => $this->getContent($blockid, 0, 0, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				], we_base_util::SECONDS_DAY);

			we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_h_' . $blockid, [
				'defaultC' => $this->getContent($blockid, 0, 1, '', '', '', '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'femaleC' => $this->getContent($blockid, 0, 1, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'maleC' => $this->getContent($blockid, 0, 1, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_firstname_lastnameC' => $this->getContent($blockid, 0, 1, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_lastnameC' => $this->getContent($blockid, 0, 1, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstname_lastnameC' => $this->getContent($blockid, 0, 1, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstnameC' => $this->getContent($blockid, 0, 1, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'lastnameC' => $this->getContent($blockid, 0, 1, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'default' => $this->getContent($blockid, 0, 1, '', '', '', '', ''),
				'female' => $this->getContent($blockid, 0, 1, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'male' => $this->getContent($blockid, 0, 1, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_firstname_lastname' => $this->getContent($blockid, 0, 1, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_lastname' => $this->getContent($blockid, 0, 1, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname_lastname' => $this->getContent($blockid, 0, 1, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname' => $this->getContent($blockid, 0, 1, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', ''),
				'lastname' => $this->getContent($blockid, 0, 1, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'inlines' => ($this->Model->blocks[$blockid]->Pack ? $this->cacheInlines($buffer, $blockcache) : []),
				], we_base_util::SECONDS_DAY);
		}
		// END cache newlsetter blocks

		return $ret;
	}

	//FIMXE: these files are never read again! maybe this is #0008648 ?
	function cacheInlines(&$buffer, $blockcache){
		$trenner = '\s*';
		$patterns = [
			'/<(img' . $trenner . '[^>]+src' . $trenner . "[\=\"|\=\'|\=\\\\|\=]*" . $trenner . ")([^\'\"> ?\\\]*)([^\"\' \\\\>]*)(" . $trenner . '[^>]*)>/si',
			'/<(body' . $trenner . '[^>]+background' . $trenner . "[\=\"|\=\'|\=\\\\|\=]*" . $trenner . ")([^\'\"> ?\\\]*)([^\"\' \\\\>]*)(" . $trenner . '[^>]*)>/si',
			'/<(table' . $trenner . '[^>]+background' . $trenner . "[\=\"|\=\'|\=\\\\|\=]*" . $trenner . ")([^\'\"> ?\\\]*)([^\"\' \\\\>]*)(" . $trenner . '[^>]*)>/si',
			'/<(td' . $trenner . '[^>]+background' . $trenner . "[\=\"|\=\'|\=\\\\|\=]*" . $trenner . ")([^\'\"> ?\\\]*)([^\"\' \\\\>]*)(" . $trenner . '[^>]*)>/si',
			'/background' . $trenner . ':' . $trenner . '([^url]*url' . $trenner . "\([\"|\'|\\\\])?(.[^\)|^\"|^\'|^\\\\]+)([\"|\'|\\\\])?/si",
			'/background-image' . $trenner . ':' . $trenner . '([^url]*url' . $trenner . "\([\"|\'|\\\\])?(.[^\)|^\"|^\'|^\\\\]+)([\"|\'|\\\\])?/si",
		];

		$match = $inlines = [];

		foreach($buffer as $v){
			foreach($patterns as $pattern){
				if(preg_match_all($pattern, $v, $match)){
					foreach($match[2] as $name){
						if(!isset($inlines[$name])){
							$newname = we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_i_' . we_base_file::getUniqueID();
							$inlines[$name] = $newname;
							we_base_file::insertIntoCleanUp($newname, we_base_util::SECONDS_DAY);
							//this will format the files for mailing, so the file is just to be included
							we_base_file::save($newname, chunk_split(base64_encode(we_base_file::load($name)), 76, "\n"));
						}
					}
				}
			}
		}
		return $inlines;
	}

	function getCleanMail($mail){
		$match = [];
		$pattern = '|[_\.0-9a-z-]+@([0-9a-z-]+\.)+[a-z]{2,6}|i';
		if(preg_match($pattern, $mail, $match)){
			return ($match[0]);
		}
		return '';
	}

	public function getShowImportBox(){
		return $this->show_import_box;
	}

	public function getShowExportBox(){
		return $this->show_export_box;
	}

	public function getHomeScreen(){
		$content = we_html_button::create_button('new_newsletter', "javascript:we_cmd('new_newsletter');", '', '', !we_base_permission::hasPerm(we_base_perm::NEWSLETTER_NEW)) .
			'<br/>' .
			we_html_button::create_button('new_newsletter_group', "javascript:we_cmd('new_newsletter_group');", '', '', !we_base_permission::hasPerm(we_base_perm::NEWSLETTER_NEW));

		return parent::getActualHomeScreen(we_modules_info::NEWSLETTER, $content, we_html_element::form(['name' => 'we_form'], $this->getHiddens(['ncmd' => 'home']) . we_html_element::hidden('home', 0)));
	}

	private static function we_getObjectFileByID($id, $includepath = ''){
		$mydoc = new we_objectFile();
		$mydoc->initByID($id, OBJECT_FILES_TABLE, we_contents_base::LOAD_MAIN_DB);
		return $mydoc->getDocumentData($includepath);
	}

	/*
	 * Grafische Aufbereitung des Versandlogs im Tab "Auswertung"
	 */

	function getHTMLReporting(){
		$this->db->query('SELECT Log,stamp,DATE_FORMAT(stamp,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS LogTime FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ' AND Log IN(\'' . we_newsletter_newsletter::LOG_START_SEND . '\', \'' . we_newsletter_newsletter::LOG_END_SEND . '\') ORDER BY stamp ASC');

		$newsletterMailOrders = [];
		$newsletterMailOrdersCnt = 0;

		while($this->db->next_record()){
			switch($this->db->f('Log')){
				case we_newsletter_newsletter::LOG_START_SEND:
					$newsletterMailOrders[++$newsletterMailOrdersCnt]['start_send'] = $this->db->f('stamp');
					$newsletterMailOrders[$newsletterMailOrdersCnt]['startTime'] = $this->db->f('LogTime');
					break;
				case we_newsletter_newsletter::LOG_END_SEND:
					$newsletterMailOrders[$newsletterMailOrdersCnt]['end_send'] = $this->db->f('stamp');
			}
		}

		$parts = [];

		foreach($newsletterMailOrders as $key => $newsletterMailOrder){
			$table = new we_html_table(['class' => 'defaultfont', 'style' => 'width: 588px'], 1, 5);
			$results = $this->db->getAllFirstq('SELECT Log,COUNT(1) FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ' AND Log NOT IN (\'' . we_newsletter_newsletter::LOG_START_SEND . '\',\'' . we_newsletter_newsletter::LOG_END_SEND . '\') AND stamp BETWEEN "' . $newsletterMailOrder['start_send'] . '" AND "' . (isset($newsletterMailOrder['end_send']) ? $newsletterMailOrder['end_send'] : 'NOW()') . '" GROUP BY Log', false);

			$allRecipients = array_sum($results);

			/* process bar blocked by blacklist */
			$allBlockedByBlacklist = (array_key_exists(we_newsletter_newsletter::LOG_MAIL_BLACK, $results) ? $results[we_newsletter_newsletter::LOG_MAIL_BLACK] : 0);
			$percentBlockedByBlacklist = we_base_util::getPercent($allRecipients, $allBlockedByBlacklist, 2);

			$pbByB = new we_gui_progressBar($percentBlockedByBlacklist, 150, 'blacklist' . $key);

			$table->addRow();
			$table->setColContent(1, 0, we_html_element::span(['id' => 'blacklist_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_are_black]')));
			$table->setColContent(1, 1, $pbByB->getHTML());
			$table->setCol(1, 2, ['style' => "padding: 0 5px;"], we_html_element::span(['id' => 'blacklist_total', 'style' => 'color:' . (($allBlockedByBlacklist > 0) ? 'red' : 'green') . ';'], $allBlockedByBlacklist));
			$table->setCol(1, 3, ['style' => "padding: 0 5px;"], '<i class="fa fa-lg ' . ($allBlockedByBlacklist == 0 ? "fa-check fa-ok" : "fa-close fa-cancel") . '"></i>');
			//todo: statt show black list, sollte show_log begrenzt auf Log=email_is_black + $start_send + start_end
			$table->setCol(1, 4, ['style' => 'width: 35px'], (($allBlockedByBlacklist == 0) ? '' : we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:top.opener.top.we_cmd('black_list');"))));

			/* process bar blocked by domain check */
			$allBlockedByDomainCheck = (array_key_exists(we_newsletter_newsletter::LOG_DOMAIN_NOK, $results) ? $results[we_newsletter_newsletter::LOG_DOMAIN_NOK] : 0);
			$percentBlockedByDomain = we_base_util::getPercent($allRecipients, $allBlockedByDomainCheck, 2);

			$pbBbD = new we_gui_progressBar($percentBlockedByDomain, 150, 'domain' . $key);

			$table->addRow();
			$table->setColContent(2, 0, we_html_element::span(['id' => 'domain_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_nok]')));
			$table->setColContent(2, 1, $pbBbD->getHTML());
			$table->setCol(2, 2, ['style' => "padding: 0 5px;"], we_html_element::span(['id' => 'domain_total', 'style' => 'color:' . (($allBlockedByDomainCheck > 0) ? 'red' : 'green') . ';'], $allBlockedByDomainCheck));
			$table->setCol(2, 3, ['style' => "padding: 0 5px;"], '<i class="fa fa-lg ' . ($allBlockedByDomainCheck == 0 ? "fa-check fa-ok" : "fa-close fa-cancel") . '"></i>');
			//todo: statt domain, sollte show_log begrenzt auf Log=domain_nok + $start_send + start_end
			$table->setCol(2, 4, ['style' => 'width: 35px'], (($allBlockedByDomainCheck == 0) ? '' : we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:top.opener.top.we_cmd('domain_check');"))));

			/* process bar all clear recipients */
			$allClearRecipients = (array_key_exists(we_newsletter_newsletter::LOG_MAIL_SENT, $results) ? $results[we_newsletter_newsletter::LOG_MAIL_SENT] : 0);
			$percentClearRecipients = we_base_util::getPercent($allRecipients, $allClearRecipients, 2);

			$pbCR = new we_gui_progressBar($percentClearRecipients, 150, 'recipients' . $key);

			$table->addRow();
			$table->setColContent(3, 0, we_html_element::span(['id' => 'recipients_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_success]')));
			$table->setColContent(3, 1, $pbCR->getHTML());
			$table->setCol(3, 2, ['style' => "padding: 0 5px;"], we_html_element::span(['id' => 'recipients_total', 'style' => 'color:' . (($allClearRecipients <= 0) ? 'red' : 'green') . ';'], $allClearRecipients));
			$table->setCol(3, 3, ['style' => "padding: 0 5px;"], '<i class="fa fa-lg ' . ($allClearRecipients == $allRecipients ? "fa-check fa-ok" : "fa-exclamation-triangle fa-cancel") . '" title="' . ($allClearRecipients < $allRecipients ? g_l('modules_newsletter', '[reporting][mailing_advice_not_success]') : '') . '"></i>');
			//todo: statt show_log, sollte show_log begrenzt auf Log=email_sent + $start_send + start_end
			$table->setCol(3, 4, ['style' => 'width: 35px'], we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:top.opener.top.we_cmd('show_log')")));

			/* total recipients */
			$table->addRow();
			$table->setColContent(4, 0, we_html_element::b(g_l('modules_newsletter', '[reporting][mailing_all_emails]')));
			$table->setCol(4, 2, ['colspan' => 2, 'style' => 'padding: 0 5px;'], we_html_element::b($allRecipients));

			$parts[] = [
				'headline' => g_l('modules_newsletter', '[reporting][mailing_send_at]') . '&nbsp;' . $newsletterMailOrder['startTime'],
				'html' => $table->getHTML() . we_html_element::br()
			];
		}

		return $parts ?: [[
			'headline' => g_l('modules_newsletter', '[reporting][mailing_not_done]'),
		]];
	}

	public function getHTMLLog($status, $start, $end){
		$this->db->query('SELECT Log,Param,DATE_FORMAT(stamp,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS LogTime FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ($status !== false ? ' AND Log=' . $status : '') . ($start && $end ? ' AND stamp BETWEEN FROM_UNIXTIME(' . $start . ') AND FROM_UNIXTIME(' . $end . ')' : '') . ' ORDER BY stamp DESC');

		$content = '';
		while($this->db->next_record()){
			$log = g_l('modules_newsletter', '[' . $this->db->f('Log') . ']');
			$param = $this->db->f('Param');
			$content .= we_html_element::div(['class' => 'defaultfont'], $this->db->f('LogTime') . '&nbsp;' . ($param ? sprintf($log, $param) : $log));
		}
		return $content;
	}

}
