<?php

/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_object_search extends we_search_base{
	var $height;
	var $searchname = [];
	var $searchlocation = [];
	var $searchfield = [];
	var $show;
	private static $intFields = [];
	private static $realFields = [];

	function __construct(){
		parent::__construct();
		if(isset($sessDat) && is_array($sessDat)){
			foreach($sessDat as $cur){
				$this->{$cur} = $cur;
			}
		}
	}

	function init(array $sessDat = []){
		foreach($sessDat as $v){
			$this->{$v} = $v;
		}
	}

	function getFields($name, $size, $select, $Path, $multi = '', $fields = true, $properties = true){
		$objID = f('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE Path="' . $GLOBALS['DB_WE']->escape($Path) . '"');
		if(!$objID){
			return '';
		}
		$opts = '';
		$all = [];
		$tableInfoFields = $GLOBALS['DB_WE']->metadata(OBJECT_X_TABLE . $objID, we_database_base::META_NAME);
		$tableInfoProperties = $GLOBALS['DB_WE']->metadata(OBJECT_FILES_TABLE, we_database_base::META_NAME);

		if($properties){
			foreach($tableInfoProperties as $cur){
				switch($cur){
					case 'Text':
						$opts .= '<option value="' . $cur . '" ' . (($select == $cur) ? "selected" : "") . '>' . g_l('modules_object', '[objectname]') . '</option>';
						$all[] = $cur;
						break;
					case 'Path':
						$opts .= '<option value="' . $cur . '" ' . (($select == $cur) ? "selected" : "") . '>' . g_l('modules_object', '[objectpath]') . '</option>';
						$all[] = $cur;
						break;
					case 'ID':
						$opts .= '<option value="' . $cur . '" ' . (($select == $cur) ? "selected" : "") . '>' . g_l('modules_object', '[objectid]') . '</option>';
						$all[] = $cur;
						break;
					case 'Url':
						$opts .= '<option value="' . $cur . '" ' . (($select == $cur) ? "selected" : "") . '>' . g_l('modules_object', '[objecturl]') . '</option>';
						$all[] = $cur;
						break;
				}
			}
		}

		if($fields){
			foreach($tableInfoFields as $cur){
				if($cur != 'ID' && substr($cur, 0, 3) != 'OF_' && stripos($cur, we_contents_objectBase::TYPE_MULTIOBJECT) !== 0 && stripos($cur, "object") !== 0){
					$regs = explode('_', $cur, 2);
					if(count($regs) == 2){
						$opts .= '<option value="' . $cur . '" ' . (($select == $cur) ? "selected" : "") . '>' . $regs[1] . '</option>';
					}
					$all[] = $cur;
				}
			}
		}

		if(count($all)){
			$opts = '<option value="' . implode(',', $all) . '">' . g_l('modules_object', '[allFields]') . '</option>' . $opts;
		}
		$onchange = (substr($select, 0, 4) != "meta" && substr($select, 0, 4) != "date" && substr($select, 0, 8) != "checkbox" ? 'onchange="changeit(this.value);"' : 'onchange="changeitanyway(this.value);"');
		return '<select name="' . $name . '" class="weSelect" size="' . $size . '" ' . $multi . ' ' . $onchange . '>' . $opts . '</select>';
	}

	function getJSinWEsearchobj($name){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'object/object_search.js', '', ['id' => 'loadVarObject_search', 'data-searchObj' => setDynamicVar([
					'anzahl' => $this->anzahl,
					'name' => $name,
				])]
		);
	}

	function greenOnly($GreenOnly, $pid, $cid){
		if($GreenOnly){
			return we_objectFile::makePIDTail($pid, $cid) . ' AND `of`.Published>0';
		}
		return '';
	}

	function getWorkspaces(array $foo, $len){
		if(!$foo){
			return '-';
		}
		$paths = id_to_path($foo, FILE_TABLE, null, true);
		$out = [];
		foreach($foo as $id){
			if(empty($id)){
				continue;
			}
			$p = $paths[$id];
			$out[] = '
<a href="javascript:setWs(\'' . $p . '\',\'' . $id . '\')" class="middlefont" title="' . $p . '">' . we_base_util::shortenPath($p, $len) . '</a>';
		}

		return implode('<br/>', $out);
	}

	function searchfor(array $searchnames, array $searchfield, array $searchlocation, $tablename, $alias = '', $rows = -1, $start = 0, $order = '', $desc = 0){
		$realFields = $this->getRealFields($tablename);
		$intFields = $this->getIntFields($tablename);
		$allFields = $this->db->metadata($tablename, we_database_base::META_NAME);
		foreach($searchnames as $i => $searchname){
			$filteredFields = [];
			$fieldsToFilterOut = [];

			$type = is_array($searchname) ? 0 : (!preg_match('/^[-+]?\d*\.?\d+$/', $searchname) ? 1 : (!preg_match('/^[-+]?\d+$/', $searchname) ? 2 : 0));
			switch($type){
				case 1:
					$fieldsToFilterOut = array_merge($fieldsToFilterOut, $realFields);
				//no break!
				case 2:
					$fieldsToFilterOut = array_merge($fieldsToFilterOut, $intFields);
			}

			$arrSearchfield = explode(',', trim($searchfield[$i], ','));
			foreach($arrSearchfield as $f){
				if(!in_array($f, $fieldsToFilterOut)){
					$filteredFields[] = (in_array($f, $allFields) ? $alias . '.' : '') . $f;
				}
			}
			$searchfield[$i] = implode(',', $filteredFields);
		}

		return parent::searchfor($searchnames, $searchfield, $searchlocation, '', $alias, $rows, $start, $order, $desc);
	}

	private function getIntFields($tablename){
		if(self::$intFields || !$tablename){
			return self::$intFields;
		}

		foreach($this->db->metadata($tablename) as $f){
			if(in_array($f['type'], [we_database_base::TYPE_INT, we_database_base::TYPE_TINYINT, we_database_base::TYPE_SMALLINT, we_database_base::TYPE_MEDIUMINT, we_database_base::TYPE_BIGINT])){
				self::$intFields[] = $f['name'];
			}
		}
		return self::$intFields;
	}

	private function getRealFields($tablename){
		if(self::$realFields || !$tablename){
			return self::$realFields;
		}

		foreach($this->db->metadata($tablename) as $f){
			if(in_array($f['type'], [we_database_base::TYPE_DOUBLE, we_database_base::TYPE_FLOAT, we_database_base::TYPE_DECIMAL])){
				self::$realFields[] = $f['name'];
			}
		}
		return self::$realFields;
	}

	function removeFilter($position){
		if($this->height == 0){
			$this->objsearch = '';
			$this->objsearchField = '';
			$this->objlocation = '';
			return;
		}

		foreach($this->objsearch as $idx => $value){
			if($idx == $position){
				unset($this->objsearch[$position]);
			} elseif($idx >= $position){
				if(!isset($this->objsearch[($idx - 1)])){
					$this->objsearch[($idx - 1)] = '';
				}
				$this->objsearch[($idx - 1)] = $this->objsearch[$idx];
				unset($this->objsearch[$idx]);
			}
		}

		foreach($this->objsearchField as $idx => $value){
			if($idx == $position){
				unset($this->objsearchField[$position]);
			} elseif($idx >= $position){
				if(!isset($this->objsearchField[($idx - 1)])){
					$this->objsearchField[($idx - 1)] = '';
				}
				$this->objsearchField[($idx - 1)] = $this->objsearchField[$idx];
				unset($this->objsearchField[$idx]);
			}
		}

		foreach($this->objlocation as $idx => $value){
			if($idx == $position){
				unset($this->objlocation[$position]);
			} elseif($idx >= $position){
				if(!isset($this->objlocation[($idx - 1)])){
					$this->objlocation[($idx - 1)] = '';
				}
				$this->objlocation[($idx - 1)] = $this->objlocation[$idx];
				unset($this->objlocation[$idx]);
			}
		}

		$this->height--;
	}

	public function searchFields($TableID, $WorkspaceID, $GreenOnly, $Path){
		$where = (!empty($this->searchname) ?
			$this->searchfor($this->searchname, $this->searchfield, $this->searchlocation, OBJECT_X_TABLE . $TableID, 'obx', -1, 0, '', 0) . $this->greenOnly($GreenOnly, $WorkspaceID, $TableID) :
			$this->greenOnly($GreenOnly, $WorkspaceID, $TableID));
		$whereRestrictOwners = ' AND (`of`.RestrictOwners=0 OR `of`.CreatorID=' . intval($_SESSION['user']['ID']) . ' OR FIND_IN_SET(' . intval($_SESSION['user']['ID']) . ',`of`.Owners)) ';

		$this->settable(OBJECT_X_TABLE . $TableID . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON obx.OF_ID=`of`.ID');
		$this->setwhere(($where ? $where . ' AND ' : '') . '`of`.Path LIKE "' . $Path . '/%" AND `of`.ID!=0 AND `of`.IsFolder=0 ' . $whereRestrictOwners);
		$this->searchquery('', 'obx.*,`of`.ID,`of`.Text,`of`.Path,`of`.ParentID,`of`.Workspaces,`of`.Published,`of`.IsSearchable,`of`.Charset,`of`.Language,`of`.Url,`of`.TriggerID,`of`.ModDate,`of`.WebUserID');
	}

}
