<?php

/**
 * webEdition CMS
 *
 * $Rev: 32dc09780205 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-01-08 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_sidebar_frames{

	public static function getHTML($what){
		switch($what){
			case 'content':
				echo self::getHTMLContent();
				break;
			default:
				echo self::getHTMLFrameset();
				break;
		}
	}

	private static function getHTMLFrameset(){
		return '
<div id="weSidebarBody">
	<div id="weSidebarHeader">
		<div id="Headline">' . g_l('sidebar', '[headline]') . '</div>
		<div id="CloseButton">
			<span class="close" onclick="WE().layout.sidebar.close();">
				<i class="fa fa-close fa-lg "></i>
			</span>

		</div>
	</div>
	<div id="weSidebarContentDiv">
		<iframe id="weSidebarContent" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadSidebarDocument" name="weSidebarContent"></iframe>
	</div>
</div>';
	}

	private static function showSidebarText(array $textArray){
		$ret = '';
		foreach($textArray as $text){
			$link = "%s";
			if(!empty($text['link'])){

				if(stripos($text['link'], 'javascript:') === 0){
					$text['link'] = str_replace("\"", "'", $text['link']); #6625
					$text['link'] = str_replace("`", "'", $text['link']); #6625
					$link = '<a href="' . $text['link'] . '">%s</a>';
				} else {
					$link = '<a href="' . $text['link'] . '" target="_blank">%s</a>';
				}
			}

			$headline = (empty($text['headline']) ? '' : sprintf($link, $text['headline']));
			$ret .= '
<tr>
	<td style="vertical-align:top;padding-top:5px;" colspan="2"><strong>' . $headline . '</strong><br/><br/>' . $text['text'] . '</td>
</tr>';
		}
		return $ret;
	}

	private static function getDefaultContent(){
		return we_html_tools::getHtmlTop('sideBar', '', '', '', we_html_element::body(['class' => "weSidebarBody"], '<table class=" defaultfont">' .
					self::showSidebarText(g_l('sidebar', '[default]')) .
					(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ?
					self::showSidebarText(g_l('sidebar', '[admin]')) :
					'') .
					'</table>'
		));
	}

	public static function getHTMLContent(){
		$id = we_base_request::_(we_base_request::INT, 'we_cmd', '', 1);
		$file = we_base_request::_(we_base_request::URL, 'we_cmd', '', 2);
		$params = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 3);
		define('WE_SIDEBAR', true);

		if($id){
			$file = id_to_path($id, FILE_TABLE, $GLOBALS['DB_WE'], false, false, true);
		} else {
			if(stripos($file, "http://") === 0 || stripos($file, "https://") === 0){
				//not implemented
				//header("Location: " . $file);
				exit();
			}
		}
		if(!file_exists($_SERVER['DOCUMENT_ROOT'] . $file) || !is_file($_SERVER['DOCUMENT_ROOT'] . $file)){
			$file = id_to_path(intval(SIDEBAR_DEFAULT_DOCUMENT), FILE_TABLE, $GLOBALS['DB_WE'], false, false, true);
			if(!$file || substr($file, -1) === '/' || $file === 'default'){
				$file = '';
			}
		}

		if($file){
			//manipulate GET/REQUEST for document
			$_GET = [];
			parse_str($params, $_GET);
			$_REQUEST = $_GET;
			ob_start();
			include($_SERVER['DOCUMENT_ROOT'] . $file);
			$cnt = 0;
			$SrcCode = str_replace('<head>', '<head>' . we_html_element::jsScript('/webEdition/js/global.js'), ob_get_clean(), $cnt);
		} else {
			$cnt = 0;
			$SrcCode = self::getDefaultContent();
		}

		return we_gui_SEEM::parseDocument(($cnt ? $SrcCode : we_html_element::jsScript('/webEdition/js/global.js') . $SrcCode));
	}

}
