<?php

/**
 * webEdition CMS
 *
 * $Rev: e670e6ad6538 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-04 18:37 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_weTag_tag{
	/* temporary disabled for php 5.x */

	//abstract public static function tag(array $attribs = [], $content = '', $internal = false);

	public static function parse(array $attribs, $content){
		$tagname = strtr(get_called_class(), ['we_tag_' => '']);
		return '<?= ' .
			($content ?
			// Tag besitzt Endtag
			we_weTag_parser::printTag($tagname, $attribs, $content, true) . '; ?>' :
			// Tag ohne Endtag
			we_weTag_parser::printTag($tagname, $attribs) . '; ?>'
			);
	}

//tag utility functions

	/**
	 * get an attribute from $attribs, and return its value according to default
	 * @param string $name attributes name
	 * @param array $attribs array containg the attributes
	 * @param mixed $default default value
	 * @return mixed returns the attributes value or default if not set
	 */
	public static function getParserAttribute($name, array $attribs, $default = '', $type = we_base_request::RAW, $isDeprecated = false){
		return self::getAttribute($name, $attribs, $default, $type, $isDeprecated, false);
	}

	/**
	 * get an attribute from $attribs, and return its value according to default
	 * @param string $name attributes name
	 * @param array $attribs array containg the attributes
	 * @param mixed $default default value
	 * @param bool $useGlobal check if attribute value is a php-variable and is found in $GLOBALS
	 * @return mixed returns the attributes value or default if not set
	 */
	public static function getAttribute($name, array $attribs, $default = '', $type = we_base_request::RAW, $isDeprecated = false, $useGlobal = true){
		if(!isset($attribs[$name])){
			return is_string($default) ? htmlspecialchars_decode($default) : $default;
		}
		$regs = [];
		if($isDeprecated){
			t_e('deprecated', 'Attribute ' . $name . ' is deprecated and will be removed. Please update your code!');
		}
		if($useGlobal && !is_array($attribs[$name]) && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $attribs[$name], $regs)){
			$value = (isset($regs[2]) ?
				self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
				(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
		} else {
			$value = $attribs[$name];
		}

		$value = we_base_request::filterVar($value, (is_bool($type) ? we_base_request::BOOL : $type), $default);

		return is_string($value) ? htmlspecialchars_decode($value) : $value;
	}

	/**
	 * get an attribute from $attribs, and return its value according to default; unset attribs after access
	 * @param string $name attributes name
	 * @param array $attribs array containg the attributes
	 * @param mixed $default default value
	 * @param bool $useGlobal check if attribute value is a php-variable and is found in $GLOBALS
	 * @return mixed returns the attributes value or default if not set
	 */
	protected static function getAttributeR($name, array &$attribs, $default = '', $type = we_base_request::RAW, $isDeprecated = false, $useGlobal = true){
		if(!isset($attribs[$name])){
			return $default;
		}
		$ret = self::getAttribute($name, $attribs, $default, $type, $isDeprecated, $useGlobal);
		unset($attribs[$name]);
		return $ret;
	}

	public static function parseError($text, $extra = ''){
		if(defined('IGNORE_PARSE_ERROR')){
			return '';
		}
		t_e('warning', html_entity_decode($text, ENT_QUOTES, $GLOBALS['we']['Charset']['Backend']), g_l('weClass', '[template]') . ': ' . we_weTag_parser::$curFile, $extra);
		return '<b>' . g_l('parser', '[error_in_template]') . ':</b>' . $text . "<br/>\n" . g_l('weClass', '[template]') . ': ' . we_weTag_parser::$curFile;
	}

	public static function attributeMissing(array $attribs, $attrs, $tag, $canBeEmpty = false){
		$tag = str_replace(['we_tag_', 'we_parse_tag_'], '', $tag);
		if(!is_array($attrs)){
			$attrs = [$attrs => $canBeEmpty];
		}
		foreach($attrs as $attr => $canBeEmpty){
			if($canBeEmpty){
				if(!isset($attribs[$attr])){
					return self::parseError(sprintf(g_l('parser', '[attrib_missing2]'), $attr, $tag));
				}
			} elseif(!isset($attribs[$attr]) || $attribs[$attr] === ''){
				return self::parseError(sprintf(g_l('parser', '[attrib_missing]'), $attr, $tag));
			}
		}
		return '';
	}

	public static function redirectTagOutput($returnvalue, $nameTo, $to = 'screen'){
		switch(isset($GLOBALS['we']['calculate']) ? 'calculate' : $to){
			case 'request':
				self::setVarArray($_REQUEST, $nameTo, $returnvalue);
				return null;
			case 'post':
				self::setVarArray($_POST, $nameTo, $returnvalue);
				return null;
			case 'get':
				self::setVarArray($_GET, $nameTo, $returnvalue);
				return null;
			case 'global':
				self::setVarArray($GLOBALS, $nameTo, $returnvalue);
				return null;
			case 'session':
				self::setVarArray($_SESSION, $nameTo, $returnvalue);
				return null;
			case 'top':
				$GLOBALS['we']['main']['doc']->setElement($nameTo, $returnvalue);
				return null;
			case 'block' :
				$nameTo = self::getPostName($nameTo);
			case 'self' :
				$GLOBALS['we_doc']->setElement($nameTo, $returnvalue);
				return null;
			case 'sessionfield' :
				if(isset($_SESSION['webuser'][$nameTo])){
					$_SESSION['webuser'][$nameTo] = $returnvalue;
				}
				return null;
			case 'calculate':
				return we_base_util::std_numberformat($returnvalue);
			case 'screen':
			default:
				return $returnvalue;
		}
	}

	/**
	 * get the full name of an Attribute with applied postTagName if set
	 * @param string $var
	 * @return string
	 */
	public static function getPostName($var){
		if($var && isset($GLOBALS['we']['postTagName'])){
			return $var . $GLOBALS['we']['postTagName'];
		}
		return $var;
	}

	public static function getDocForTag($docAttr, $maindefault = false){
		switch($docAttr){
			case 'self':
				return $GLOBALS['we_doc'];
			case 'top':
				return $GLOBALS['we']['main']['doc'];
			default:
				return ($maindefault ?
					$GLOBALS['we']['main']['doc'] :
					$GLOBALS['we_doc']);
		}
	}

	private static function setVarArray(&$arr, $string, $value){
		if(strpos($string, '[') === false){
			$arr[$string] = $value;
			return;
		}
		$current = &$arr;

		/* 	$arr_matches = [];
		  preg_match('/[^\[\]]+/', $string, $arr_matches);
		  $first = $arr_matches[0];
		  preg_match_all('/\[([^\]]*)\]/', $string, $arr_matches, PREG_PATTERN_ORDER);
		  $arr_matches = $arr_matches[1];
		  array_unshift($arr_matches, $first); */
		$arr_matches = preg_split('/\]\[|\[|\]/', $string);
		$last = count($arr_matches) - 1;
		unset($arr_matches[$last--]); //preg split has an empty element at the end
		foreach($arr_matches as $pos => $dimension){
			if(empty($dimension)){
				$dimension = count($current);
			}
			if($pos == $last){
				$current[$dimension] = $value;
				return;
			}
			if(!isset($current[$dimension])){
				$current[$dimension] = [];
			}
			$current = &$current[$dimension];
		}
	}

	public static function pre_listview($newLV){
		//add current lv to the stack
		if(we_tag_listview::$current){
			we_tag_listview::$lvArray[] = we_tag_listview::$current;
		}
		//set new lv
		$GLOBALS['lv'] = we_tag_listview::$current = $newLV;
	}

//this function is used by all tags adding elements to lvArray
	public static function post_listview(){
		if(empty(we_tag_listview::$lvArray)){
			we_tag_listview::$current = null;
			unset($GLOBALS['lv']);
			return;
		}
		//get the last lv
		$GLOBALS['lv'] = we_tag_listview::$current = array_pop(we_tag_listview::$lvArray);
	}

	public static function getOutLang(array $attribs, $fallBackBackend = false){
		$lang = self::getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
		if(!$lang){
			$doc = self::getDocForTag(self::getAttribute('doc', $attribs, 'self', we_base_request::STRING));
			$lang = $doc->Language;
		}
		if(!$lang){
			$lang = ($fallBackBackend ? array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) : $GLOBALS['we']['lang']['DefaultFrontendLanguage']);
		}
		$langcode = substr($lang, 0, 2);
		return [$lang, $langcode];
	}

	public static function modulMissingError($modul, $tag){
		return self::parseError(sprintf(g_l('parser', '[module_missing]'), $modul, str_replace(['we_tag_', 'we_parse_tag_'], '', $tag)));
	}

	protected static function deprecated($since){
		$tagname = strtr(get_called_class(), ['we_tag_' => '']);
		t_e('call of deprecated tag we:"' . $tagname . '" which is deprecated since ' . $since . ' will be removed soon.');
	}

	public static function getArrayValue($var, $name, $arrayIndex, $isset = false){
		$arr_matches = preg_split('/\]\[|\[|\]/', $arrayIndex);
		if(count($arr_matches) > 1){
			unset($arr_matches[count($arr_matches) - 1]);
		}
		if($name !== null){
			$arr_matches[0] = $name;
		}
		foreach($arr_matches as $cur){
			if(!isset($var[$cur])){
				return ($isset ? false : '');
			}
			$var = $var[$cur];
		}
		return ($isset ? true : $var);
	}

	private static function cutSimpleText($text, $len){
		if($len >= strlen($text)){
			return $text;
		}
		$text = substr($text, 0, $len);
		//cut to last whitespace, if any.
		return substr($text, 0, max([
				strrpos($text, ' '),
				strrpos($text, '.'),
				strrpos($text, ','),
				strrpos($text, '/'),
				strrpos($text, "\n"),
				strrpos($text, "\t"),
			]) ?: $len
		);
	}

	public static function cutText($text, $max = 0, $striphtml = false){
		$text = $striphtml ? strip_tags($text) : $text;
		if((!$max) || (strlen($text) <= $max)){
			return $text;
		}
		//no tags, simple cut off
		if($striphtml || strpos($text, '<') === false){
			return self::cutSimpleText($text, $max) . ($striphtml ? ' ...' : ' &hellip;');
		}

		$ret = '';
		$tags = $foo = [];
		//split text on tags, entities and "rest"
		preg_match_all('%(&#?[[:alnum:]]+;)|([^<&]*)|<(/?)([[:alnum:]]+)([ \t\r\n]+[[:alnum:]]+[ \t\r\n]*=[ \t\r\n]*"[^"]*")*[ \t\r\n]*(/?)>%sm', $text, $foo, PREG_SET_ORDER);

		foreach($foo as $cur){
			switch(count($cur)){
				case 2://entity
					if($max > 0){
						$ret .= $cur[0];
						$max -= 1;
					}
					break;
				case 3://text
					if($max > 0){
						$len = strlen($cur[0]);
						$ret .= ($len > $max ? self::cutSimpleText($cur[0], $max) : $cur[0]);
						$max -= $len;
						if($max <= 0){
							$ret .= ($striphtml ? ' ...' : ' &hellip;');
						}
					}
					break;
				case 7://tags
					if($max > 0){
						$ret .= $cur[0];
						if(!$cur[6]){//!selfclosing
							if($cur[3]){//close
								array_pop($tags);
							} else {
								$tags[] = $cur[4];
							}
						}
					}
					break;
			}
		}

//close open tags
		while($tags){
			$ret .= '</' . array_pop($tags) . '>';
		}

		return $ret;
	}

	public static function inputRadioField($name, $value, $itsValue, $atts){
		//  This function replaced fnc: we_getRadioField
		$atts['type'] = 'radio';
		$atts['name'] = $name;
		$atts['value'] = oldHtmlspecialchars($itsValue, -1, 'ISO-8859-1', false);
		if($value == $itsValue){
			$atts['checked'] = 'checked';
		}
		return getHtmlTag('input', $atts);
	}

	public static function textareaField($name, $value, array $atts){
		$atts['name'] = $name;
		return getHtmlTag('textarea', $atts, oldHtmlspecialchars($value), true);
	}

	public static function inputTextInputField($name, $value, array $atts){
		$atts['type'] = 'text';
		$atts['name'] = $name;
		$atts['value'] = oldHtmlspecialchars($value);
		return getHtmlTag('input', $atts);
	}

	public static function inputCheckboxField($name, $value, array $attr){
		//  returns a checkbox with associated hidden-field
		$tmpname = md5(uniqid(__FUNCTION__, true)); // #6590, changed from: uniqid(time())
		if($value){
			$attr['checked'] = 'checked';
		}
		$attr['type'] = 'checkbox';
		$attr['value'] = 1;
		$attr['name'] = $tmpname;
		$attr['onclick'] = 'this.form.elements[\'' . $name . '\'].value=(this.checked) ? 1 : 0';

		return getHtmlTag('input', $attr) . getHtmlTag('input', ['type' => 'hidden', 'name' => $name, 'value' => oldHtmlspecialchars($value)]);
	}

	public static function selectField($name, $value, array $values, array $attribs = [], $addMissing = true){
		$attribs['name'] = $name;
		$content = '';
		$isin = 0;
		foreach($values as $option){
			$opt = oldHtmlspecialchars($option, -1, 'ISO-8859-1', false);
			if($option == $value){
				$content .= getHtmlTag('option', ['value' => $opt, 'selected' => 'selected'], $opt, true);
				$isin = 1;
			} else {
				$content .= getHtmlTag('option', ['value' => $opt], $opt, true);
			}
		}
		if((!$isin) && $addMissing && $value != ''){
			$content .= getHtmlTag('option', ['value' => oldHtmlspecialchars($value), 'selected' => 'selected'
				], oldHtmlspecialchars($value), true);
		}
		return getHtmlTag('select', $attribs, $content, true);
	}

}
