<?php

/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 * @param		$attribs
 * @param		$content
 * @return		string
 */
abstract class we_tag_a extends we_weTag_tag{
	private const removeAttribs = [
		'id',
		'shop',
		'amount',
		'delshop',
		'delarticle',
		'hidedirindex',
		'shopname',
		'return',
		'edit',
		'type',
		'button',
		'hrefonly',
		'confirm',
		'editself',
		'delete',
		'params'
	];

	public static function parse(array $attribs, $content){
		return '<?php ob_start();?>' . $content . '<?= ' . we_weTag_parser::printTag('a', $attribs, 'ob_get_clean()', false, true) . '; ?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(we_tag_listview::$current && we_tag_listview::$current instanceof stdClass){
			$id = we_tag_listview::$current->ID;
		} else {
			// check for id attribute
			if(($foo = self::attributeMissing($attribs, 'id', __CLASS__))){
				return $foo;
			}

			// get attributes
			$id = self::getAttribute('id', $attribs, 0, we_base_request::STRING);
			if($id === 'self' && !defined('WE_REDIRECTED_SEO')){
				$id = $GLOBALS['we']['main']['id'];
			}
		}
		$confirm = self::getAttribute('confirm', $attribs, '', we_base_request::STRING);
		$button = self::getAttribute('button', $attribs, false, we_base_request::BOOL);
		$hrefonly = self::getAttribute('hrefonly', $attribs, false, we_base_request::BOOL);
		$return = self::getAttribute('return', $attribs, false, we_base_request::BOOL);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$shop = self::getAttribute('shop', $attribs, false, we_base_request::BOOL);
		$delarticle = self::getAttribute('delarticle', $attribs, false, we_base_request::BOOL);
		$delshop = self::getAttribute('delshop', $attribs, false, we_base_request::BOOL);
		list($anchor, $param) = we_base_util::getParamStr(self::getAttribute('params', $attribs, '', we_base_request::STRING));

		$edit = self::getAttribute('edit', $attribs) ?: ($shop || $delarticle || $delshop ? 'shop' : '');
		$url = self::getUrl($id, $attribs);

		if(!$url && !($GLOBALS['we']['main']['doc'] instanceof we_template)){
			return ($GLOBALS['we']['editmode'] ? self::parseError('in we:a attribute id not exists!') : '');
		}

		if($edit){
			$param = array_merge($param, self::getEditParams($edit, $attribs, $delshop, $delarticle));
		}

		if($return){
			$param['we_returnpage'] = getScriptName() . '?' . $_SERVER['QUERY_STRING'];
		}

		if($hrefonly){
			return $url . $anchor . ($param ? '?' . http_build_query($param) : '');
		}

		//	remove unneeded attributes from array
		$attribs = removeAttribs($attribs, self::removeAttribs);

		if($button){ //	show button
			$attribs['type'] = 'button';
			$attribs['onclick'] = ($target ? "var we_win=window.open('','" . $target . "');we_win" : 'self') .
				".document.location='" . $url . $anchor . ($param ? '?' . http_build_query($param) : '') . "';";

			$attribs = removeAttribs($attribs, ['target']); //	not html - valid

			if($confirm){
				$confirm = str_replace("'", "\\'", $confirm);
				$attribs['onclick'] = 'if(window.confirm(\'' . $confirm . '\')){' . $attribs['onclick'] . '}';
			}
			return getHtmlTag('button', $attribs, $content, true);
		}
//	show normal link
		$attribs['href'] = $url . $anchor . ($param ? '?' . http_build_query($param) : '');

		if($confirm){
			$attribs['onclick'] = 'return window.confirm(\'' . $confirm . '\'));';
		}

		return getHtmlTag('a', $attribs, $content, true);
	}

	private static function getUrl($id, array $attribs){
		if($id === 'self' && defined('WE_REDIRECTED_SEO')){
			return '/' . WE_REDIRECTED_SEO;
		}
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		// init variables
		$row = getHash('SELECT Path,IsFolder,IsDynamic FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id));
		$url = (!$row ? '' : $row['Path'] . ($row['IsFolder'] ? '/' : ''));
		$path_parts = pathinfo($url);
		if($hidedirindex && TAGLINKS_DIRECTORYINDEX_HIDE && seoIndexHide($path_parts['basename'])){
			return ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
		}
		return $url;
	}

	private static function getEditParams($edit, array $attribs, $delshop, $delarticle): array{
		$delete = self::getAttribute('delete', $attribs, false, true);
		$editself = self::getAttribute('editself', $attribs, false, true);
		$listview = !empty(we_tag_listview::$current);

		switch($edit){
			case 'shop':
				if($delshop && (($foo = self::attributeMissing($attribs, 'shopname', __CLASS__)))){
					return $foo;
				}

				$amount = self::getAttribute('amount', $attribs, 1);
				$shopname = self::getAttribute('shopname', $attribs);
				$ifShopname = ($shopname ? ['shopname' => $shopname] : []);

				if($delshop){ //delete basket
					return array_merge(['deleteshop' => 1, 't' => time()], $ifShopname);
				}
				$customReq = [];
				if(we_tag_listview::$current){
					switch(true){
						case (we_tag_listview::$current instanceof we_listview_shopItems):
							$articleID = we_tag_listview::$current->getActItemField('id');
							$type = we_tag_listview::$current->getActItemField('type');
							$customReq = we_tag_listview::$current->getCustomFieldsAsRequest();
							break;
						case (isset(we_tag_listview::$current->Model) ? we_tag_listview::$current->Model instanceof we_objectFile : false):
						case (we_tag_listview::$current instanceof we_listview_objectBase):
							$articleID = we_tag_listview::$current->f(we_listview_base::ID_FIELD);
							$type = we_shop_shop::OBJECT;
							break;
						case (we_tag_listview::$current instanceof we_listview_document):
						default:
							$articleID = we_tag_listview::$current->f(we_listview_base::ID_FIELD);
							$type = we_shop_shop::DOCUMENT;
							break;
					}
				} else {
					$articleID = isset($GLOBALS['we_obj']->ID) ? $GLOBALS['we_obj']->ID : $GLOBALS['we_doc']->ID;
					$type = isset($GLOBALS['we_obj']->ID) ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT;
				}

				// is it a variant ????
				$variant = // normal variant on document
					($GLOBALS['we_doc']->Variant ? // normal listView or document
					[we_base_variants::REQUEST => $GLOBALS['we_doc']->Variant] :
					// variant inside shoplistview!
					(we_tag_listview::$current && we_tag_listview::$current->f(we_listview_base::VARIANT_FIELD) ?
					[we_base_variants::REQUEST => we_tag_listview::$current->f(we_listview_base::VARIANT_FIELD)] :
					[]));

				$trans = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction');

				//	preview mode in seem
				if($trans && !empty($_SESSION['weS']['we_data'][$trans][0]['ContentType']) && $_SESSION['weS']['we_data'][$trans][0]['ContentType'] === we_base_ContentTypes::OBJECT_FILE){
					$type = we_shop_shop::OBJECT;
				}

				if($delarticle){ // delete article
					return array_merge(['del_shop_artikelid' => $articleID, 'type' => $type, 't' => time()], $variant, $customReq, $ifShopname);
				} // increase/decrease amount of articles
				return array_merge(['shop_artikelid' => $articleID, 'shop_anzahl' => $amount, 'type' => $type, 't' => time()], $variant, $customReq, $ifShopname);

			case 'object':
				$oid = ($listview ?
					(we_tag_listview::$current && we_tag_listview::$current->f(we_listview_base::ID_FIELD) ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) : 0) :
					(!empty($GLOBALS['we_obj']) && isset($GLOBALS['we_obj']->ID) && $editself ? $GLOBALS['we_obj']->ID : 0));

				//FIXME: make sure only the selected object can be deleted - sth unique not user-known has to be added to prevent denial of service
				if($oid){
					return [($delete ? we_tag_delete::DEL_OBJ_PARAM : we_tag_write::EDIT_OBJ_PARAM) => $oid];
				}
				return ['edit_object' => 1];
			case 'document':
				$did = ($listview ?
					(we_tag_listview::$current && we_tag_listview::$current->f(we_listview_base::ID_FIELD) ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) : 0) :
					(isset($GLOBALS['we_doc']) && isset($GLOBALS['we_doc']->ID) && $editself ? $GLOBALS['we_doc']->ID : 0));

				//FIXME: make sure only the selected document can be deleted - sth unique not user-known has to be added to prevent denial of service
				if($did){
					return [($delete ? we_tag_delete::DEL_DOC_PARAM : we_tag_write::EDIT_DOC_PARAM) => $did];
				}
				return ['edit_document' => 1];
		}
		return [];
	}

}

function we_tag_a(array $attribs, $content){
	return we_tag_a::tag($attribs, $content);
}
