<?php

/**
 * webEdition CMS
 *
 * $Rev: 8821437b0bd8 $
 * $Author: Sascha Nemecek <nemecek@wienfluss.net> $
 * $Date: 2020-07-08 23:41 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_conditionAdd extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'field', __CLASS__))){
			return $foo;
		}

		// initialize possible Attributes
		$field = self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$value = strtr(self::getAttribute('value', $attribs, '', we_base_request::RAW), ['&gt;' => '>', '&lt;' => '<']);
		$compare = self::getAttribute('compare', $attribs, '=', we_base_request::RAW);
		$var = self::getAttribute('var', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$property = self::getAttribute('property', $attribs, false, we_base_request::BOOL);
		$exactmatch = self::getAttribute('exactmatch', $attribs, false, we_base_request::BOOL);
		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		// end initialize possible Attributes

		if(!we_tag_condition::$conditionName){//ignore if not in tag
			return '';
		}

		$regs = [];
		if($var && strtolower($compare) === 'like'){
			if(preg_match('/^(%)?([^%]+)(%)?$/', $var, $regs)){
				$var = $regs[2];
			}
		}
		$escape = true;
		switch(strtolower($type)){
			case 'now':
				$value = time();
				$escape = false;
				break;
			case 'today':
				$value = strtotime('today');
				$escape = false;
				break;
			case 'sessionfield':
				if($var && isset($_SESSION['webuser'][$var])){
					$value = $_SESSION['webuser'][$var];
				}
				break;
			case 'document':
				if($var){
					$doc = self::getDocForTag($docAttr, false);
					$value = ($property ? $doc->$var : $doc->getElement($var));
				}
				break;
			case 'request':
				if($var && isset($_REQUEST[$var])){
					$value = we_base_request::_(we_base_request::HTML, $var);
				}
				break;
			default:
				if($var && isset($GLOBALS[$var])){
					$value = $GLOBALS[$var];
				}
		}
		$collate = we_database_base::getCharsetCollation();
		if($exactmatch && $collate != ''){
			if(strpos($collate, 'latin1') !== false){
				$compare = 'COLLATE latin1_bin ' . $compare;
			} elseif(strpos($collate, 'utf') !== false){
				$compare = 'COLLATE utf8_bin ' . $compare;
			}
		}
		$value = (isset($regs[1]) ? $regs[1] : '') . $value . (isset($regs[3]) ? $regs[3] : '');

		if(strlen($field) && isset($GLOBALS[we_tag_condition::$conditionName])){
			$GLOBALS[we_tag_condition::$conditionName] .= '(' . $field . ' ' . $compare . ' ' . ($escape ? '"' : '') . $GLOBALS['DB_WE']->escape($value) . ($escape ? '"' : '') . ') ';
		} else {
			$GLOBALS[we_tag_condition::$conditionName] .= (preg_match('/^(.*)AND ?$/', $GLOBALS[we_tag_condition::$conditionName]) ? '1 ' : '0 ');
		}
		return '';
	}

}

function we_tag_conditionAdd(array $attribs){
	return we_tag_conditionAdd::tag($attribs);
}
