<?php

/**
 * webEdition CMS
 *
 * $Rev: f6962871c2fa $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-27 15:40 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_userInput extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$property = self::getAttribute('property', $attribs, false, we_base_request::BOOL);
		$format = self::getAttribute('format', $attribs, '', we_base_request::STRING);
		$checked = self::getAttribute('checked', $attribs, false, we_base_request::BOOL);
		$inlineedit = self::getAttribute('inlineedit', $attribs, true, we_base_request::BOOL);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$editable = self::getAttribute('editable', $attribs, true, we_base_request::BOOL);
		$hidden = self::getAttribute('hidden', $attribs, false, we_base_request::BOOL);
		$size = self::getAttribute('size', $attribs, '', we_base_request::UNIT);
		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
		$removeFirstParagraph = self::getAttribute('removefirstparagraph', $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);

		if($hidden && ($type != 'date')){
			$type = 'hidden';
		}

		$object = (we_tag_form::$name ?
			(isset($GLOBALS['we_object'][we_tag_form::$name]) ?
			$GLOBALS['we_object'][we_tag_form::$name] :
			(isset($GLOBALS['we_document'][we_tag_form::$name]) ?
			$GLOBALS['we_document'][we_tag_form::$name] :
			(isset($GLOBALS['we_doc']) ?
			$GLOBALS['we_doc'] :
			false))) :
			'');

		if($object){
			$tableID = '';
			if($object instanceof we_objectFile){
				$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
				$tableID = $object->TableID;
			}
			if($property){
				$isset = isset($object->{$name});
				$orgVal = $isset ? $object->{$name} : $value;
			} else {
				$isset = (!$object->ID && $object->getElement($name) === '' ?
					false :
					$object->issetElement($name));

				$orgVal = $isset ? $object->getElement($name) : $value;
			}

			$content = $object->getFieldByVal($orgVal, $type, $attribs, true, $object->ParentID, $object->Path, $tableID);
		} else {
			$orgVal = $value;
			$isset = false;
		}

		$fieldname = 'we_ui_' . (we_tag_form::$name ?: '') .
			($property ? ( '_' . $name) : ('[' . $name . ']') );

		if(!$editable && !$hidden){
			switch($type){
				case 'img':
				case 'binary':
				case 'flashmovie':
				case 'video':
				case 'audio':
					break;
				default:
					$hidden = getHtmlTag('input', [
						'type' => 'hidden',
						'name' => $fieldname,
						'value' => oldHtmlspecialchars($orgVal),
						'xml' => $xml
					]);
					return (($type != 'hidden') ? $content : '') . $hidden;
			}
		}
		switch($type){
			case 'img' :
				$imgDataId = we_base_request::_(we_base_request::HTML, we_fileupload::USERINPUT_TYPE_IMG . $name, md5(uniqid(__CLASS__, true)));

				if($editable){
					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}

					if(!isset($_SESSION[$imgDataId])){
						$_SESSION[$imgDataId] = [];
					}
					$_SESSION[$imgDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
					//$_SESSION[$imgDataId]['maxfilesize'] = sefl::getAttribute('maxfilesize',$attribs);
					$_SESSION[$imgDataId]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
					$_SESSION[$imgDataId]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
					$_SESSION[$imgDataId]['quality'] = self::getAttribute('quality', $attribs, 8, we_base_request::INT);
					$_SESSION[$imgDataId]['keepratio'] = self::getAttribute('keepratio', $attribs, true, we_base_request::BOOL);
					$_SESSION[$imgDataId]['maximize'] = self::getAttribute('maximize', $attribs, false, we_base_request::BOOL);
					$_SESSION[$imgDataId]['id'] = $orgVal ?: '';

					$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
					$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
					$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
					$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
					$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
					$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::STRING);

					if($_SESSION[$imgDataId]['id']){
						$attribs['id'] = $_SESSION[$imgDataId]['id'];
					}

					if(isset($_SESSION[$imgDataId]['serverPath']) && strpos($_SESSION[$imgDataId]['serverPath'], TEMP_PATH) === false){
						$src = '/' . ltrim(substr($_SESSION[$imgDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

						$imgTag = getHtmlTag('img', [
							'src' => $src,
							'alt' => " ",
							'width' => $_SESSION[$imgDataId]['imgwidth'],
							'height' >= $_SESSION[$imgDataId]['imgheight'],
						]);
					} else {
						unset($attribs['width'], $attribs['height']);
						$imgTag = $GLOBALS['we_doc']->getField($attribs, 'img');
					}

					$checked = (!empty($_SESSION[$imgDataId]['doDelete'])) ? ' checked' : '';
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_userinput weEditTable" style="border-color: ' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $imgTag . '
			<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_IMG . $name . '" value="' . $imgDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="' . implode(',', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::IMAGE)) . '"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}

				$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_IMG . $name . '" value="' . $imgDataId . '" />';

				if(isset($_SESSION[$imgDataId]['serverPath'])){
					$src = '/' . ltrim(substr($_SESSION[$imgDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

					return getHtmlTag('img', ['src' => $src,
							'alt' => '',
							'style' => 'width:' . $_SESSION[$imgDataId]["imgwidth"] . 'px;height:' . $_SESSION[$imgDataId]["imgheight"] . 'px',
						]) . $hidden;
				}

				if(!empty($_SESSION[$imgDataId]['doDelete'])){
					return $hidden;
				}

				if((!empty($_SESSION[$imgDataId]['id'])) || (isset($orgVal) && $orgVal)){//Fix #9835
					unset($attribs['width'], $attribs['height']);
					$attribs['id'] = $_SESSION[$imgDataId]['id'] ? $_SESSION[$imgDataId]['id'] : $orgVal;
					return $GLOBALS['we_doc']->getField($attribs, 'img') . $hidden;
				}

				return '';
			case 'video' :
				$videoDataId = we_base_request::_(we_base_request::HTML, '' . we_fileupload::USERINPUT_TYPE_VIDEO . $name, md5(uniqid(__CLASS__, true)));

				if($editable){
					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}

					if(!isset($_SESSION[$videoDataId])){
						$_SESSION[$videoDataId] = [];
					}
					$_SESSION[$videoDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
					$_SESSION[$videoDataId]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
					$_SESSION[$videoDataId]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
					$_SESSION[$videoDataId]['id'] = $orgVal ?: '';

					$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
					$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
					$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
					$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
					$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
					$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

					if($_SESSION[$videoDataId]['id']){
						$attribs['id'] = $_SESSION[$videoDataId]['id'];
					}

					if(isset($_SESSION[$videoDataId]['serverPath'])){
						$src = '/' . ltrim(substr($_SESSION[$videoDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
						$videoTag = '';
					} else {
						$attribs = removeAttribs($attribs, ['width', 'height']);
						$videoTag = (!empty($attribs['id']) ?
							$GLOBALS['we_doc']->getField($attribs, 'video') : '');
					}

					$checked = (!empty($_SESSION[$videoDataId]['doDelete']) ? ' checked' : '');
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $videoTag . '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_VIDEO . $name . '" value="' . $videoDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="video/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_VIDEO . $name . '" value="' . $videoDataId . '" />';

				if(isset($_SESSION[$videoDataId]['serverPath'])){
					$src = '/' . ltrim(substr($_SESSION[$videoDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

					return $hidden;
				}
				if(!empty($_SESSION[$videoDataId]['id'])){

					if(!empty($_SESSION[$videoDataId]['doDelete'])){
						return $hidden;
					}
					$attribs = removeAttribs($attribs, ['width', 'height']);
					$attribs['id'] = $_SESSION[$videoDataId]['id'];
					return $GLOBALS['we_doc']->getField($attribs, 'video') . $hidden;
				}

				return '';
			case 'audio' :
				$audioDataId = we_base_request::_(we_base_request::HTML, '' . we_fileupload::USERINPUT_TYPE_AUDIO . $name, md5(uniqid(__CLASS__, true)));

				if($editable){
					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}

					if(!isset($_SESSION[$audioDataId])){
						$_SESSION[$audioDataId] = [];
					}
					$_SESSION[$audioDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
					$_SESSION[$audioDataId]['id'] = $orgVal ?: '';

					$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
					$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
					$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
					$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
					$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
					$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

					if($_SESSION[$audioDataId]['id']){
						$attribs['id'] = $_SESSION[$audioDataId]['id'];
					}

					if(isset($_SESSION[$audioDataId]['serverPath'])){
						$src = '/' . ltrim(substr($_SESSION[$audioDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
						$audioTag = '';
					} else {
						$audioTag = (!empty($attribs['id']) ? $GLOBALS['we_doc']->getField($attribs, 'audio') : '');
					}

					$checked = (!empty($_SESSION[$audioDataId]['doDelete']) ? ' checked' : '');
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $audioTag . '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_AUDIO . $name . '" value="' . $audioDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="audio/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_AUDIO . $name . '" value="' . $audioDataId . '" />';

				if(isset($_SESSION[$audioDataId]['serverPath'])){
					$src = '/' . ltrim(substr($_SESSION[$audioDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

					return $hidden;
				}
				if(!empty($_SESSION[$audioDataId]['id'])){

					if(!empty($_SESSION[$audioDataId]['doDelete'])){
						return $hidden;
					}
					$attribs['id'] = $_SESSION[$audioDataId]['id'];
					return $GLOBALS['we_doc']->getField($attribs, 'audio') . $hidden;
				}

				return '';
			case 'flashmovie' :
				$flashmovieDataId = we_base_request::_(we_base_request::HTML, '' . we_fileupload::USERINPUT_TYPE_FLASHMOVIE . $name, md5(uniqid(__CLASS__, true)));

				if($editable){
					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}

					if(!isset($_SESSION[$flashmovieDataId])){
						$_SESSION[$flashmovieDataId] = [];
					}
					$_SESSION[$flashmovieDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
					//$_SESSION[$imgDataId]['maxfilesize'] = self::getAttribute('maxfilesize',$attribs);
					$_SESSION[$flashmovieDataId]['width'] = self::getAttribute('width', $attribs, 0, we_base_request::INT);
					$_SESSION[$flashmovieDataId]['height'] = self::getAttribute('height', $attribs, 0, we_base_request::INT);
					$_SESSION[$flashmovieDataId]['id'] = $orgVal ?: '';

					$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
					$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
					$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
					$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
					$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
					$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

					if($_SESSION[$flashmovieDataId]['id']){
						$attribs['id'] = $_SESSION[$flashmovieDataId]['id'];
					}

					if(isset($_SESSION[$flashmovieDataId]['serverPath'])){
						$src = '/' . ltrim(substr($_SESSION[$flashmovieDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
						$flashmovieTag = '';
					} else {
						$attribs = removeAttribs($attribs, ['width', 'height']);
						// Include Flash class
						$flashmovieTag = (!empty($attribs['id']) ?
							$GLOBALS['we_doc']->getField($attribs, 'flashmovie') :
							'<img src="' . ICON_DIR . 'no_flashmovie.gif" alt="" width="64" height="64" />');
					}

					$checked = (!empty($_SESSION[$flashmovieDataId]['doDelete']) ? ' checked' : '');
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $flashmovieTag . '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_FLASHMOVIE . $name . '" value="' . $flashmovieDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="application/x-shockwave-flash"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_FLASHMOVIE . $name . '" value="' . $flashmovieDataId . '" />';

				if(isset($_SESSION[$flashmovieDataId]['serverPath'])){
					$src = '/' . ltrim(substr($_SESSION[$flashmovieDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

					return $hidden;
				}
				if(!empty($_SESSION[$flashmovieDataId]['id'])){

					if(!empty($_SESSION[$flashmovieDataId]['doDelete'])){
						return $hidden;
					}
					$attribs = removeAttribs($attribs, ['width', 'height']);
					$attribs['id'] = $_SESSION[$flashmovieDataId]['id'];
					return $GLOBALS['we_doc']->getField($attribs, 'flashmovie') . $hidden;
				}

				return '';
			case 'binary' :
				$binaryDataId = we_base_request::_(we_base_request::HTML, we_fileupload::USERINPUT_TYPE_BINARY . $name, md5(uniqid(__CLASS__, true)));

				if($editable){
					if(($foo = self::attributeMissing($attribs, 'parentid', __CLASS__))){
						return $foo;
					}

					if(!isset($_SESSION[$binaryDataId])){
						$_SESSION[$binaryDataId] = [];
					}
					$_SESSION[$binaryDataId]['parentid'] = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
					//$_SESSION[$binaryDataId]['maxfilesize'] = self::getAttribute('maxfilesize',$attribs);

					$_SESSION[$binaryDataId]['id'] = $orgVal ?: '';

					$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
					$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
					$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
					$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
					$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
					$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

					if($_SESSION[$binaryDataId]['id']){
						$attribs['id'] = $_SESSION[$binaryDataId]['id'];
					}

					if(isset($_SESSION[$binaryDataId]['serverPath']) && strpos($_SESSION[$binaryDataId]['serverPath'], TEMP_PATH) === false){
						$src = '/' . ltrim(substr($_SESSION[$binaryDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
						$imgTag = '<a href="' . $src . '" target="_blank">' . $src . '</a>';
					} else {
						$binaryTag = $GLOBALS['we_doc']->getField($attribs, 'binary');
						$t = explode('_', $binaryTag[0]);
						unset($t[1], $t[0]);
						$fn = implode('_', $t);
						$imgTag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
					}

					$checked = (!empty($_SESSION[$binaryDataId]['doDelete']) ? ' checked' : '');
					$inputstyle = ($size ? 'width:' . $size . 'em;' : '') . $inputstyle;
					return '<table class="we we_userinput weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $imgTag . '
			<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_BINARY . $name . '" value="' . $binaryDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="application/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" name="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="' . we_fileupload::USERINPUT_TYPE_DEL . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				if(!isset($_SESSION[$binaryDataId])){
					$_SESSION[$binaryDataId] = [];
				}
				$_SESSION[$binaryDataId]['id'] = $orgVal ?: '';
				if($_SESSION[$binaryDataId]['id']){
					$attribs['id'] = $_SESSION[$binaryDataId]['id'];
				}
				$hidden = '<input type="hidden" name="' . we_fileupload::USERINPUT_TYPE_BINARY . $name . '" value="' . $binaryDataId . '" />';

				if(isset($_SESSION[$binaryDataId]["serverPath"])){
					$src = '/' . ltrim(substr($_SESSION[$binaryDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
					return $hidden;
				}
				if(empty($_SESSION[$binaryDataId]['id'])){
					return '';
				}
				if(!empty($_SESSION[$binaryDataId]['doDelete'])){
					return $hidden;
				}

				$attribs['id'] = $_SESSION[$binaryDataId]['id'];
				$binaryTag = $GLOBALS['we_doc']->getField($attribs, 'binary');
				$t = explode('_', $binaryTag[0]);
				unset($t[1], $t[0]);
				$fn = implode('_', $t);
				$imgTag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
				return we_html_element::span(['class' => 'we we_userinput'], $imgTag . $hidden);
			case 'textarea' :
				if(!self::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL)){
					$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr',
						'values', 'hidden', 'editable', 'format', 'property', 'size', 'maxlength', 'fontnames',
						'bgcolor', 'editorcss', 'ignoredocumentcss', 'buttonpos', '_name_orig', 'hideautobr'
					]);
					$atts['name'] = $fieldname;
					$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';

					return getHtmlTag('textarea', $atts, oldHtmlspecialchars(($content ?: $value)), true);
				}

				$charset = self::getAttribute('charset', $attribs, 'iso-8859-1', we_base_request::STRING);

				return (!$inlineedit ? STYLESHEET_MINIMAL : '') .
					we_html_forms::weTextarea($fieldname, ($content ?: $value), $attribs, false, '', false, false, $xml, $removeFirstParagraph, $charset, true, $name);
			case 'checkbox' :
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
					'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);
				if((!$isset) && $checked){
					$content = 1;
				}
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return self::inputCheckboxField($fieldname, $content, $atts);
			case 'date' :
				$orgVal = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL) ? time() : ($orgVal ?: 0);
				if($hidden){
					$attsHidden = ['type' => 'hidden',
						'name' => $fieldname,
						'value' => $orgVal ?: time(),
						'xml' => $xml
					];
					return getHtmlTag('input', $attsHidden);
				}
				$year = date('Y');
				$minyear = self::getAttribute('minyear', $attribs, 0, we_base_request::INT);
				switch($minyear ? $minyear[0] : ''){
					case '+':
						$minyear = $year + intval(substr($minyear, 1));
						break;
					case '-':
						$minyear = $year - intval(substr($minyear, 1));
						break;
					default:
						$minyear = intval($minyear);
						break;
				}
				$maxyear = self::getAttribute('maxyear', $attribs, 0, we_base_request::INT);
				switch($maxyear ? $maxyear[0] : ''){
					case '+':
						$maxyear = $year + intval(substr($maxyear, 1));
						break;
					case '-':
						$maxyear = $year - intval(substr($maxyear, 1));
						break;
					default:
						$maxyear = intval($maxyear);
						break;
				}
				return we_html_element::span(['class' => 'we we_userinput'], we_html_tools::getDateInput('we_ui_' . (we_tag_form::$name ?: '') . '[we_date_' . $name . ']', $orgVal, false, $format, '', '', $xml, $minyear, $maxyear));

			case 'country':
				$newAtts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
					'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);

				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = ($lang ?
					substr($lang, 0, 2) :
					array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));


				if(WE_COUNTRIES_TOP){
					$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
					foreach($topCountries as $countrykey => &$countryvalue){
						$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
					}
					unset($countryvalue);
				} else {
					$topCountries = [];
				}
				if(WE_COUNTRIES_SHOWN){
					$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
					foreach($shownCountries as $countrykey => &$countryvalue){
						$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
					}
					unset($countryvalue);
				} else {
					$shownCountries = [];
				}
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);

				$options = '';
				if(WE_COUNTRIES_DEFAULT != ''){
					$options .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
				}
				foreach($topCountries as $countrykey => &$countryvalue){
					$options .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
				}
				if(!empty($topCountries) && !empty($shownCountries)){
					$options .= '<option value="-" disabled="disabled">----</option>';
				}
				foreach($shownCountries as $countrykey2 => &$countryvalue2){
					$options .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue2) . '</option>';
				}
				unset($countryvalue, $countryvalue2);
				$newAtts['size'] = (isset($atts['size']) ? $atts['size'] : 1);
				$newAtts['name'] = $fieldname;
				$newAtts['class'] = (isset($newAtts['class']) ? $newAtts['class'] . ' ' : '') . 'we we_userinput';
				return getHtmlTag('select', $newAtts, $options, true);
			case 'language':
				$newAtts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
					'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);

				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = ($lang ?
					substr($lang, 0, 2) :
					array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) );

				$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
				foreach($frontendL as &$lcvalue){
					$lccode = explode('_', $lcvalue);
					$lcvalue = $lccode[0];
				}
				$frontendLL = [];
				foreach($frontendL as &$lcvalue){
					$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
				}

				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($frontendLL, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);
				$options = '';
				foreach($frontendLL as $langkey => &$langvalue){
					$options .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>';
				}
				unset($langvalue);
				$newAtts['size'] = (isset($atts['size']) ? $atts['size'] : 1);
				$newAtts['name'] = $fieldname;
				$newAtts['class'] = (isset($newAtts['class']) ? $newAtts['class'] . ' ' : '') . 'we we_userinput';
				return getHtmlTag('select', $newAtts, $options, true);
			case 'select' :
				$options = '';
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
					'hidden', 'editable', 'format', 'property', 'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength'
				]);
				if($values){
					foreach($values as $txt){
						$options .= getHtmlTag('option', ($txt == $orgVal ? ['selected' => 'selected'] : []), trim($txt), true);
					}
				} elseif($object && !empty($object->DefArray['meta_' . $name])){
					foreach($object->DefArray['meta_' . $name]['meta'] as $key => $val){
						$atts2 = ($orgVal == $key ? ['value' => $key, 'selected' => 'selected'] : ['value' => $key]);
						$attsOption = removeAttribs(array_merge($atts, $atts2), ['class']);
						$options .= getHtmlTag('option', $attsOption, $val, true);
					}
				}
				$atts['size'] = (empty($atts['size']) ? 1 : $atts['size']);
				$atts['name'] = $fieldname;
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return getHtmlTag('select', $atts, $options, true);
			case 'radio' :
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden',
					'editable', 'format', 'property', 'rows', 'cols', 'width', 'height', 'bgcolor', 'fontnames'
				]);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return (!$isset ?
						self::inputRadioField($fieldname, ($checked ? $value : $value . 'dummy'), $value, $atts) :
						self::inputRadioField($fieldname, $orgVal, $value, $atts));

			case 'hidden':
				return getHtmlTag('input', ['type' => 'hidden', 'name' => $fieldname, 'value' => oldHtmlspecialchars(self::getAttribute('value', $attribs, $content, we_base_request::STRING)),
					'xml' => $xml,]);
			case 'print' :
				return $orgVal;
			case 'choice' :
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
					'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames', 'maxlength'
				]);
				$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);
				return we_html_element::span(['class' => 'we we_userinput'], we_html_tools::inputChoiceField($fieldname, $orgVal, $values, $atts, $mode));
			case 'password':
				$atts = array_merge(removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format',
					'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']), ['type' => 'password', 'name' => $fieldname, 'value' => oldHtmlspecialchars($orgVal)]
				);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';

				return getHtmlTag('input', $atts);
			case 'textinput':
			default:
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
					'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);
				$atts['class'] = (isset($atts['class']) ? $atts['class'] . ' ' : '') . 'we we_userinput';
				return self::inputTextInputField($fieldname, $orgVal, $atts);
		}
	}

}

function we_tag_userInput(array $attribs, $content){
	return we_tag_userInput::tag($attribs, $content);
}
