<?php

/**
 * webEdition CMS
 *
 * $Rev: 5c097ca0d7b5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 23:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_write extends we_weTag_tag{
	const EDIT_DOC_PARAM = 'we_editDocument_ID';
	const EDIT_OBJ_PARAM = 'we_editObject_ID';
	const OK = 0;
	const ERR_INTERN = 1;
	const ERR_NOT_ALLOWED_EDIT = 4;
	const ERR_PROTECTION = 5;
	const ERR_DUPLICATE_NAME = 8;
	const ERR_FIELDS_NOT_VALID = 9;
	const ERR_WRITE = 7;

	public static $status = [
		'object' => [],
		'document' => [],
	];

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);

		switch($type){
			case 'object':
				if(($foo = self::attributeMissing($attribs, 'classid', __CLASS__))){
					return $foo;
				}
				break;
			default:
				$type = 'document'; //make sure type is known!
				if(($foo = self::attributeMissing($attribs, 'doctype', __CLASS__))){
					return $foo;
				}
				break;
		}

		$name = self::getAttribute('formname', $attribs, (we_tag_form::$name ?: we_base_constants::DEFAULT_FORM), we_base_request::STRING);
		$publish = self::getAttribute('publish', $attribs, false, we_base_request::BOOL);
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$charset = self::getAttribute('charset', $attribs, 'iso-8859-1', we_base_request::STRING);
		$categories = self::getAttribute('categories', $attribs, '', we_base_request::STRING);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$userid = self::getAttribute('userid', $attribs, '', we_base_request::STRING); // deprecated  use protected=true instead
		$protected = self::getAttribute('protected', $attribs, false, we_base_request::BOOL);
		$admin = self::getAttribute('admin', $attribs, '', we_base_request::STRING);
		$mail = self::getAttribute('mail', $attribs, '', we_base_request::STRING); //FIXME: email_list
		$mailfrom = self::getAttribute('mailfrom', $attribs, '', we_base_request::EMAIL);
		$forceedit = self::getAttribute('forceedit', $attribs, false, we_base_request::BOOL) && !empty($_SESSION['webuser']['registered']);
		$workspaces = self::getAttribute('workspaces', $attribs, [], we_base_request::INTLISTA);
		$objname = preg_replace('/[^a-z0-9_-]/i', '', self::getAttribute('name', $attribs, '', we_base_request::STRING));
		$onduplicate = ($objname === '' ? 'overwrite' : self::getAttribute('onduplicate', $attribs, 'increment', we_base_request::STRING));
		$onpredefinedname = self::getAttribute('onpredefinedname', $attribs, 'appendto', we_base_request::STRING);
		$workflowname = self::getAttribute('workflowname', $attribs, '', we_base_request::STRING);
		$workflowuserid = self::getAttribute('workflowuserid', $attribs, 0, we_base_request::INT);
		$doworkflow = ($workflowname != '' && $workflowuserid != 0);
		$searchable = self::getAttribute('searchable', $attribs, true, we_base_request::BOOL);
		$language = self::getAttribute('language', $attribs, '', we_base_request::STRING);

		//init with ok
		if(!we_base_request::_(we_base_request::BOOL, 'edit_' . $type)){
			self::$status[$type][$name] = 9;
			return '';
		}

		switch($type){
			case 'document':
				$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
				$doctype = self::getAttribute('doctype', $attribs, '', we_base_request::STRING);
				$id = we_base_request::_(we_base_request::INT, self::EDIT_DOC_PARAM, 0);
				$ok = we_webEditionDocument::initDocument($name, $tid, $doctype, $categories, $id, true);
				break;
			case 'object':
				$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
				$id = we_base_request::_(we_base_request::INT, self::EDIT_OBJ_PARAM, 0);

				if($classid && f('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid))){
					if(!$id || f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $id . ' AND TableID=' . intval($classid))){
						$ok = we_objectFile::initObject(intval($classid), $name, $categories, intval($parentid), $id, true);
					} else {
						self::$status['object'][$name] = self::ERR_INTERN;
						t_e('Object ' . $id . ' is no element of class ' . intval($classid) . '!');
						return '';
					}
				} else {
					self::$status['object'][$name] = self::ERR_INTERN;
					t_e('Table ' . intval($classid) . ' does not exist!');
					return '';
				}
				break;
		}

		if(!$ok){
			self::$status['object'][$name] = self::ERR_INTERN;
			return '';
		}
		$isOwner = !empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser']['ID']) && (
			($protected && ($_SESSION['webuser']['ID'] == $GLOBALS['we_' . $type][$name]->WebUserID)) ||
			($userid && ($_SESSION['webuser']['ID'] == $GLOBALS['we_' . $type][$name]->getElement($userid)))
			);

		$isAdmin = !empty($_SESSION['webuser']['registered']) && $admin && !empty($_SESSION['webuser'][$admin]);

		$isNew = (($GLOBALS['we_' . $type][$name]->ID == 0) ? ($admin/* only if this field is used */ ? $isAdmin : true) : false); //FR #8411

		if(!($isAdmin || $isNew || $isOwner || $forceedit)){
			self::$status[$type][$name] = self::ERR_NOT_ALLOWED_EDIT;
			return '';
		}

		$doWrite = true;
		//$newObject = ($GLOBALS['we_'.$type][$name]->ID) ? false : true;
		if($protected){
			if(!isset($_SESSION['webuser']['ID']) || !isset($_SESSION['webuser']['registered']) || !$_SESSION['webuser']['registered']){
				self::$status[$type][$name] = self::ERR_PROTECTION;
				return '';
			}
			if(!$GLOBALS['we_' . $type][$name]->WebUserID){
				$GLOBALS['we_' . $type][$name]->WebUserID = $_SESSION['webuser']['ID'];
			}
		} elseif($userid){
			if(!isset($_SESSION['webuser']['ID']) || !isset($_SESSION['webuser']['registered']) || !$_SESSION['webuser']['registered']){
				self::$status[$type][$name] = self::ERR_PROTECTION;
				return '';
			}
			if(!$GLOBALS['we_' . $type][$name]->getElement($userid)){
				$GLOBALS['we_' . $type][$name]->setElement($userid, $_SESSION['webuser']['ID']);
			}
		}
		self::$status[$type][$name] = self::OK;
		self::checkAndCreateBinary($name, ($type === 'document' ? 'we_document' : 'we_object'));

		//FIXME: we should probably use checkFieldsOnSave?!
		$GLOBALS['we_' . $type][$name]->checkPathDiffAndCreate();
		if(!$objname){
			$GLOBALS['we_' . $type][$name]->correctDoublePath();
		}
		if(isset($GLOBALS['we_doc'])){
			$_WE_DOC_SAVE = $GLOBALS['we_doc'];
		}
		$GLOBALS['we_doc'] = &$GLOBALS['we_' . $type][$name];
		$GLOBALS['we_doc']->IsSearchable = $searchable;
		switch($language){
			case 'self':
			case 'top':
				$docLanguage = self::getDocForTag($language);
				$language = $docLanguage->Language;
				unset($docLanguage);
		}
		if($language){
			$GLOBALS['we_doc']->Language = $language;
		} else {
			//preset lang
			$GLOBALS['we_doc']->initLanguageFromParent();
		}
		if($workspaces && $type === 'object'){
			$GLOBALS['we_object'][$name]->Workspaces = implode(',', $workspaces);
		}

		$GLOBALS['we_' . $type][$name]->Path = $GLOBALS['we_' . $type][$name]->getPath();

		if(we_modules_info::isActive(we_modules_info::OBJECT) && $type === 'object'){
			$objname = self::getObjName($objname, 'object', $name, $onpredefinedname);
			$doWrite = self::correctObjName($objname, 'object', $name, $onduplicate);
		}

		if($doWrite){
			$ret = $GLOBALS['we_' . $type][$name]->we_save();
			if($ret && $publish && !$doworkflow){
				$ret = ($type === 'document' && (!$GLOBALS['we_' . $type][$name]->IsDynamic) && isset($GLOBALS['we_doc']) ? // on static HTML Documents we have to do it different
					$GLOBALS['we_doc']->we_publish() :
					$GLOBALS['we_' . $type][$name]->we_publish());
			}

			if($ret){
				if($doworkflow){
					$wf_text = $workflowname . '  ';
					switch($type){
						default:
						case 'document':
							$wf_text .= 'Document ID: ' . $GLOBALS['we_doc']->ID;
							$tab = FILE_TABLE;
							break;
						case 'object':
							$wf_text .= 'Object ID: ' . $GLOBALS['we_doc']->ID;
							$tab = OBJECT_FILES_TABLE;
							break;
					}
					$workflowID = we_workflow_utility::getWorkflowID($workflowname, $tab);

					if(!we_workflow_utility::insertDocInWorkflow($GLOBALS['we_doc']->ID, $GLOBALS['we_doc']->Table, $workflowID, $workflowuserid, $wf_text, $GLOBALS['DB_WE'])){
						t_e('error inserting document to workflow. Additional data:', $GLOBALS['we_doc']->Table, $workflowID, $workflowuserid);
					}
				}
				$GLOBALS['we_object_write_ID'] = $GLOBALS['we_doc']->ID;

				/**
				 * Fix #9818
				 * now we have to set the new document/object ID as request value to avoid
				 * createing more documents/objects by reload the webform (<we:form type="object | document">) by an user
				 */
				$requestVarName = 'we_edit' . ucfirst($type) . '_ID';
				$_REQUEST[$requestVarName] = $GLOBALS['we_doc']->ID;
			} else {
				//save or publish failed
				$doWrite = false;
				self::$status[$type][$name] = self::ERR_WRITE;
			}
		}

		unset($GLOBALS['we_doc']);
		if(isset($_WE_DOC_SAVE)){
			$GLOBALS['we_doc'] = $_WE_DOC_SAVE;
			unset($_WE_DOC_SAVE);
		}
		$_REQUEST['we_returnpage'] = $GLOBALS['we_' . $type][$name]->getElement('we_returnpage');

		if($doWrite && $mail){
			if(!$mailfrom){
				$mailfrom = 'dontReply@' . $_SERVER['SERVER_NAME'];
			}
			$path = $GLOBALS['we_' . $type][$name]->Path;
			switch($type){
				case 'object':
					$classname = f('SELECT Text FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid));
					$mailtext = sprintf(g_l('global', '[std_mailtext_newObj]'), $path, $classname) . "\n" .
						($triggerid ? id_to_path($triggerid) . '?' . WEOBJ_PARAM . '=' : 'ObjectID: ') .
						$GLOBALS['we_object'][$name]->ID;
					$subject = g_l('global', '[std_subject_newObj]');
					break;
				default:
				case 'document':
					$mailtext = sprintf(g_l('global', '[std_mailtext_newDoc]'), $path) . "\n" . $GLOBALS['we_' . $type][$name]->getHttpPath();
					$subject = g_l('global', '[std_subject_newDoc]');
					break;
			}
			$phpmail = new we_mail_mail($charset, $mail, $subject, $mailfrom);
			$phpmail->addTextPart($mailtext);
			$phpmail->buildMessage();
			$phpmail->send();
		}

		if(we_tag_sessionStart::$started){
			unset($_SESSION['weS']['we_' . $type . '_session_' . $name]); //fix #8051
		}
		return '';
	}

	private static function getObjName($objname, $type, $name, $onpredefinedname){
		if($GLOBALS['we_' . $type][$name]->Text === ''){
			return $objname ?: (1 + intval(f('SELECT MAX(ID) AS ID FROM ' . OBJECT_FILES_TABLE)));
		}
		switch($onpredefinedname){
			case 'appendto':
				return ($objname ? $GLOBALS['we_' . $type][$name]->Text . '_' . $objname : $GLOBALS['we_' . $type][$name]->Text);
			case 'infrontof':
				return $objname . ($objname ? '_' . $GLOBALS['we_' . $type][$name]->Text : $GLOBALS['we_' . $type][$name]->Text);
			case 'overwrite':
				return $objname ?: $GLOBALS['we_' . $type][$name]->Text;
			default:
				return $objname;
		}
	}

	protected static function checkValidObjFields(we_objectFile $obj, $name){
		$fieldsValid = $obj->checkFieldsOnSave();
		if(!empty($fieldsValid)){
			self::$status['object'][$name] = self::ERR_FIELDS_NOT_VALID;
			return false;
		}
		return true;
	}

	private static function correctObjName($objname, $type, $name, $onduplicate){
		$objexists = f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $GLOBALS['we_' . $type][$name]->ParentID . ' AND Text="' . $GLOBALS['DB_WE']->escape($objname) . '"');
		if(!$objexists){
			$GLOBALS['we_' . $type][$name]->Text = $objname;
			$GLOBALS['we_' . $type][$name]->Path = str_replace('//', '/', $GLOBALS['we_' . $type][$name]->Path . '/' . $objname);
			return self::checkValidObjFields($GLOBALS['we_' . $type][$name], $name);
		}
		switch($onduplicate){
			default:
			case 'abort':
				self::$status[$type][$name] = self::ERR_DUPLICATE_NAME;
				return false;
			case 'overwrite':
				$GLOBALS['we_' . $type][$name]->ID = $objexists;
				$GLOBALS['we_' . $type][$name]->Path = str_replace('//', '/', $GLOBALS['we_' . $type][$name]->Path . '/' . $objname);
				$GLOBALS['we_' . $type][$name]->Text = $objname;
				return self::checkValidObjFields($GLOBALS['we_' . $type][$name], $name);
			case 'increment':
				$z = 1;
				$footext = $objname . '_' . $z;
				while(f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $GLOBALS['we_' . $type][$name]->ParentID . ' AND Text="' . $GLOBALS['DB_WE']->escape($footext) . '"')){
					$z++;
					$footext = $objname . '_' . $z;
				}
				$GLOBALS['we_' . $type][$name]->Path = str_replace('//', '/', $GLOBALS['we_' . $type][$name]->Path . '/' . $footext);
				$GLOBALS['we_' . $type][$name]->Text = $footext;
				return self::checkValidObjFields($GLOBALS['we_' . $type][$name], $name);
		}
	}

	private static function checkAndCreateBinary($formname, $type = 'we_document'){
		$webuserId = !empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID']) ? $_SESSION['webuser']['ID'] : 0;
		$regs = [];

		$checks = [
			we_fileupload::USERINPUT_TYPE_BINARY => [
				'application/',
				we_base_ContentTypes::APPLICATION,
				'application',
				[]
			],
			we_fileupload::USERINPUT_TYPE_FLASHMOVIE => [
				we_base_ContentTypes::FLASH,
				we_base_ContentTypes::FLASH,
				'image',
				['width' => 'imgwidth',
					'height' => 'imgheight',
					'origwidth' => 'imgwidth',
					'origheight' => 'imgheight'
				]
			],
			we_fileupload::USERINPUT_TYPE_VIDEO => [
				'video/',
				we_base_ContentTypes::VIDEO,
				'video',
				['width' => 'width',
					'height' => 'height'
				]
			],
			we_fileupload::USERINPUT_TYPE_AUDIO => [
				'audio/',
				we_base_ContentTypes::AUDIO,
				'audio',
				[]
			],
			we_fileupload::USERINPUT_TYPE_IMG => [
				'image/',
				we_base_ContentTypes::IMAGE,
				'image',
				['width' => 'imgwidth',
					'height' => 'imgheight',
					'origwidth' => 'imgwidth',
					'origheight' => 'imgheight'
				]
			],
		];
		foreach($_REQUEST as $key => $dataID){
			$contentType = '';
			foreach($checks as $check => $checkData){
				if(preg_match('|^' . $check . '(.*)$|', $key, $regs)){
					list($matchType, $contentType, $dataType, $moreAttribs) = $checkData;
					break;
				}
			}
			if(!$contentType || !isset($_SESSION[$dataID])){
				continue;
			}
			$name = $regs[1];
			$id = empty($_SESSION[$dataID]['id']) ? 0 : intval($_SESSION[$dataID]['id']);
			$contentType = !empty($_SESSION[$dataID]['type']) && $_SESSION[$dataID]['type'] === we_base_ContentTypes::PDF ? we_base_ContentTypes::PDF : $contentType;

			if(!empty($_SESSION[$dataID]['doDelete'])){
				if($id){
					$document = we_base_ContentTypes::getObject($contentType);
					$document->initByID($id);
					if($document->WebUserID == $webuserId){
						//everything ok, now delete
						we_base_delete::deleteEntry($id, FILE_TABLE);
						$GLOBALS[$type][$formname]->setElement($name, 0);
					}
				}
			} elseif(isset($_SESSION[$dataID]['serverPath'])){
				if(strpos($_SESSION[$dataID]['type'], $matchType) === 0){
					$document = we_base_ContentTypes::getObject($contentType);
					if($id){
						// document has already an image
						// so change binary data
						$document->initByID($id);
					} else {
						$document->setParentID($_SESSION[$dataID]['parentid']);
					}

					$document->Table = FILE_TABLE;
					$document->ContentType = $contentType; // needed to switch between otherDocument and PDF
					$document->Published = time();
					$document->WebUserID = $webuserId;
					$document->Filename = $_SESSION[$dataID]['fileName'];
					$document->Extension = $_SESSION[$dataID]['extension'];
					$document->Text = $_SESSION[$dataID]['text'];
					$document->Path = rtrim($document->getParentPath(), '/') . '/' . $document->Text;
					$document->setElement('data', $_SESSION[$dataID]['serverPath'], $dataType);
					$document->setElement('filesize', $_SESSION[$dataID]['size'], 'attrib', we_contents_base::ETYPE_NUMBER);
					foreach($moreAttribs as $eName => $sKey){
						$document->setElement($eName, $_SESSION[$dataID][$sKey], 'attrib', we_contents_base::ETYPE_NUMBER);
					}

					$document->we_save();

					if(!empty($_SESSION[$dataID]['unique'])){
						$document->Filename = str_replace($_SESSION[$dataID]['unique'], $document->ID, $document->Filename);
					} else {
						$t = explode('_', $document->Filename);
						$t[1] = $document->ID;
						$document->Filename = implode('_', $t);
					}
					$document->Text = $document->Filename . $document->Extension;

					$document->Path = rtrim($document->getParentPath(), '/') . '/' . $document->Filename . $document->Extension;
					$document->we_save();

					$GLOBALS[$type][$formname]->setElement($name, $document->ID);
				}
			}
			unset($_SESSION[$dataID]);
		}
	}

}

function we_tag_write(array $attribs){
	return we_tag_write::tag($attribs);
}
